/**********************************************************************
Copyright (c) 2004 Andy Jefferson and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
    ...
**********************************************************************/
package org.jpox;

import javax.jdo.InstanceLifecycleListener;

/**
 * Wrapper for a LifecycleListener for an array of classes. If the classes is null, applies
 * to all classes.
 * 
 * @version $Revision: 1.3 $
 */
public class LifecycleListenerSpecification
{
    /** The classes to which this listener applies */
    Class[] classes;

    /** The listener. */
    InstanceLifecycleListener listener;
    
    /**
     * The {@link javax.jdo.InstanceLifecycleEvent} class has not fine grained
     * LifeCycle event types defined, and in order to facilite the internal
     * coversation and raise the correct events we define fine grained types
     * here
     */
    public static final int EVENT_POST_CREATE = 1;
    
    public static final int EVENT_POST_LOAD = 2;
    
    public static final int EVENT_PRE_STORE = 3;
    public static final int EVENT_POST_STORE = 4;
    
    public static final int EVENT_PRE_CLEAR = 5;
    public static final int EVENT_POST_CLEAR = 6;
    
    public static final int EVENT_PRE_DELETE = 7;
    public static final int EVENT_POST_DELETE = 8;
    
    public static final int EVENT_PRE_DIRTY = 9;
    public static final int EVENT_POST_DIRTY = 10;
    
    public static final int EVENT_PRE_DETACH = 11;
    public static final int EVENT_POST_DETACH = 12;
    
    public static final int EVENT_PRE_ATTACH = 13;
    public static final int EVENT_POST_ATTACH = 14;

    /**
     * Constructor.
     * @param listener The listener
     * @param classes The classes supported by the listener
     */
    public LifecycleListenerSpecification(InstanceLifecycleListener listener, Class[] classes)
    {
        this.classes = classes;
        this.listener = listener;
    }

    /**
     * Accessor for the listener for this specification
     * @return The listener
     */
    public InstanceLifecycleListener getListener()
    {
        return listener;
    }

    /**
     * Accessor for the classes in this specification
     * @return The classes
     */
    public Class[] getClasses()
    {
        return classes;
    }
    
    /**
     * Accessor for whether the listener supported the class.
     * Actually checks if the passed class is a subclass of one of the listener classes.
     * @param cls The class
     * @return Whether it is supported
     */
    public boolean forClass(Class cls)
    {
        if (classes == null)
        {
            return true;
        }
        
        for (int i=0;i<classes.length;i++)
        {
            if (classes[i].isAssignableFrom(cls))
            {
                return true;
            }
        }
        return false;
    }
}