/**********************************************************************
Copyright (c) 2004 Erik Bengtson and others.
All rights reserved.   This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
2004 Andy Jefferson - added MetaDataManager
    ...
**********************************************************************/
package org.jpox;

import java.util.HashMap;
import java.util.Map;

import org.jpox.metadata.MetaDataManager;

/**
 * Represents a context of a PMF instance.
 *
 * @version $Revision: 1.2 $ 
 */
public class PMFContext
{
    /**
     * Hash for StoreManagers, key: user+password
     */
    private Map storeMgrs = new HashMap();

    /**
     * MetaDataManager. This will replace metaDataByClass and
     * classesWithoutMetaData when its fully enabled.
     */
    private MetaDataManager metaDataManager=null;

    private final PMFConfiguration pmfConfiguration;

    /**
     * Constructor for the context.
     * @param pmfConfiguration The pmf configuration
     */
    public PMFContext(PMFConfiguration pmfConfiguration)
    {
        this.pmfConfiguration = pmfConfiguration;
    }

    /**
     * Accessor for the Meta-Data Manager.
     * @return Returns the MetaDataManager.
     */
    public MetaDataManager getMetaDataManager()
    {
        if (metaDataManager == null)
        {
            // Allocate a MetaData manager handling our type of MetaData files
            String jdo_file_suffix=pmfConfiguration.getJdoMetaDataFileExtension();
            String orm_file_suffix=pmfConfiguration.getOrmMetaDataFileExtension();
            String orm_mapping_name=pmfConfiguration.getMapping();
            String default_inheritance_strategy = pmfConfiguration.getDefaultInheritanceStrategy();
            if (jdo_file_suffix != null || orm_file_suffix != null || orm_mapping_name != null || default_inheritance_strategy != null)
            {
                metaDataManager = new MetaDataManager(jdo_file_suffix, orm_file_suffix, orm_mapping_name, default_inheritance_strategy);
            }
            else
            {
                metaDataManager = new MetaDataManager();
            }
        }

        return metaDataManager;
    }

    /**
     * Accessor for the Store Managers. 
     * @return Returns the storeMgrs.
     */
    public Map getStoreMgrs()
    {
        return storeMgrs;
    }

    /**
     * Accessor for the PMF Configuration. 
     * @return Returns the pmfConfiguration.
     */
    public PMFConfiguration getPmfConfiguration()
    {
        return pmfConfiguration;
    }
}