/**********************************************************************
Copyright (c) 2002 Kelly Grizzle and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
2003 Erik Bengtson - added getObjectByAID
2004 Andy Jefferson - added MetaDataManager
   ...
**********************************************************************/
package org.jpox;

import java.io.PrintWriter;
import java.sql.Connection;
import java.sql.SQLException;
import javax.jdo.spi.PersistenceCapable;
import org.jpox.metadata.MetaDataManager;
import org.jpox.metadata.FieldMetaData;
import org.jpox.sco.SCO;
import org.jpox.store.FieldValues;
import org.jpox.store.StoreManager;

/**
 * Extension of SUN's JDO PersistenceManager. Adds aspects particular to this
 * implementation.
 *
 * @version $Revision: 1.2 $
 **/
public interface PersistenceManager extends javax.jdo.PersistenceManager
{
    StoreManager getStoreManager();

    MetaDataManager getMetaDataManager();


    /**
     * Accessor for the connection to the datasource.
     * @param forWriting Whether the connection is read-only or read-write.
     * @return The Connection
     * @throws SQLException
     */
    Connection getConnection(boolean forWriting) throws SQLException;

    /**
     * Accessor for the connection to the datasource.
     * @param forWriting Whether the connection is read-only or read-write.
     * @param useOpenedConnection whether to use an already opened connection, if one is opened. 
     * @return The Connection
     * @throws SQLException
     */
    Connection getConnection(boolean forWriting, boolean useOpenedConnection) throws SQLException;

    /**
     * Checks whether a connection to the datasource is open.
     * @return true if the Connection is open
     * @throws SQLException
     */
    boolean isConnectionOpen();
    
    void releaseConnection(Connection conn) throws SQLException;

    void enlistInTransaction(StateManager sm);

    void evictFromTransaction(StateManager sm);

    void removeStateManager(StateManager sm);

	Object getObjectById(Object id, FieldValues fv);

    /**
     * Accessor for an object given the object id.
     * @param id Id of the object.
     * @param validate Whether to validate the object state
     * @param checkInheritance Whether look to the database to determine which
     * class this object is. This parameter is a hint. Set false, if it's
     * already determined the correct pcClass for this pc "object" in a certain
     * level in the hierarchy. Set to true and it will look to the database.
     * @return The Object
     */
    public Object getObjectById(Object id, boolean validate, boolean checkInheritance);
    
	Object getObjectById(Object id, FieldValues fv,Class pcClass, boolean ignoreCache);
	Object getObjectByAID(Class pcClass, FieldValues fv, boolean ignoreCache, boolean checkInheritance);

    void makePersistent(Object obj, FieldValues preInsertChanges);
    
    SCO newSCOInstance(Class type, Object owner, FieldMetaData fmd);

    StateManager findStateManager(PersistenceCapable pc);

    void hereIsStateManager(StateManager sm, PersistenceCapable pc);

    void markDirty(StateManager sm);

    void clearDirty(StateManager sm);

    /**
     * Tests whether this object is being inserted.
     * @param pc the object to verify the status
     * @return true if this instance is inserting.
     */
    boolean isInserting(PersistenceCapable pc);
    
	/**
	 * Whether the datastore operations are delayed until commit.
     * In optimistic transactions this is automatically enabled.
	 * @return true if datastore operations are delayed until commit
     */
    boolean isDelayDatastoreOperationsEnabled();
        
    void dump(Object obj, PrintWriter out);
    
	public ClassLoaderResolver getClassLoaderResolver();
	
	/**
	 * Accessor for a handle to this PersistenceManager.
	 * @return The handle.
	 **/
	public PersistenceManager getPMHandle();

	public void addStateManager(StateManager sm);

	public StateManager getStateManagerById(Object myID);

    /**
     * Gets the context in which this PersistenceManager is running
     * @return Returns the pmfContext.
     */	
	public PMFContext getPMFContext();

}