/**********************************************************************
Copyright (c) 2004 Andy Jefferson and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
2004 Andy Jefferson - added sysproperty and classpath input options
2004 Andy Jefferson - changed to be derived from Java taskdef
2004 Andy Jefferson - removed redundant methods. Changed to se fork=true always
    ...
**********************************************************************/
package org.jpox;

import java.io.File;
import java.util.Vector;
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.DirectoryScanner;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.taskdefs.Java;
import org.apache.tools.ant.types.FileSet;

/**
 * SchemaTool Ant Task.
 * Accepts the following parameters
 * <UL>
 * <LI><B>mode</B> Mode of operation (<B>"create"</B>, "delete", "validate", "dbinfo").</LI>
 * <LI><B>verbose</B> Verbose output.</LI>
 * </UL>
 * 
 * @version $Revision: 1.2 $
 */
public class SchemaToolTask extends Java
{
    private boolean createMode=true;
    private boolean deleteMode=false;
    private boolean validateMode=false;
    private boolean dbinfoMode=false;

    Vector filesets = new Vector();

    /**
     * Constructor.
     */
    public SchemaToolTask()
    {
        setClassname("org.jpox.SchemaTool");
        setFork(true);
    }

    /**
     * Execute method, to execute the task.
     **/
    public void execute()
    throws BuildException
    {
        if (createMode)
        {
            createArg().setValue("-create");
        }
        else if (deleteMode)
        {
            createArg().setValue("-delete");
        }
        else if (validateMode)
        {
            createArg().setValue("-validate");
        }
        else if (dbinfoMode)
        {
            createArg().setValue("-dbinfo");
        }
        for( int i=0; i<getFiles().length; i++ )
        {
            createArg().setFile(getFiles()[i]);
        }

        setFork(true);

        super.execute();
    }

    public void addFileSet(FileSet fs)
    {
        filesets.addElement(fs);
    }
    
    protected File[] getFiles()
    {
        Vector v = new Vector();
        final int size = filesets.size();
        for (int i = 0; i < size; i++)
        {
            FileSet fs = (FileSet) filesets.elementAt(i);
            DirectoryScanner ds = fs.getDirectoryScanner(getProject());
            ds.scan();
            String[] f = ds.getIncludedFiles();
            for (int j = 0; j < f.length; j++)
            {
                String pathname = f[j];
                File file = new File(ds.getBasedir(), pathname);
                file = getProject().resolveFile(file.getPath());
                v.addElement(file);
            }
        }
        File[] files = new File[v.size()];
        v.copyInto(files);
        return files;
    }

    /**
     * set verbose
     * @param verbose Whether to give verbose output
     */
    public void setVerbose(boolean verbose)
    {
        if (verbose)
        {
            createArg().setValue("-v");
            log("SchemaTool verbose: " + verbose, Project.MSG_VERBOSE);
        }
    }

    /**
     * Sets the mode of operation.
     * @param mode The mode of operation ("create", "delete", "validate", "dbinfo")
     */
    public void setMode(String mode)
    {
        if (mode == null)
        {
            return;
        }
        if (mode.equalsIgnoreCase("create"))
        {
            this.createMode = true;
            this.deleteMode = false;
            this.validateMode = false;
            this.dbinfoMode = false;
        }
        else if (mode.equalsIgnoreCase("delete"))
        {
            this.createMode = false;
            this.deleteMode = true;
            this.validateMode = false;
            this.dbinfoMode = false;
        }
        else if (mode.equalsIgnoreCase("validate"))
        {
            this.createMode = false;
            this.deleteMode = false;
            this.validateMode = true;
            this.dbinfoMode = false;
        }
        else if (mode.equalsIgnoreCase("dbinfo"))
        {
            this.createMode = false;
            this.deleteMode = false;
            this.validateMode = false;
            this.dbinfoMode = true;
        }        
        else
        {
            System.err.println("SchemaTool : The mode parameter accepts values of \"create\", \"delete\", \"validate\", and \"dbinfo\"");
        }
    }
}