/**********************************************************************
Copyright (c) 2002 Kelly Grizzle and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
2002 Mike Martin - unknown changes
2003 Erik Bengtson - removed exist() operation
2004 Andy Jefferson - added getHighestFieldNumber()
    ...
**********************************************************************/
package org.jpox;

import java.io.PrintWriter;

import javax.jdo.spi.PersistenceCapable;

import org.jpox.metadata.MetaDataManager;
import org.jpox.state.ActivityState;
import org.jpox.store.StoreManager;
import org.jpox.store.fieldmanager.FieldManager;

/**
 * Extension to SUN's JDO StateManager class. This makes the assumption that a
 * StateManager corresponds to ONE PersistenceCapable object. As a result of
 * this assumption various methods are added to the basic JDO definition.
 *
 * @version $Revision: 1.4 $
 **/
public interface StateManager extends javax.jdo.spi.StateManager
{
    PersistenceCapable getObject();

    Object getInternalObjectId();
	Object getExternalObjectId(PersistenceCapable pc);

	/**
	  * Returns the PersistenceManager that owns the StateManager instance
      * @return PersistenceManager
	 */
    PersistenceManager getPersistenceManager();

    StoreManager getStoreManager();

    MetaDataManager getMetaDataManager();

    void makeDirty(int fieldNumber);

    /**
     * Marks the given field dirty for issuing an update after the insert.
     * @param pc The Persistence Capable object
     * @param fieldNumber The no of field to mark as dirty. 
     */
    void updateFieldAfterInsert(PersistenceCapable pc, int fieldNumber);

    /**
     * Update the acitvity state.
     */
    void changeActivityState( ActivityState activityState);
    
    void makeTransactional();

    void makeNontransactional();

    void makeTransient();

	void makePersistent();

    /**
     * Method to attach to this the detached PersistenceCapable instance
     * 
     * @since 1.1
     * 
     * @param detachedPC
     *            the detached PersistenceCapable instance to be attached
     * @param makeTransactional
     *            whether to make this as transactional
     */
    public Object attachCopy(PersistenceCapable detachedPC, boolean makeTransactional);

    /**
     * Method to make detached copy of this instance
     * 
     * @since 1.1
     * 
     * @return the detached PersistenceCapable instance
     */
    public Object detachCopy();
    
	/**
	 * Makes Transactional Transient instances persistent
	 */
	void makePersistentTransactionalTransient();

    /**
     * Validates whether the persistence capable instance exists in the
     * datastore. If the instance does not exist in the datastore, this method
     * will fail raising a javax.jdo.JDOObjectNotFoundException.
     */
    public void validate();
    
    void evict();

    void refresh();

    void retrieve(boolean dfgOnly);

    void postCommit(javax.jdo.Transaction tx);

    void preRollback();

    void deletePersistent();

    void loadDefaultFetchGroup();
      
    void flush();

    Object provideField(int fieldNumber);

    void provideFields(int fieldNumbers[], FieldManager fm);

    void replaceField(int fieldNumber, Object value);

    void replaceFields(int fieldNumbers[], FieldManager fm);

    Object newSCOInstance(int fieldNumber, Object value);

    void replaceSCOField(int fieldNumber, Object value);

    Object newSCOEmptyInstance(int fieldNumber);
    
    void dump(PrintWriter out);
    
    /**
     * Tests whether this object is being inserted.
     * @return true if this instance is inserting.
     */
    boolean isInserting();
    
	/**
     * If the id is obtained after inserting the object into the database, set
     * new a new id for persistent classes (for example, increment).
     * @param id the id received from the datastore
     */
	void setPostStoreNewObjectId(Object id);  
	
	/**
	 * Accessor the value in the datastore for the version column
	 * @return The version column value
	 */
    public Object getOptimisticDatastoreVersion();

    /**
	 * Sets the value for the version column in a transaction not yet committed
     * @param optimisticTransactionalVersion
     */
    public void setOptimisticTransactionalVersion(Object optimisticTransactionalVersion);
    
    /**
	 * Sets the value for the version column in the datastore
     * @param optimisticDatastoreVersion
     */
    public void setOptimisticDatastoreVersion(Object optimisticDatastoreVersion);

    /**
     * Accessor for the highest field number
     * @return Highest field number
     */
    public int getHighestFieldNumber();
}