/**********************************************************************
Copyright (c) 2002 Kelly Grizzle and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
2003 Andy Jefferson - commented
    ...
**********************************************************************/
package org.jpox;

import java.sql.Connection;
import java.sql.SQLException;
import javax.jdo.JDOConnection;

/**
 * Extension of SUN's JDO Transaction.
 *
 * @version $Revision: 1.1 $
 **/ 
public interface Transaction extends javax.jdo.Transaction
{
    /**
     * Accessor for the connection to the datasource.
     * @param forWriting Whether the connection is read-only or read-write.
     * @return The Connection
     * @throws SQLException
     */    
    Connection getConnection(boolean forWriting) throws SQLException;

    /**
     * Accessor for the connection to the datasource.
     * @param forWriting Whether the connection is read-only or read-write.
     * @param useOpenedConnection whether to use an already opened connection, if one is opened. 
     * @return The Connection
     * @throws SQLException
     */
    Connection getConnection(boolean forWriting, boolean useOpenedConnection) throws SQLException;

    /**
     * Checks whether a connection to the datasource is open.
     * @return true if the Connection is open
     * @throws SQLException
     */
    boolean isConnectionOpen();
    
    /**
     * Method to release the Connection.
     * @param conn The Connection to release.
     * @throws SQLException Thrown when an error occurs releasing the
     * connection.
     **/    
    void releaseConnection(Connection conn) throws SQLException;

    /**
     * Accessor for the transaction isolation level.
     * @return The transaction isolation level.
     **/
    int getTransactionIsolation();

    /**
     * Mutator for the transaction isolation level.
     * @param isolationLevel The isolation level required.
     **/    
    void setTransactionIsolation(int isolationLevel);

    /**
     * Accessor for whether to use an update lock when fetching data.
     * @return Whether to use an update lock.
     **/    
    boolean useUpdateLockOnFetch();
    
    /**
     * Accessor for the connection to the datasource.
     * If this method is called while a datastore transaction is active,
     * the object returned will be enlisted in the current transaction.
     * If called in an optimistic transaction or outside an active
     * transaction, the object returned will not be enlisted in any transaction.
     * 
     * The object must be returned to the JDO implementation prior to calling
     * any JDO method or performing any action on any persistent instance that
     * might require the JDO implementation to use a connection. If the object
     * has not been returned and the JDO implementation needs a connection,
     * a JDOUserException is thrown. The object is returned to the JDO
     * implementation by calling the standard method on the object.
     * 
        //TODO understand better the JDO spec
        //TODO if the object not enlisted is not closed, raise an exception?
     * @return The Connection
     * @throws SQLException
     */
    JDOConnection getJDOConnection();    
}
