/**********************************************************************
Copyright (c) 2004 Andy Jefferson and others. All rights reserved.
This program and the accompanying materials are made available under 
the terms of the JPOX License v1.0 which accompanies this distribution.

Contributors:
    ...
**********************************************************************/
package org.jpox.cache;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

/**
 * Implementation of a Level 1 cache keeping hard (strong) references
 * to the objects. This means that objects are not garbage collected, 
 * and have to be removed directly by calls to remove(). This differs
 * from the WeakRefCache/SoftRefCache which do not guarantee to retain
 * objects.
 * 
 * @version $Revision: 1.1 $
 */
public class HardRefCache implements Level1Cache
{
    private Map hardCache = new HashMap();

	public HardRefCache()
	{
		//default constructor
	}

	public Object put(Object key, Object value)
	{
		return hardCache.put(key,value);        
	}

	public Object get(Object key)
	{
		return hardCache.get(key);
	}

	public boolean containsKey(Object key)
	{
		return hardCache.containsKey(key);
	}

	public Object remove(Object key)
	{
		return hardCache.remove(key);       
	}

	public void clear()
	{
		hardCache.clear();
	}
    
	/* (non-Javadoc)
	 * @see java.util.Map#containsValue(java.lang.Object)
	 */
	public boolean containsValue(Object value)
	{
		return hardCache.containsValue(value);
	}

	/* (non-Javadoc)
	 * @see java.util.Map#entrySet()
	 */
	public Set entrySet()
	{
		return hardCache.entrySet();
	}

	/* (non-Javadoc)
	 * @see java.util.Map#isEmpty()
	 */
	public boolean isEmpty()
	{
		return hardCache.isEmpty();
	}

	/* (non-Javadoc)
	 * @see java.util.Map#keySet()
	 */
	public Set keySet()
	{
		return hardCache.keySet();
	}

	/* (non-Javadoc)
	 * @see java.util.Map#putAll(java.util.Map)
	 */
	public void putAll(Map t)
	{
		hardCache.putAll(t);
	}

	/* (non-Javadoc)
	 * @see java.util.Map#size()
	 */
	public int size()
	{
		return hardCache.size();
	}

	/* (non-Javadoc)
	 * @see java.util.Map#values()
	 */
	public Collection values()
	{
		return hardCache.values();
	}
}