/**********************************************************************
Copyright (c) 2004 Andy Jefferson and others. All rights reserved.
This program and the accompanying materials are made available under 
the terms of the JPOX License v1.0 which accompanies this distribution.

Contributors:
    ...
**********************************************************************/
package org.jpox.cache;

import javax.jdo.DataStoreCache;
import javax.jdo.spi.PersistenceCapable;

/**
 * Interface for any Level 2 Cache used by JPOX.
 * Builds on the JDO definition, to provide the typical controls 
 * required by JPOX itself. Clearly users will be able to access
 * these, but then they provide added value over JDO 2.
 * 
 * @version $Revision: 1.1 $
 */
public interface Level2Cache extends DataStoreCache
{
    /**
     * Accessor for the number of pinned objects in the cache.
     * @return Number of pinned objects
     */
    int getNumberOfPinnedObjects();
    
    /**
     * Accessor for the number of unpinned objects in the cache.
     * @return Number of unpinned objects
     */
    int getNumberOfUnpinnedObjects();

    /**
     * Accessor for the total number of objects in the L2 cache.
     * @return Number of objects
     */
    int getSize();

    /**
     * Accessor for an object from the cache.
     * @param oid The Object ID
     * @return The PersistenceCapable object
     */
    PersistenceCapable get(Object oid);

    /**
     * Method to put an object in the cache.
     * @param oid The Object id for this object
     * @param pc The PersistenceCapable object
     * @return The value previously associated with this oid
     */
    PersistenceCapable put(Object oid, PersistenceCapable pc);

    /**
     * Accessor for whether the cache is empty.
     * @return Whether it is empty.
     */
    boolean isEmpty();

    /**
     * Method to clear the cache.
     */
    void clear();

    /**
     * Accessor for whether an object with the specified id is in the cache
     * @param oid The object id
     * @return Whether it is in the cache
     */
    boolean containsOid(Object oid);
    
    /**
     * Accessor for whether the specified persistence capable exists in the cache.
     * @param pc The persistence capable
     * @return Whether it is present.
     */
    boolean containsObject(PersistenceCapable pc);
}