/**********************************************************************
Copyright (c) 2003 Erik Bengtson and others. All rights reserved.
This program and the accompanying materials are made available under
the terms of the JPOX License v1.0 which accompanies this distribution. 

Contributors:
    ...
**********************************************************************/
package org.jpox.cache;

import java.util.Collection;
import java.util.Map;
import java.util.Set;

import org.jpox.util.WeakValueMap;

/**
 * Level 1 Cache using Weak referenced objects in a Map.
 * <p>If the garbage collector clears the reference, the corresponding key is
 * automatically removed from the map.
 *
 * @see java.lang.ref.WeakReference
 * @version $Revision: 1.2 $
 */
public class WeakRefCache implements Level1Cache
{
	private Map weakCache = new WeakValueMap();

	public WeakRefCache()
	{
		//default constructor
	}

    public Object put(Object key, Object value)
    {
		return weakCache.put(key,value);        
    }

    public Object get(Object key)
    {
		return weakCache.get(key);
    }

    public boolean containsKey(Object key)
    {
		return weakCache.containsKey(key);
    }

    public Object remove(Object key)
    {
		return weakCache.remove(key);       
    }

    public void clear()
    {
		weakCache.clear();
    }
    
    /* (non-Javadoc)
     * @see java.util.Map#containsValue(java.lang.Object)
     */
    public boolean containsValue(Object value)
    {
		return weakCache.containsValue(value);
    }

    /* (non-Javadoc)
     * @see java.util.Map#entrySet()
     */
    public Set entrySet()
    {
		return weakCache.entrySet();
    }

    /* (non-Javadoc)
     * @see java.util.Map#isEmpty()
     */
    public boolean isEmpty()
    {
		return weakCache.isEmpty();
    }

    /* (non-Javadoc)
     * @see java.util.Map#keySet()
     */
    public Set keySet()
    {
        return weakCache.keySet();
    }

    /* (non-Javadoc)
     * @see java.util.Map#putAll(java.util.Map)
     */
    public void putAll(Map t)
    {
		weakCache.putAll(t);
    }

    /* (non-Javadoc)
     * @see java.util.Map#size()
     */
    public int size()
    {
        return weakCache.size();
    }

    /* (non-Javadoc)
     * @see java.util.Map#values()
     */
    public Collection values()
    {
        return weakCache.values();
    }
}