/**********************************************************************
Copyright (c) 2004 Andy Jefferson and others. All rights reserved.
This program and the accompanying materials are made available under
the terms of the JPOX License v1.0 which accompanies this distribution.

Contributors:
    ...
**********************************************************************/
package org.jpox.exceptions;

import javax.jdo.JDOUserException;

import org.jpox.util.Localiser;

/**
 * A <tt>ClassNotDetachableException</tt> is thrown if an attempt is
 * made to detach an object whose class is not detachable.
 *
 * @version $Revision: 1.2 $
 */
public class ClassNotDetachableException extends JDOUserException
{
    private static final Localiser LOCALISER=Localiser.getInstance("org.jpox.exceptions.Localisation");

    /**
     * Constructs a class-not-detachable exception with the specified
     * detail message.
     * @param class_name Name of the class
     */
    public ClassNotDetachableException(String class_name)
    {
        super(LOCALISER.msg("Exception.ClassNotDetachable",class_name));
    }

    /**
     * Constructs a class-not-detachable exception with the specified
     * detail message and nested exception.
     * @param class_name name of the class
     * @param nested the nested exception(s).
     */
    public ClassNotDetachableException(String class_name, Exception nested)
    {
        super(LOCALISER.msg("Exception.ClassNotDetachable",class_name), nested);
    }

    /**
     * Constructs a class-not-detachable exception for many objects with the specified
     * detail message and nested exceptions.
     * @param nested the nested exception(s).
     */
    public ClassNotDetachableException(Throwable[] nested)
    {
        super(LOCALISER.msg("Exception.ClassesNotDetachable"), nested);
    }
}