/**********************************************************************
Copyright (c) 2002 Mike Martin (TJDO) and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
2002 Kelly Grizzle (TJDO)
2003 Andy Jefferson - commented and localised.
2004 Andy Jefferson - changed to take the class name.
    ...
**********************************************************************/
package org.jpox.exceptions;

import javax.jdo.JDOUserException;

import org.jpox.util.Localiser;

/**
 * A <tt>ClassNotPersistenceCapableException</tt> is thrown if an attempt is
 * made to persist an object whose class is not persistence-capable. This may
 * be because the class has not been appropriately enhanced to be made
 * persistence-capable.
 *
 * @version $Revision: 1.1 $
 * @see org.jpox.metadata.ClassMetaData
 */
public class ClassNotPersistenceCapableException extends JDOUserException
{
    private static final Localiser LOCALISER=Localiser.getInstance("org.jpox.exceptions.Localisation");

    /**
     * Constructs a class-not-persistence-capable exception with the specified
     * detail message.
     * @param class_name Name of the class
     */
    public ClassNotPersistenceCapableException(String class_name)
    {
        super(LOCALISER.msg("Exception.ClassNotPersistenceCapable",class_name));
    }

    /**
     * Constructs a class-not-persistence-capable exception with the specified
     * detail message and nested exception.
     * @param class_name name of the class
     * @param nested the nested exception(s).
     */
    public ClassNotPersistenceCapableException(String class_name, Exception nested)
    {
        super(LOCALISER.msg("Exception.ClassNotPersistenceCapable",class_name), nested);
    }
}
