/**********************************************************************
Copyright (c) 04-Jun-2004 Andy Jefferson and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution.

Contributors:
    Kikucho Kousuke - org.jpox.enhancer.conf.JDOConfigExtension
    ...
**********************************************************************/
package org.jpox.metadata;

/**
 * Representation of the Meta-Data for an extension.
 * Takes the form of key and value. The vendor will be "jpox".
 * <P>Represents the following MetaData element 
 * <PRE>
 * &lt;!ELEMENT extension (extension)*&gt;
 * &lt;!ATTLIST extension vendor-name CDATA #REQUIRED&gt;
 * &lt;!ATTLIST extension key CDATA #IMPLIED&gt;
 * &lt;!ATTLIST extension value CDATA #IMPLIED&gt;
 * </PRE> 
 * @since 1.1 
 * @version $Revision: 1.1 $
 */
public class ExtensionMetaData
{
    /**
     * vendor-name tag value.
     */
    protected String vendorName;

    /**
     * key tag value.
     */
    protected String key;

    /**
     * value tag value.
     */
    protected String value;

    /**
     * Constructor
     * @param vendorName vendor-name tag value
     * @param key key tag value
     * @param value  value tag value
     */
    public ExtensionMetaData(String vendorName, String key, String value)
    {
        this.vendorName = vendorName;
        this.key = key;
        this.value = value;
    }

    /**
     * Accessor for the key tag value.
     * @return key tag value
     */
    public String getKey()
    {
        return key;
    }

    /**
     * Accessor for the value tag value .
     * @return value tag value
     */
    public String getValue()
    {
        return value;
    }

    /**
     * Accessor for the vendor-name tag value.
     * @return vendor-name tag value
     */
    public String getVendorName()
    {
        return vendorName;
    }

    /**
     * Returns a string representation of the object.
     * @return a string representation of the object.
     */
    public String toString()
    {
        return "<extension vendor-name=\"" + vendorName + "\" key=\"" + key + "\" value=\"" + value + "\"/>";
    }
}
