/**********************************************************************
Copyright (c) 09-Jun-2004 Andy Jefferson and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
    ...
**********************************************************************/
package org.jpox.metadata;

/**
 * Class defining the possibilities for persistence, in terms of the type of
 * persistence, and the types that are capable to be supported. 
 * @since 1.1 
 * @version $Revision: 1.1 $
 */
public class FieldPersistenceModifier
{
    /**
     * persistence-modifier="persistent"
     */
    public static final FieldPersistenceModifier PERSISTENT = new FieldPersistenceModifier(1);

    /**
     * persistence-modifier="transactional"
     */
    public static final FieldPersistenceModifier TRANSACTIONAL = new FieldPersistenceModifier(2);

    /**
     * persistence-modifier="none"
     */
    public static final FieldPersistenceModifier NONE = new FieldPersistenceModifier(3);

    public static final FieldPersistenceModifier DEFAULT = new FieldPersistenceModifier(-1);

    /**
     * persistent|transactional|none id
     */
    private final int typeId;

    /**
     * constructor
     * @param i type id
     */
    private FieldPersistenceModifier(int i)
    {
        this.typeId = i;
    }

    /**
     * Indicates whether some other object is "equal to" this one.
     * @param o the reference object with which to compare. 
     * @return true if this object is the same as the obj argument; false otherwise.
     */
    public boolean equals(Object o)
    {
        if (o instanceof FieldPersistenceModifier)
        {
            return ((FieldPersistenceModifier)o).typeId == typeId;
        }
        return false;
    }

    /**
     * Returns a string representation of the object.
     * @return a string representation of the object.
     */
    public String toString()
    {
        switch (typeId)
        {
            case 1 :
                return "persistent";
            case 2 :
                return "transactional";
            case 3 :
                return "none";
        }
        return "";
    }

    protected int getType()
    {
        return typeId;
    }

    /**
     * Return FieldPersistenceModifier from String.
     * @param value persistence-modifier attribute value
     * @return Instance of FieldPersistenceModifier. 
     *         If value invalid, return null.
     */
    public static FieldPersistenceModifier getFieldPersistenceModifier(final String value)
    {
        if (value == null)
        {
            return null;
        }
        else if (FieldPersistenceModifier.PERSISTENT.toString().equalsIgnoreCase(value))
        {
            return FieldPersistenceModifier.PERSISTENT;
        }
        else if (FieldPersistenceModifier.TRANSACTIONAL.toString().equalsIgnoreCase(value))
        {
            return FieldPersistenceModifier.TRANSACTIONAL;
        }
        else if (FieldPersistenceModifier.NONE.toString().equalsIgnoreCase(value))
        {
            return FieldPersistenceModifier.NONE;
        }
        return null;
    }
}
