/**********************************************************************
Copyright (c) Jun 20, 2004 Erik Bengtson and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
    ...
**********************************************************************/
package org.jpox.metadata;

/**
 * Foreign keys represent a consistency constraint in the database that must be
 * maintained. This class enumerates the actions which happens when foreign-keys
 * are updated or deleted.
 * 
 * @author Erik Bengtson
 * @version $Revision: 1.1 $
 */
public abstract class ForeignKeyAction
{
    private final int typeId;

    /**
     * constructor
     * @param i type id
     */
    protected ForeignKeyAction(int i)
    {
        this.typeId = i;
    }

    /**
     * Indicates whether some other object is "equal to" this one.
     * @param o the reference object with which to compare. 
     * @return true if this object is the same as the obj argument; false otherwise.
     */
    public boolean equals(Object o)
    {
        if (o instanceof ForeignKeyAction)
        {
            return ((ForeignKeyAction)o).typeId == typeId;
        }
        return false;
    }

    /**
     * Accessor for the type.
     * @return The type
     */
    protected int getType()
    {
        return typeId;
    }    
}
