/**********************************************************************
Copyright (c) 04-Jun-2004 Andy Jefferson and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
    Kikuchi Kousuke - org.jpox.enhancer.conf.IdentityType
    ...
**********************************************************************/
package org.jpox.metadata;

/**
 * Representation of the values for identity-type.
 * <br>
 * &lt;!ATTLIST class identity-type (application|datastore|nondurable) #IMPLIED&gt;
 *
 * @since 1.1 
 * @version $Revision: 1.1 $
 */
public class IdentityType
{
    /**
     * identity-type="application"
     */
    public static final IdentityType APPLICATION = new IdentityType(1);

    /**
     * identity-type="datastore"
     */
    public static final IdentityType DATASTORE = new IdentityType(2);

    /**
     * identity-type="nondurable"
     */
    public static final IdentityType NONDURABLE = new IdentityType(3);

    private final int typeId;
    private IdentityType(int i)
    {
        this.typeId = i;
    }

    /**
     * Indicates whether some other object is "equal to" this one.
     * @param o the reference object with which to compare. 
     * @return true if this object is the same as the obj argument; false otherwise.
     */
    public boolean equals(Object o)
    {
        if (o instanceof IdentityType)
        {
            return ((IdentityType)o).typeId == typeId;
        }
        return false;
    }

    /**
     * Returns a string representation of the object.
     * @return a string representation of the object.
     */
    public String toString()
    {
        switch (typeId)
        {
            case 1 :
                return "application";
            case 2 :
                return "datastore";
            case 3 :
                return "nondurable";
        }
        return "";
    }

    public int getType()
    {
        return typeId;
    }

    /**
     * Return IdentityType from String.
     * @param value identity-type attribute value
     * @return Instance of IdentityType. If parse failed, return null.
     */
    public static IdentityType getIdentityType(final String value)
    {
        if (value == null)
        {
            return null;
        }
        else if (IdentityType.APPLICATION.toString().equalsIgnoreCase(value))
        {
            return IdentityType.APPLICATION;
        }
        else if (IdentityType.DATASTORE.toString().equalsIgnoreCase(value))
        {
            return IdentityType.DATASTORE;
        }
        else if (IdentityType.NONDURABLE.toString().equalsIgnoreCase(value))
        {
            return IdentityType.NONDURABLE;
        }
        return null;
    }
}
