/**********************************************************************
Copyright (c) 2004 Andy Jefferson and others. All rights reserved.
This program and the accompanying materials are made available under 
the terms of the JPOX License v1.0 which accompanies this distribution.

Contributors:
    ...
**********************************************************************/
package org.jpox.metadata;

/**
 * Representation of whether an item is indexed or not.
 * @version $Revision: 1.1 $
 */
public class IndexedValue
{
    public static final IndexedValue TRUE=new IndexedValue(1);

    public static final IndexedValue FALSE=new IndexedValue(2);

    public static final IndexedValue UNIQUE=new IndexedValue(3);

    /**
     * The type id.
     */
    private final int typeId;

    /**
     * constructor
     * @param i type id
     */
    private IndexedValue(int i)
    {
        this.typeId = i;
    }

    /**
     * Indicates whether some other object is "equal to" this one.
     * @param o the reference object with which to compare. 
     * @return true if this object is the same as the obj argument;
     *         false otherwise.
     */
    public boolean equals(Object o)
    {
        if (o instanceof IndexedValue)
        {
            return ((IndexedValue)o).typeId == typeId;
        }
        return false;
    }

    /**
     * Returns a string representation of the object.
     * @return a string representation of the object.
     */
    public String toString()
    {
        switch (typeId)
        {
            case 1 :
                return "true";
            case 2 :
                return "false";
            case 3 :
                return "unique";
        }
        return "";
    }

    /**
     * Accessor for the type.
     * @return Type
     **/
    public int getType()
    {
        return typeId;
    }

    public static IndexedValue getIndexedValue(final String value)
    {
        if (value == null)
        {
            return null;
        }
        else if (IndexedValue.TRUE.toString().equals(value))
        {
            return IndexedValue.TRUE;
        }
        else if (IndexedValue.FALSE.toString().equals(value))
        {
            return IndexedValue.FALSE;
        }
        else if (IndexedValue.UNIQUE.toString().equals(value))
        {
            return IndexedValue.UNIQUE;
        }
        return IndexedValue.TRUE;
    }
}