/**********************************************************************
Copyright (c) 14-Jun-2004 Andy Jefferson and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
    ...
**********************************************************************/
package org.jpox.metadata;

/**
 * Representation of the values for inheritance "strategy".
 *
 * @since 1.1
 * @version $Revision: 1.1 $
 */
public class InheritanceStrategy
{
    /**
     * strategy="no-table"
     */
    public static final InheritanceStrategy NO_TABLE=new InheritanceStrategy(1);

    /**
     * strategy="new-table"
     */
    public static final InheritanceStrategy NEW_TABLE=new InheritanceStrategy(2);

    /**
     * strategy="superclass-table"
     */
    public static final InheritanceStrategy SUPERCLASS_TABLE=new InheritanceStrategy(3);

    /**
     * The type id.
     */
    private final int typeId;

    /**
     * constructor
     * @param i type id
     */
    private InheritanceStrategy(int i)
    {
        this.typeId = i;
    }

    /**
     * Indicates whether some other object is "equal to" this one.
     * @param o the reference object with which to compare. 
     * @return true if this object is the same as the obj argument;
     *         false otherwise.
     */
    public boolean equals(Object o)
    {
        if (o instanceof InheritanceStrategy)
        {
            return ((InheritanceStrategy)o).typeId == typeId;
        }
        return false;
    }

    /**
     * Returns a string representation of the object.
     * @return a string representation of the object.
     */
    public String toString()
    {
        switch (typeId)
        {
            case 1 :
                return "no-table";
            case 2 :
                return "new-table";
            case 3 :
                return "superclass-table";
        }
        return "";
    }

    /**
     * Accessor for the type.
     * @return Type
     **/
    public int getType()
    {
        return typeId;
    }

    public static InheritanceStrategy getInheritanceStrategy(final String value)
    {
        if (value == null)
        {
            return InheritanceStrategy.NEW_TABLE;
        }
        else if (InheritanceStrategy.NO_TABLE.toString().equals(value))
        {
            return InheritanceStrategy.NO_TABLE;
        }
        else if (InheritanceStrategy.NEW_TABLE.toString().equals(value))
        {
            return InheritanceStrategy.NEW_TABLE;
        }
        else if (InheritanceStrategy.SUPERCLASS_TABLE.toString().equals(value))
        {
            return InheritanceStrategy.SUPERCLASS_TABLE;
        }
        return InheritanceStrategy.NEW_TABLE;
    }
}
