/**********************************************************************
Copyright (c) 04-Jun-2004 Andy Jefferson and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution.

Contributors:
    Kikucho Kousuke - org.jpox.enhancer.conf.InvalidMetaDataException
    ...
**********************************************************************/
package org.jpox.metadata;

import java.io.PrintStream;
import java.io.PrintWriter;

import org.jpox.util.Localiser;

/**
 * Representation of an exception thrown when an error occurs in Meta-Data
 * definition.
 * TODO Change this to extend org.jpox.util.LocalisedRuntimeException
 *
 * @version $Revision: 1.1 $
 */
public class InvalidMetaDataException extends RuntimeException
{
    /** Message resources key */
    protected String messageKey;
    protected Throwable cause;

    /**
     * Default Constructor
     */
    public InvalidMetaDataException()
    {
        super();
    }

    /**
     * Constructor with message resource and cause exception
     * @param localiser message resources
     * @param key message resources key
     * @param cause cause exception
     */
    public InvalidMetaDataException(Localiser localiser,
                                    String key,
                                    Throwable cause)
    {
        this(localiser, key, "", "", "");
        this.cause = cause;
    }

    /**
     * Constructor with message resource, message param and cause exception
     * @param localiser message resources
     * @param key message resources key
     * @param param1 message resources param0
     * @param cause cause exception
     */
    public InvalidMetaDataException(Localiser localiser,
                                    String key,
                                    Object param1,
                                    Throwable cause)
    {
        this(localiser, key, param1, "", "");
        this.cause = cause;
    }

    /**
     * Constructor with message resource, message params and cause exception
     * @param localiser message resources
     * @param key message resources key
     * @param param1 message resources param0
     * @param param2 message resources param1
     * @param cause cause exception
     */
    public InvalidMetaDataException(Localiser localiser,
                                    String key,
                                    Object param1,
                                    Object param2,
                                    Throwable cause)
    {
        this(localiser, key, param1, param2, "");
        this.cause = cause;
    }

    /**
     * Constructor with message resource, message params and cause exception
     * @param localiser message resources
     * @param key message resources key
     * @param param1 message resources param0
     * @param param2 message resources param1
     * @param param3 message resources param2
     * @param cause cause exception
     */
    public InvalidMetaDataException(Localiser localiser,
                                    String key,
                                    Object param1,
                                    Object param2,
                                    Object param3,
                                    Throwable cause)
    {
        super(localiser.msg(key, param1, param2, param3));
        this.messageKey = key;
        this.cause = cause;
    }

    /**
     * Constructor with message resource 
     * @param localiser message resources
     * @param key message resources key
     */
    public InvalidMetaDataException(Localiser localiser,
                                    String key)
    {
        this(localiser, key, "", "", "");
    }

    /**
     * Constructor with message resource, message param
     * @param localiser message resources
     * @param key message resources key
     * @param param1 message resources param0
     */
    public InvalidMetaDataException(Localiser localiser,
                                    String key,
                                    Object param1)
    {
        this(localiser, key, param1, "", "");

    }

    /**
     * Constructor with message resource, message params
     * @param localiser message resources
     * @param key message resources key
     * @param param1 message resources param0
     * @param param2 message resources param1
     */
    public InvalidMetaDataException(Localiser localiser,
                                    String key,
                                    Object param1,
                                    Object param2)
    {
        this(localiser, key, param1, param2, "");
    }

    /**
     * Constructor with message resource, message params
     * @param localiser message resources
     * @param key message resources key
     * @param param1 message resources param0
     * @param param2 message resources param1
     * @param param3 message resources param2
     */
    public InvalidMetaDataException(Localiser localiser,
                                    String key,
                                    Object param1,
                                    Object param2,
                                    Object param3)
    {
        super(localiser.msg(key, param1, param2, param3));
        this.messageKey = key;
    }

    /**
     * Constructor with message resource, message params
     * @param localiser message resources
     * @param key message resources key
     * @param param1 message resources param1
     * @param param2 message resources param2
     * @param param3 message resources param3
     * @param param4 message resources param4
     */
    public InvalidMetaDataException(Localiser localiser,
                                    String key,
                                    Object param1,
                                    Object param2,
                                    Object param3,
                                    Object param4)
    {
        super(localiser.msg(key, param1, param2, param3, param4));
        this.messageKey = key;
    }

    /**
     * Return message resource key
     * @return Message resource key
     */
    public String getMessageKey()
    {
        return messageKey;
    }

    /* (non-Javadoc)
     * @see java.lang.Throwable#printStackTrace()
     */
    public void printStackTrace()
    {
        super.printStackTrace();
        if (cause != null)
        {
            cause.printStackTrace();
        }
    }

    /* (non-Javadoc)
     * @see java.lang.Throwable#printStackTrace(java.io.PrintStream)
     */
    public void printStackTrace(PrintStream s)
    {
        super.printStackTrace(s);
        if (cause != null)
        {
            cause.printStackTrace(s);
        }
    }

    /* (non-Javadoc)
     * @see java.lang.Throwable#printStackTrace(java.io.PrintWriter)
     */
    public void printStackTrace(PrintWriter s)
    {
        super.printStackTrace(s);
        if (cause != null)
        {
            cause.printStackTrace(s);
        }
    }
}
