/**********************************************************************
Copyright (c) 09-Jun-2004 Andy Jefferson and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
    Kikuchi Kousuke - org.jpox.enhancer.conf.NullValue
    ...
**********************************************************************/
package org.jpox.metadata;

/**
 * jdo config null-value class. 
 * <br>
 * &lt;!ATTLIST field null-value (exception|default|none) 'none'&gt;
 *
 * @since 1.1 
 * @version $Revision: 1.1 $
 */
public class NullValue
{
    /**
     * null-value="exception"
     */
    public static final NullValue EXCEPTION = new NullValue(1);

    /**
     * null-value="default"
     */
    public static final NullValue DEFAULT = new NullValue(2);

    /**
     * null-value="none"
     */
    public static final NullValue NONE = new NullValue(3);

    /**
     * The type id
     */
    private final int typeId;

    /**
     * constructor
     * @param i type id
     */
    private NullValue(int i)
    {
        this.typeId = i;
    }

    /**
     * Indicates whether some other object is "equal to" this one.
     * @param o the reference object with which to compare. 
     * @return true if this object is the same as the obj argument; false otherwise.
     */
    public boolean equals(Object o)
    {
        if (o instanceof NullValue)
        {
            return ((NullValue)o).typeId == typeId;
        }
        return false;
    }

    /**
     * Returns a string representation of the object.
     * @return a string representation of the object.
     */
    public String toString()
    {
        switch (typeId)
        {
            case 1 :
                return "exception";
            case 2 :
                return "default";
            case 3 :
                return "none";
        }
        return "";
    }

    /**
     * Accessor for the type.
     * @return Type
     **/
    public int getType()
    {
        return typeId;
    }

    public static NullValue getNullValue(final String value)
    {
        if (value == null)
        {
            return NullValue.NONE;
        }
        else if (NullValue.DEFAULT.toString().equals(value))
        {
            return NullValue.DEFAULT;
        }
        else if (NullValue.EXCEPTION.toString().equals(value))
        {
            return NullValue.EXCEPTION;
        }
        else if (NullValue.NONE.toString().equals(value))
        {
            return NullValue.NONE;
        }
        return NullValue.NONE;
    }
}
