/**********************************************************************
Copyright (c) 2004 Andy Jefferson and others. All rights reserved.
This program and the accompanying materials are made available under 
the terms of the JPOX License v1.0 which accompanies this distribution.

Contributors:
    ...
**********************************************************************/
package org.jpox.metadata;

/**
 * The property element declares mapping between a virtual field of an implemented 
 * interface and the corresponding persistent field of a persistence-capable class. 
 * The name attribute is required, and declares the name for the property. The naming 
 * conventions for JavaBeans property names is used: the property name is the same as 
 * the corresponding get method for the property with the get removed and the resulting
 * name lowercased. 
 * The field-name attribute is required; it associates a persistent field with the 
 * named property.
 *
 * <H3>MetaData Element</H3>
 * The MetaData Element represented here is as follows
 * <PRE>
 * <!ELEMENT property ((collection|map|array|column)? , extension*)>
 * <!ATTLIST property name CDATA #REQUIRED>
 * <!ATTLIST property column CDATA #IMPLIED>
 * </PRE>
 * 
 * @version $Revision: 1.1 $
 */
public class PropertyMetaData extends ExtendableMetaData
{
    /** Name of the field */
    protected final String name;

    /** Column name for field. */
    protected final String columnName;

    // TODO Add collection/map/array/column

    /**
     * Constructor.
     * @param parent Parent interface object
     */
    public PropertyMetaData(final InterfaceMetaData parent,
                            final String name,
                            final String columnName)
    {
        super(parent);
        this.name = name;
        this.columnName = columnName;
    }

    /**
     * Accessor for columnName.
     * @return Returns the columnName.
     */
    public String getColumnName()
    {
        return columnName;
    }

    /**
     * Accessor for name.
     * @return Returns the name.
     */
    public String getName()
    {
        return name;
    }

    // ------------------------------ Utilities --------------------------------
 
    /**
     * Returns a string representation of the object.
     * @return a string representation of the object.
     */
    public String toString()
    {
        return toString("","");
    }

    /**
     * Returns a string representation of the object.
     * This can be used as part of a facility to output a MetaData file. 
     * @param prefix prefix string
     * @param indent indent string
     * @return a string representation of the object.
     */
    public String toString(String prefix,String indent)
    {
        StringBuffer sb = new StringBuffer();
        sb.append(prefix).append("<property name=\"" + name + "\"" + (columnName != null ? (" column=\"" + columnName + "\">\n") : ">\n"));

        // Add extensions
        sb.append(super.toString(prefix + indent,indent)); 

        sb.append(prefix + "</property>\n");
        return sb.toString();
    }
}