/**********************************************************************
Copyright (c) 2003 Erik Bengtson and others. All rights reserved.   
This program and the accompanying materials are made available under 
the terms of the JPOX License v1.0 which accompanies this distribution. 

Contributors:
2003 Andy Jefferson - coding standards
    ...
**********************************************************************/
package org.jpox.poid;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.NoSuchElementException;

/**
 * Persistent Object id Block. 
 *
 * @version $Revision: 1.1 $
 */
public class PoidBlock implements Serializable
{
    // the next id index
    private int nextIndex=0;
    
    /*
     * we maintain the ids in order.
     */
    private final List poidBlock;

    /**
     * Constructor for PoIdBlock.
     * @param oid The block of objects that will be considered the "IDs"
     */
    public PoidBlock(Object[] oid)
    {
        poidBlock = new ArrayList(Arrays.asList(oid));
    }

    /**
     * Constructor for PoIdBlock.
     * @param oid The list of objects that will be considered the "IDs"
     */
    public PoidBlock(List oid)
    {
        poidBlock = new ArrayList(oid);
    }

    /**
     * Get the next Poid, or null if block ids exhausted
     * @return The next POID
     */
    public Poid next()
    {
        if (nextIndex >= poidBlock.size())
        {
            throw new NoSuchElementException();
        }

        return new Poid(poidBlock.get(nextIndex++));
    }

    /**
     * check if it has more elements in block
     * @return boolean true when has more elements
     */
    public boolean hasNext()
    {
        return (nextIndex < poidBlock.size());
    }
}
