/**********************************************************************
Copyright (c) 2003 Erik Bengtson and others. All rights reserved.
This program and the accompanying materials are made available under
the terms of the JPOX License v1.0 which accompanies this distribution. 

Contributors:
2003 Andy Jefferson - coding standards
    ...
**********************************************************************/
package org.jpox.poid;

/**
 * PoidFactory creates instances of PoidGenerators
 *
 * @author Erik Bengtson
 * @version $Revision: 1.1 $
 */
public class PoidFactory
{
    /**
     * Gets an instance of a Poid Generator
     *
     * @param className the Poid class name
     * @return The PoidGenerator
     */
    public static PoidGenerator getPoidGenerator(String className)
    {
        PoidGenerator poidGenerator;
        try
        {
            poidGenerator = (PoidGenerator)Class.forName(className).newInstance();
        }
        catch (InstantiationException e)
        {
            poidGenerator = getDefaultPoidGenerator();
        }
        catch (IllegalAccessException e)
        {
            poidGenerator = getDefaultPoidGenerator();
        }
        catch (ClassNotFoundException e)
        {
            poidGenerator = getDefaultPoidGenerator();
        }
        catch (ClassCastException e)
        {
            poidGenerator = getDefaultPoidGenerator();
        }
        return poidGenerator;               
    }
    
    /**
     * Gets the default Poid Generator for the Poid API
     * 
     * @return The PoidGenerator
     */
    private static PoidGenerator getDefaultPoidGenerator()
    {
        return new SequenceTablePoidGenerator();
    }
}
