/**********************************************************************
Copyright (c) 2003 David Jencks and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
2003 Andy Jefferson - added comments and description.
    ...
**********************************************************************/
package org.jpox.resource;

import java.io.PrintWriter;
import java.util.Iterator;
import java.util.Set;

import javax.resource.ResourceException;
import javax.resource.spi.ConnectionManager;
import javax.resource.spi.ConnectionRequestInfo;
import javax.resource.spi.ManagedConnection;
import javax.resource.spi.ManagedConnectionFactory;
import javax.resource.spi.security.PasswordCredential;
import javax.security.auth.Subject;

import org.jpox.AbstractPersistenceManagerFactory;

/**
 * Implementation of the JCA adapter for use of JPOX in J2EE environments.
 * Provides an implementation of the methods for ManagedConnectionFactory as
 * well as setters/getters for properties required in the J2EE environment.
 *
 * @version $Revision: 1.1 $
 */
public class ManagedConnectionFactoryImpl extends AbstractPersistenceManagerFactory
    implements ManagedConnectionFactory
{
    public ManagedConnectionFactoryImpl() 
    {
    }

    // ----------------------- Setters/Getters for JCA -------------------------
 
    // --------------- Implementation of ManagedConnectionFactory --------------
 
    /**
     * Set the user name for the data store connection.
     * @param userName the user name for the data store connection.
     * @deprecated
     */
    public synchronized void setUserName(String userName)
    {
        this.setConnectionUserName(userName);
    }
    
    /**
     * Set the password for the data store connection.
     * @param password the password for the data store connection.
     * @deprecated
     */
    public synchronized void setPassword(String password)
    {
        this.setConnectionPassword(password);
    }
    
    /**
     * Equality operator.
     * @param other The object to compare with
     * @return Whether the objects are equal
     */
    public boolean equals(Object other)
    {
        if (this == other) 
        {
            return true;
        }
      
        if (!(other instanceof ManagedConnectionFactoryImpl)) 
        {
             return false;
        }
        return super.equals(other);
    }

    /**
     * Creator for the connection factory
     * @return The connection factory
     * @exception javax.resource.ResourceException Thrown if an error occurs
     */
    public Object createConnectionFactory()
    throws ResourceException
    {
        throw new ResourceException("NYI");
    }

    /**
     * Creator for the connection factory.
     * @param cm ConnectionManager
     * @return The connection factory.
     * @exception javax.resource.ResourceException Thrown if an error occurs
     */
    public Object createConnectionFactory(ConnectionManager cm) 
    throws ResourceException
    {
        freezeConfiguration();
        return new PersistenceManagerFactoryImpl(this, cm);
    }

    /**
     * Creator for a managed connection.
     * @param subject The subject (what ?)
     * @param cri Connection request info.
     * @return The managed connection.
     * @exception javax.resource.ResourceException Thrown if an error occurs
     */
    public ManagedConnection createManagedConnection(Subject subject, ConnectionRequestInfo cri)
    throws ResourceException
    {
        PasswordCredential pc = getPasswordCredential(subject);
        return new ManagedConnectionImpl(this,pc);
    }

    /**
     * Method to match managed connections.
     * @param mcs <description>
     * @param subject <description>
     * @param cri <description>
     * @return <description>
     * @exception javax.resource.ResourceException Thrown if an error occurs
     */
    public ManagedConnection matchManagedConnections(Set mcs, Subject subject, ConnectionRequestInfo cri) 
    throws ResourceException
    {
        PasswordCredential pc = getPasswordCredential(subject);
        for (Iterator i = mcs.iterator(); i.hasNext();)
        {
            Object o = i.next();
            if (!(o instanceof ManagedConnectionImpl))
            {
                continue;
            }

            ManagedConnectionImpl mc = (ManagedConnectionImpl)o;
            if (!(mc.getManagedConnectionFactory().equals(this)))
            {
                continue;
            }
            if ((pc == null && mc.getPasswordCredential() == null) ||
                pc.equals(mc.getPasswordCredential())) 
            {
                return mc;
            }
        }
        return null;
    }

    /**
     * Accessor for the Log writer
     * @return The Log Writer
     * @exception javax.resource.ResourceException Thrown if an error occurs.
     */
    public PrintWriter getLogWriter()
    throws ResourceException
    {
        return null;
    }

    /**
     * Mutator for the Log Writer
     * @param writer The log writer
     * @exception javax.resource.ResourceException Thrown if an error occurs
     */
    public void setLogWriter(PrintWriter writer)
    throws ResourceException
    {
    }

    /**
     * Accessor for the Password credentials
     * @param subject The subject.
     * @return The password credential
     * @throws javax.resource.ResourceException Thrown if an error occurs.
     */
    PasswordCredential getPasswordCredential(Subject subject) 
    throws ResourceException
    {
        if (subject == null) 
        {
            if (getConnectionUserName() == null ||
                getConnectionPassword() == null)
            {
                return null;
            }
            PasswordCredential pc=new PasswordCredential(getConnectionUserName(), getConnectionPassword().toCharArray());
            pc.setManagedConnectionFactory(this);
            return pc;
        }

        for (Iterator i=subject.getPrivateCredentials().iterator();i.hasNext();)
        {
            Object o = i.next();
            if (o instanceof PasswordCredential) 
            {
                PasswordCredential pc = (PasswordCredential)o;
                if (this.equals(pc.getManagedConnectionFactory())) 
                {
                    return pc;
                }
            }
        }
        throw new ResourceException("No credentials found for ManagedConnectionFactory: " + this);
    }
}
