/**********************************************************************
Copyright (c) 2003 David Jencks and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
2004 Erik Bengtson - method close is delegated to AbstractPersistenceManagerFactory
    ...
**********************************************************************/
package org.jpox.resource;

import java.util.Collection;
import java.util.Properties;

import javax.jdo.DataStoreCache;
import javax.jdo.InstanceLifecycleListener;
import javax.jdo.JDOException;
import javax.jdo.PersistenceManager;
import javax.jdo.PersistenceManagerFactory;
import javax.naming.NamingException;
import javax.naming.Reference;
import javax.resource.ResourceException;
import javax.resource.cci.Connection;
import javax.resource.cci.ConnectionFactory;
import javax.resource.cci.ConnectionSpec;
import javax.resource.cci.RecordFactory;
import javax.resource.cci.ResourceAdapterMetaData;
import javax.resource.spi.ConnectionManager;

import org.jpox.AbstractPersistenceManagerFactory;

/**
 * PersistenceManagerFactoryImpl for J2EE
 * @version $Revision: 1.3 $
 */
public class PersistenceManagerFactoryImpl implements ConnectionFactory, PersistenceManagerFactory
{
    private final ManagedConnectionFactoryImpl mcf;
    private final ConnectionManager cm;

    private Reference ref;

    public PersistenceManagerFactoryImpl(final ManagedConnectionFactoryImpl mcf, final ConnectionManager cm) 
    {
        this.mcf = mcf;
        this.cm = cm;      
    }

    public AbstractPersistenceManagerFactory getAbstractPersistenceManagerFactory()
    {
         return mcf;
    }

   // implementation of javax.resource.Referenceable interface

    /**
     *
     * @param ref <description>
     */
    public void setReference(Reference ref)
    {
        this.ref = ref;
    }
   // implementation of javax.naming.Referenceable interface

    /**
     *
     * @return <description>
     * @exception javax.naming.NamingException <description>
     */
    public Reference getReference() throws NamingException
    {
        return ref;
    }
   // implementation of javax.resource.cci.ConnectionFactory interface

    /**
     *
     * @return <description>
     * @exception javax.resource.ResourceException <description>
     */
    public Connection getConnection() throws ResourceException
    {
        PersistenceManagerImpl pmi = (PersistenceManagerImpl)cm.allocateConnection(mcf, null);
        pmi.setPersistenceManagerFactory(this);
        return pmi;
    }

    /**
     *
     * @param cs <description>
     * @return <description>
     * @exception javax.resource.ResourceException <description>
     */
    public Connection getConnection(ConnectionSpec cs) throws ResourceException
    {
        return getConnection();
    }

    /**
     *
     * @return <description>
     * @exception javax.resource.ResourceException <description>
     */
    public ResourceAdapterMetaData getMetaData() throws ResourceException
    {
        throw new ResourceException("NYI");
    }

    /**
     *
     * @return <description>
     * @exception javax.resource.ResourceException <description>
     */
    public RecordFactory getRecordFactory() throws ResourceException
    {
        return null;
    }

   // implementation of javax.jdo.PersistenceManagerFactory interface

    /**
     *
     * @return <description>
     */
    public Properties getProperties()
    {
        throw new JDOException("Not available in managed environment");
    }

    /**
     *
     * @return <description>
     */
    public PersistenceManager getPersistenceManager()
    {
        try 
        {
            return (PersistenceManager)getConnection();
        }
        catch (ResourceException e)
        {
            throw new JDOException("Problem getting PersistenceManager:", new Exception[] {e});
        }
    }

    /**
     *
     * @param user Username
     * @param pw password
     * @return PersistenceManager
     */
    public PersistenceManager getPersistenceManager(String user, String pw)
    {
        throw new JDOException("Not available in managed environment");
    }

    /**
     * Mutator for the Username 
     *
     * @param username Username
     */
    public void setConnectionUserName(String username)
    {
        throw new JDOException("Not available in managed environment");
    }

    /**
     * Accessor for the username 
     *
     * @return Username
     */
    public String getConnectionUserName()
    {
        throw new JDOException("Not available in managed environment");
    }

    /**
     * Mutator for the Password 
     *
     * @param password Password
     */
    public void setConnectionPassword(String password)
    {
        throw new JDOException("Not available in managed environment");
    }

    /**
     * Mutator for the connection URL 
     *
     * @param url Connection URL for the datasource
     */
    public void setConnectionURL(String url)
    {
        throw new JDOException("Not available in managed environment");
    }

    /**
     * Accessor for the connection URL 
     *
     * @return The connection URL
     */
    public String getConnectionURL()
    {
        throw new JDOException("Not available in managed environment");
    }

    /**
     *
     * @param drivername Driver name
     */
    public void setConnectionDriverName(String drivername)
    {
        throw new JDOException("Not available in managed environment");
    }

    /**
     *
     * @return The connection driver name
     */
    public String getConnectionDriverName()
    {
        throw new JDOException("Not available in managed environment");
    }

    /**
     *
     * @param conn_factory_name <description>
     */
    public void setConnectionFactoryName(String conn_factory_name)
    {
        throw new JDOException("Not available in managed environment");
    }

    /**
     *
     * @return Connection Factory name
     */
    public String getConnectionFactoryName()
    {
        throw new JDOException("Not available in managed environment");
    }

    /**
     *
     * @param conn_factory Connection factory
     */
    public void setConnectionFactory(Object conn_factory)
    {
        throw new JDOException("Not available in managed environment");
    }

    /**
     *
     * @return The connection factory
     */
    public Object getConnectionFactory()
    {
        throw new JDOException("Not available in managed environment");
    }

    /**
     *
     * @param conn_factory_name Connection factory name
     */
    public void setConnectionFactory2Name(String conn_factory_name)
    {
        throw new JDOException("Not available in managed environment");
    }

    /**
     *
     * @return Connection factory name
     */
    public String getConnectionFactory2Name()
    {
        throw new JDOException("Not available in managed environment");
    }

    /**
     *
     * @param conn_factory Connection Factory
     */
    public void setConnectionFactory2(Object conn_factory)
    {
        throw new JDOException("Not available in managed environment");
    }

    /**
     *
     * @return <description>
     */
    public Object getConnectionFactory2()
    {
        throw new JDOException("Not available in managed environment");
    }

    /**
     *
     * @param multithreaded Whether is multithreaded or not
     */
    public void setMultithreaded(boolean multithreaded)
    {
        throw new JDOException("Not available in managed environment");
    }

    /**
     *
     * @return <description>
     */
    public boolean getMultithreaded()
    {
        throw new JDOException("Not available in managed environment");
    }

    /**
     *
     * @param optimistic <description>
     */
    public void setOptimistic(boolean optimistic)
    {
        throw new JDOException("Not available in managed environment");
    }

    /**
     *
     * @return <description>
     */
    public boolean getOptimistic()
    {
        throw new JDOException("Not available in managed environment");
    }

    /**
     *
     * @param retain_values <description>
     */
    public void setRetainValues(boolean retain_values)
    {
        throw new JDOException("Not available in managed environment");
    }

    /**
     *
     * @return <description>
     */
    public boolean getRetainValues()
    {
        throw new JDOException("Not available in managed environment");
    }

    /**
     *
     * @param restore_values <description>
     */
    public void setRestoreValues(boolean restore_values)
    {
        throw new JDOException("Not available in managed environment");
    }

    /**
     *
     * @return <description>
     */
    public boolean getRestoreValues()
    {
        throw new JDOException("Not available in managed environment");
    }

    /**
     *
     * @param nontrans_read <description>
     */
    public void setNontransactionalRead(boolean nontrans_read)
    {
        throw new JDOException("Not available in managed environment");
    }

    /**
     *
     * @return <description>
     */
    public boolean getNontransactionalRead()
    {
        throw new JDOException("Not available in managed environment");
    }

    /**
     *
     * @param nontrans_write <description>
     */
    public void setNontransactionalWrite(boolean nontrans_write)
    {
        throw new JDOException("Not available in managed environment");
    }

    /**
     *
     * @return <description>
     */
    public boolean getNontransactionalWrite()
    {
        throw new JDOException("Not available in managed environment");
    }

    /**
     *
     * @param ignore <description>
     */
    public void setIgnoreCache(boolean ignore)
    {
        throw new JDOException("Not available in managed environment");
    }

    /**
     *
     * @return <description>
     */
    public boolean getIgnoreCache()
    {
        throw new JDOException("Not available in managed environment");
    }


    /**
     * Mutator for whether to detach objects on close of the PM's
     * @param detach Whether to detach objects on close of the PM.
     */
    public void setDetachOnClose(boolean detach)
    {
        throw new JDOException("Not available in managed environment");
    }

    /**
     * Accessor for whether to detach objects on close of the PM's
     * @return Whether to detach objects on close of the PM's
     */
    public boolean getDetachOnClose()
    {
        throw new JDOException("Not available in managed environment");
    }
    
    /**
     * Mutator for the Mapping name
     * @param mapping mapping name
     */
    public void setMapping(String mapping)
    {
        throw new JDOException("Not available in managed environment");
    }
    
    /**
     * Accessor for the mapping name
     * @return mapping name
     */
    public String getMapping()
    {
        throw new JDOException("Not available in managed environment");
    }
   
    /**
     * Accessor for the pool maximum size 
     *
     * @return Pool maximum size
     */
    public int getMaxPool()
    {
        throw new JDOException("Not available in managed environment");
    }

    /**
     * Mutator for the Pool maximum size 
     *
     * @param pool_max Pool maximum size
     */
    public void setMaxPool(int pool_max)
    {
        throw new JDOException("Not available in managed environment");
    }

    /**
     * Accessor for the Pool minimum size 
     *
     * @return Pool minimum size
     */
    public int getMinPool()
    {
        throw new JDOException("Not available in managed environment");
    }

    /**
     * Mutator for the Pool minimum size 
     *
     * @param pool_min Pool minimum size
     */
    public void setMinPool(int pool_min)
    {
        throw new JDOException("Not available in managed environment");
    }

    /**
     * Accessor for the millisecs wait time 
     *
     * @return Wait time in milliseconds
     */
    public int getMsWait()
    {
        throw new JDOException("Not available in managed environment");
    }

    /**
     * Mutator for the milliseconds wait time 
     *
     * @param ms_wait Wait time in milliseconds
     */
    public void setMsWait(int ms_wait)
    {
        throw new JDOException("Not available in managed environment");
    }

    /**
     * Accessor for the supported options 
     *
     * @return Collection of supported options
     */
    public Collection supportedOptions()
    {
        return mcf.supportedOptions();
    }
    
    /**
     * Accessor for the Datastore Cache
     * @return The datastore cache
     */
    public DataStoreCache getDataStoreCache()
    {
        throw new JDOException("Not yet supported");
    }
    
    /**
     * @see javax.jdo.PersistenceManagerFactory#close()
     */
    public void close()
    {
		mcf.close();
    }

    /**
     * Accessor for whether the PMF is closed.
     * @return Whether it is closed.
     */
    public boolean isClosed()
    {
        return mcf.isClosed();
    }

		/**
		 * @see javax.jdo.PersistenceManagerFactory#addLifecycleListener(javax.jdo.InstanceLifecycleListener, java.lang.Class[])
		 */
		public void addLifecycleListener(InstanceLifecycleListener listener, Class[] classes)
		{
			mcf.addLifecycleListener(listener, classes);
		}

		/**
		 * @see javax.jdo.PersistenceManagerFactory#removeLifecycleListener(javax.jdo.InstanceLifecycleListener)
		 */
		public void removeLifecycleListener(InstanceLifecycleListener listener)
		{
			mcf.removeLifecycleListener(listener);
		}
}