/**********************************************************************
Copyright (c) 2003 Mike Martin and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
2003 Andy Jefferson - commented
    ...
**********************************************************************/
package org.jpox.sco;

import org.jpox.StateManager;

/**
 * Representation of a wrapper for a mutable SCO type supported by JPOX.
 *
 * @version $Revision: 1.1 $
 **/
public interface SCO
{
    /**
     * Assigns the value of this second-class object from an existing object
     * of the same or compatible type.
     *
     * @param o     the object from which to copy the value.
     * @throws ClassCastException
     *      if the given object is not of a type that's compatible with this
     *      second-class wrapper object.
     */
    void setValueFrom(Object o) throws ClassCastException;

    /**
     * Returns the field name.
     * 
     * @return field name
     */
    String getFieldName();

    /**
     * Returns the owner object of the SCO instance.
     * 
     * @return owner object
     */
    Object getOwner();

    /**
     * Nullifies references to the owner Object and Field.
     */
    void unsetOwner();

    /**
     * Applies any pending updates for this object to the data store.
     */
    void applyUpdates();

    /**
     * Marks object dirty.
     */
    void makeDirty();

    /**
     * Mutable second class objects are required to provide a public clone
     * method so that copying of PersistenceCapable objects can take place.
     * This mustn't throw a CloneNotSupportedException.
     * @return A clone of the object
     **/
    Object clone();
    
    Object detachCopy(StateManager sm, Object value);

    Object attachCopy(StateManager sm, Object value, boolean makeTransactional);
    
}
