/**********************************************************************
Copyright (c) 2004 Andy Jefferson and others. All rights reserved. 
This program and the accompanying materials are made available under 
the terms of the JPOX License v1.0 which accompanies this distribution. 

Contributors:
    ...
**********************************************************************/
package org.jpox.sco;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import org.jpox.StateManager;
import org.jpox.store.scostore.SetStore;

/**
 * An iterator for a SCO Collection object. Takes in the delegate and the
 * backing store, and provides iteration through the objects.
 * Only accessible from package to provide protection 
 * 
 * @version $Revision: 1.3 $
 */
class SCOCollectionIterator implements Iterator
{
    private final Iterator iter;
    private Object last = null;

    private Collection ownerSCO;

    /**
     * Constructor taking the delegate, backing store.
     * @param sm State Manager of SCO Collection to iterate
     * @param theDelegate The delegate collection
     * @param theStore The backing store (connected to the DB)
     * @param useDelegate Whether to use the delegate
     */
    public SCOCollectionIterator(Collection sco,
                                 StateManager sm,
                                 Collection theDelegate,
                                 SetStore theStore,
                                 boolean useDelegate)
    {
        ownerSCO = sco;

        // Populate our entries list
        ArrayList entries = new ArrayList();

        Iterator i=null; 
        if (useDelegate)
        {
            i = theDelegate.iterator();
        }
        else
        {
            if (theStore != null)
            {
                i = theStore.iterator(sm);
            }
            else
            {
                i = theDelegate.iterator();
            }
        }

        while (i.hasNext())
        {
            entries.add(i.next());
        }

        iter = entries.iterator();
    }

    public boolean hasNext()
    {
        return iter.hasNext();
    }

    public Object next()
    {
        return last = iter.next();
    }

    public void remove()
    {
        if (last == null)
        {
            throw new IllegalStateException();
        }

        ownerSCO.remove(last);
        last = null;
    }
}
