/**********************************************************************
Copyright (c) 2004 Andy Jefferson and others. All rights reserved. 
This program and the accompanying materials are made available under 
the terms of the JPOX License v1.0 which accompanies this distribution. 

Contributors:
    ...
**********************************************************************/
package org.jpox.sco;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;

import org.jpox.StateManager;
import org.jpox.store.scostore.ListStore;

/**
 * An iterator for a SCO List object. Takes in the delegate and the backing
 * store, and provides iteration through the objects.
 * Only accessible from package to provide protection.
 *
 * @version $Revision: 1.3 $
 */
class SCOListIterator implements ListIterator
{
    private final ListIterator iter;
    private Object last=null;
    private int currentIndex=-1;

    private List ownerSCO;

    /**
     * Constructor taking the delegate, backing store and any start index.
     * @param sco Owner SCO 
     * @param sm State Manager of SCO List to iterate 
     * @param theDelegate The delegate list
     * @param theStore The backing store (connected to the DB)
     * @param startIndex The start index position
     */
    public SCOListIterator(List sco,
                           StateManager sm,
                           List theDelegate,
                           ListStore theStore,
                           boolean useDelegate,
                           int startIndex)
    {
        ownerSCO = sco;

        // Populate our entries list
        ArrayList entries = new ArrayList();

        Iterator i=null;
        if (useDelegate)
        {
            i = theDelegate.iterator();
        }
        else
        {
            if (theStore != null)
            {
                i = theStore.iterator(sm);
            }
            else
            {
                i = theDelegate.iterator();
            }
        }
        while (i.hasNext())
        {
            entries.add(i.next());
        }

        iter = entries.listIterator();
        currentIndex = startIndex;
    }

    public void add(Object o)
    {
        currentIndex = iter.nextIndex();

        ownerSCO.add(currentIndex, o);
        last = null;
    }

    public boolean hasNext()
    {
        return iter.hasNext();
    }

    public boolean hasPrevious()
    {
        return iter.hasPrevious();
    }

    public Object next()
    {
        currentIndex = iter.nextIndex();
        return last = iter.next();
    }

    public int nextIndex()
    {
        return iter.nextIndex();
    }

    public Object previous()
    {
        currentIndex = iter.previousIndex();
        return last = iter.previous();
    }

    public int previousIndex()
    {
        return iter.previousIndex();
    }

    public void remove()
    {
        if (last == null)
        {
            throw new IllegalStateException();
        }

        ownerSCO.remove(currentIndex);
        last = null;
        currentIndex = -1;
    }

    public synchronized void set(Object o)
    {
        if (last == null)
        {
            throw new IllegalStateException("No entry to replace");
        }

        ownerSCO.set(currentIndex, o);
        iter.set(o);

        last = o;
    }
}