/**********************************************************************
Copyright (c) 2004 Andy Jefferson and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
    ...
**********************************************************************/
package org.jpox.sco;

import org.jpox.StateManager;
import org.jpox.metadata.ContainerMetaData;
import org.jpox.util.JPOXLogger;
import org.jpox.util.Localiser;

/**
 * Collection of utilities for second class objects.
 * 
 * @version $Revision: 1.1 $
 */
public class SCOUtils
{
    private static final Localiser LOCALISER=Localiser.getInstance("org.jpox.sco.Localisation");

    /**
     * Utility to return whether or not to use the container cache for the
     * collection/map for the passed StateManager SCO.
     * @param ownerSM The StateManager for the SCO field
     * @param fieldName Name of the field.
     * @return Whether to use the cache.
     */
    public static boolean useContainerCache(StateManager ownerSM, String fieldName)
    {
        if (ownerSM == null)
        {
            return false;
        }

        // Get global value for PMF
        boolean useCache = ownerSM.getPersistenceManager().getPMFContext().getPmfConfiguration().getJdoCacheCollections();

        // Check if this container has a MetaData value defined
        ContainerMetaData conmd=ownerSM.getMetaDataManager().getMetaDataForField(ownerSM.getObject().getClass(),fieldName).getContainer();
        if (conmd != null && conmd.hasExtension("cache"))
        {
            useCache = (new Boolean(conmd.getValueForExtension("cache"))).booleanValue();
        }
        if (JPOXLogger.JDO.isDebugEnabled())
        {
            JPOXLogger.JDO.debug(LOCALISER.msg("SCO.Cache.Status",fieldName,ownerSM.getObject().getClass().getName(),"" + useCache));
        }

        return useCache;
    }
}
