/**********************************************************************
Copyright (c) 2002 Mike Martin (TJDO) and others. All rights reserved. 
This program and the accompanying materials are made available under 
the terms of the JPOX License v1.0 which accompanies this distribution. 

Contributors:
2003 Andy Jefferson - commented
    ...
**********************************************************************/
package org.jpox.sco.exceptions;

import org.jpox.StateManager;
import org.jpox.util.Localiser;

/**
 * A <tt>IncompatibleTypeException</tt> is thrown if an incompatible type
 * is specified as a parameter in the construction of a second-class object
 * instance.
 *
 * @version $Revision: 1.1 $
 */
public class IncompatibleTypeException extends SCOException
{
    private static final Localiser LOCALISER=Localiser.getInstance("org.jpox.sco.Localisation");

    /**
     * Constructs an incompatible type exception.
     *
     * @param ownerSM The StateManager of the object owning the SCO field.
     * @param fieldName The name of the SCO field.
     * @param parameterName The name of the invalid type parameter.
     * @param requiredTypeName Name of required type.
     * @param requestedTypeName Name of requested type.
     */
    public IncompatibleTypeException(StateManager ownerSM,
                                     String fieldName,
                                     String parameterName,
                                     String requiredTypeName,
                                     String requestedTypeName)
    {
        super(ownerSM,fieldName,LOCALISER.msg("Exception.IncompatibleType",parameterName,requestedTypeName,requiredTypeName));
    }
}
