/**********************************************************************
Copyright (c) 2004 Erik Bengtson and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
    ...
**********************************************************************/
package org.jpox.state;

/**
 * Activity states
 *
 * @since 1.1 
 * @version $Revision: 1.1 $
 */
public class ActivityState
{
    public static final ActivityState INSERTING = new ActivityState(1);

    public static final ActivityState INSERTING_CALLBACKS = new ActivityState(2);

    public static final ActivityState INSERTING_UPDATE = new ActivityState(3);

    public static final ActivityState NONE = new ActivityState(4);

    /**
     * The type id
     */
    private final int typeId;

    /**
     * constructor
     * @param i type id
     */
    private ActivityState(int i)
    {
        this.typeId = i;
    }

    /**
     * Indicates whether some other object is "equal to" this one.
     * @param o the reference object with which to compare. 
     * @return true if this object is the same as the obj argument; false otherwise.
     */
    public boolean equals(Object o)
    {
        if (o instanceof ActivityState)
        {
            return ((ActivityState)o).typeId == typeId;
        }
        return false;
    }

    /**
     * Returns a string representation of the object.
     * @return a string representation of the object.
     */
    public String toString()
    {
        switch (typeId)
        {
            case 1 :
                return "inserting";
            case 2 :
                return "inserting-callback";
            case 3 :
                return "inserting-update";
            case 4 :
                return "none";
        }
        return "";
    }

    /**
     * Accessor for the type.
     * @return Type
     **/
    public int getType()
    {
        return typeId;
    }

    public static ActivityState getActivityState(final String value)
    {
        if (value == null)
        {
            return ActivityState.NONE;
        }
        else if (ActivityState.INSERTING.toString().equals(value))
        {
            return ActivityState.INSERTING;
        }
        else if (ActivityState.INSERTING_CALLBACKS.toString().equals(value))
        {
            return ActivityState.INSERTING_CALLBACKS;
        }
        else if (ActivityState.INSERTING_UPDATE.toString().equals(value))
        {
            return ActivityState.INSERTING_UPDATE;
        }
        else if (ActivityState.NONE.toString().equals(value))
        {
            return ActivityState.NONE;
        }
        return ActivityState.NONE;
    }
}
