/**********************************************************************
 Copyright (c) 2004 Erik Bengtson and others.
 All rights reserved. This program and the accompanying materials
 are made available under the terms of the JPOX License v1.0
 which accompanies this distribution. 

 Contributors:
 ...
 **********************************************************************/
package org.jpox.state;

import javax.jdo.spi.PersistenceCapable;

import org.jpox.StateManager;
import org.jpox.sco.SCO;
import org.jpox.store.fieldmanager.AbstractFieldManager;

/**
 * Utility class to handle the detach of fields.
 *
 * @version $Revision: 1.2 $
 **/
public class DetachFieldManager extends AbstractFieldManager
{
    private final StateManager smSource;

    private final boolean[] secondClassMutableFields;

    public DetachFieldManager(StateManager smSource, boolean[] secondClassMutableFields)
    {
        this.smSource = smSource;
        this.secondClassMutableFields = secondClassMutableFields;
    }

    public Object fetchObjectField(int fieldNumber)
    {
        SingleFieldValue sfv = new SingleFieldValue();
        smSource.provideFields(new int[]{fieldNumber}, sfv);
        Object value = sfv.fetchObjectField(fieldNumber);

        if (value == null)
        {
            return null;
        }
        else if (secondClassMutableFields[fieldNumber])
        {
            return ((SCO) smSource.newSCOEmptyInstance(fieldNumber)).detachCopy(smSource, value);
        }
        else if (value instanceof PersistenceCapable)
        {
            PersistenceCapable pc = (PersistenceCapable) value;
            if (pc.jdoIsDetached())
            {
                return pc;
            }
            return pc.jdoGetPersistenceManager().detachCopy(pc);
        }
        else
        {
            return value;
        }
    }

    /* (non-Javadoc)
     * @see javax.jdo.spi.PersistenceCapable.ObjectIdFieldSupplier#fetchBooleanField(int)
     */
    public boolean fetchBooleanField(int fieldNumber)
    {
        SingleFieldValue sfv = new SingleFieldValue();
        smSource.provideFields(new int[]{fieldNumber}, sfv);
        return sfv.fetchBooleanField(fieldNumber);
    }

    /* (non-Javadoc)
     * @see javax.jdo.spi.PersistenceCapable.ObjectIdFieldSupplier#fetchByteField(int)
     */
    public byte fetchByteField(int fieldNumber)
    {
        SingleFieldValue sfv = new SingleFieldValue();
        smSource.provideFields(new int[]{fieldNumber}, sfv);
        return sfv.fetchByteField(fieldNumber);
    }

    /* (non-Javadoc)
     * @see javax.jdo.spi.PersistenceCapable.ObjectIdFieldSupplier#fetchCharField(int)
     */
    public char fetchCharField(int fieldNumber)
    {
        SingleFieldValue sfv = new SingleFieldValue();
        smSource.provideFields(new int[]{fieldNumber}, sfv);
        return sfv.fetchCharField(fieldNumber);
    }

    /* (non-Javadoc)
     * @see javax.jdo.spi.PersistenceCapable.ObjectIdFieldSupplier#fetchDoubleField(int)
     */
    public double fetchDoubleField(int fieldNumber)
    {
        SingleFieldValue sfv = new SingleFieldValue();
        smSource.provideFields(new int[]{fieldNumber}, sfv);
        return sfv.fetchDoubleField(fieldNumber);
    }

    /* (non-Javadoc)
     * @see javax.jdo.spi.PersistenceCapable.ObjectIdFieldSupplier#fetchFloatField(int)
     */
    public float fetchFloatField(int fieldNumber)
    {
        SingleFieldValue sfv = new SingleFieldValue();
        smSource.provideFields(new int[]{fieldNumber}, sfv);
        return sfv.fetchFloatField(fieldNumber);
    }

    /* (non-Javadoc)
     * @see javax.jdo.spi.PersistenceCapable.ObjectIdFieldSupplier#fetchIntField(int)
     */
    public int fetchIntField(int fieldNumber)
    {
        SingleFieldValue sfv = new SingleFieldValue();
        smSource.provideFields(new int[]{fieldNumber}, sfv);
        return sfv.fetchIntField(fieldNumber);
    }

    /* (non-Javadoc)
     * @see javax.jdo.spi.PersistenceCapable.ObjectIdFieldSupplier#fetchLongField(int)
     */
    public long fetchLongField(int fieldNumber)
    {
        SingleFieldValue sfv = new SingleFieldValue();
        smSource.provideFields(new int[]{fieldNumber}, sfv);
        return sfv.fetchLongField(fieldNumber);
    }

    /* (non-Javadoc)
     * @see javax.jdo.spi.PersistenceCapable.ObjectIdFieldSupplier#fetchShortField(int)
     */
    public short fetchShortField(int fieldNumber)
    {
        SingleFieldValue sfv = new SingleFieldValue();
        smSource.provideFields(new int[]{fieldNumber}, sfv);
        return sfv.fetchShortField(fieldNumber);
    }

    /* (non-Javadoc)
     * @see javax.jdo.spi.PersistenceCapable.ObjectIdFieldSupplier#fetchStringField(int)
     */
    public String fetchStringField(int fieldNumber)
    {
        SingleFieldValue sfv = new SingleFieldValue();
        smSource.provideFields(new int[]{fieldNumber}, sfv);
        return sfv.fetchStringField(fieldNumber);
    }
}