/**********************************************************************
Copyright (c) 2002 Kelly Grizzle and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
2002 Mike Martin - unknown changes
2003 Andy Jefferson - commented
2003 Andy Jefferson - added localiser
    ...
**********************************************************************/
package org.jpox.state;

import javax.jdo.JDOUserException;
import javax.jdo.Transaction;

/**
 * Class representing the life cycle state of PersistentDirty
 *
 * @version $Revision: 1.1 $
 **/
class PersistentDirty extends LifeCycleState
{
    protected PersistentDirty()
    {
        isPersistent = true;        
        isDirty = true;
        isNew = false;
        isDeleted = false;
        isTransactional = true;

        stateType =  P_DIRTY;
    }

    /**
     * Method to transition to delete-persistent
     * @param sm StateManager.
     * @return new LifeCycle state.
     **/
    public LifeCycleState transitionDeletePersistent(StateManagerImpl sm)
    {
        return changeState(sm, P_DELETED);
    }

    /**
     * Method to transition to nontransactional.
     * @param sm StateManager.
     * @return new LifeCycle state.
     **/
    public LifeCycleState transitionMakeNontransactional(StateManagerImpl sm)
    {
        throw new JDOUserException(LOCALISER.msg("LifeCycle.PersistentDirtyToNontransactional"),sm.getInternalObjectId());
    }

    /**
     * Method to transition to transient.
     * @param sm StateManager.
     * @return new LifeCycle state.
     **/
    public LifeCycleState transitionMakeTransient(StateManagerImpl sm)
    {
        throw new JDOUserException(LOCALISER.msg("LifeCycle.PersistentDirtyToTransient"),sm.getInternalObjectId());
    }

    /**
     * Method to transition to commit state.
     * @param sm StateManager.
     * @param tx the Transaction been committed.
     * @return new LifeCycle state.
     **/
    public LifeCycleState transitionCommit(StateManagerImpl sm, Transaction tx)
    {
        sm.clearSavedFields();

        if (tx.getRetainValues())
        {
            return changeState(sm, P_NONTRANS);
        }
        else
        {
            sm.clearNonPrimaryKeyFields();
            return changeState(sm, HOLLOW);
        }
    }

    /**
     * Method to transition to rollback state.
     * @param sm StateManager.
     * @return new LifeCycle state.
     **/
    public LifeCycleState transitionRollback(StateManagerImpl sm)
    {
        Transaction tx = sm.getPersistenceManager().currentTransaction();
        if (tx.getRestoreValues())
        {
            sm.restoreFields();
            return changeState(sm, P_NONTRANS);
        }
        else
        {
            sm.clearNonPrimaryKeyFields();
            sm.restoreFields();
            sm.clearSavedFields();
            return changeState(sm, HOLLOW);
        }
    }

    /**
     * Method to transition to refresh state.
     * @param sm StateManager.
     * @return new LifeCycle state.
     **/
    public LifeCycleState transitionRefresh(StateManagerImpl sm)
    {
        sm.clearSavedFields();
        sm.refreshLoadedFields();
        Transaction tx = sm.getPersistenceManager().currentTransaction();
        if (tx.isActive() && !tx.getOptimistic())
        {
            return changeState(sm,P_CLEAN);
        }
        return changeState(sm,P_NONTRANS);      
    }

    /**
     * Method to return a string version of this object.
     * @return The string "P_DIRTY".
     **/
    public String toString()
    {
        return "P_DIRTY";
    }
}
