/**********************************************************************
Copyright (c) 2002 Kelly Grizzle and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
2002 Mike Martin - unknown changes
2003 Andy Jefferson - commented
2003 Andy Jefferson - added localiser
    ...
**********************************************************************/
package org.jpox.state;

import javax.jdo.JDOUserException;
import javax.jdo.Transaction;

/**
 * Class representing the life cycle state of PersistentNewDeleted.
 *
 * @version $Revision: 1.1 $
 **/
class PersistentNewDeleted extends LifeCycleState
{
    /** Protected Constructor to prevent external instantiation. */
    protected PersistentNewDeleted()
    {
		isPersistent = true;    	
        isDirty = true;
        isNew = true;
        isDeleted = true;
        isTransactional = true;

        stateType = P_NEW_DELETED;
    }

    /**
     * Method to transition to nontransactional.
     * @param sm StateManager.
     * @return new LifeCycle state.
     **/
    public LifeCycleState transitionMakeNontransactional(StateManagerImpl sm)
    {
        throw new JDOUserException(LOCALISER.msg("LifeCycle.PersistentNewDeletedToNontransactional"),sm.getInternalObjectId());
    }

    /**
     * Method to transition to transient.
     * @param sm StateManager.
     * @return new LifeCycle state.
     **/
    public LifeCycleState transitionMakeTransient(StateManagerImpl sm)
    {
        throw new JDOUserException(LOCALISER.msg("LifeCycle.PersistentNewDeletedToTransient"),sm.getInternalObjectId());
    }

    /**
     * Method to transition to commit state.
     * @param sm StateManager.
     * @param tx the Transaction been committed.
     * @return new LifeCycle state.
     **/
    public LifeCycleState transitionCommit(StateManagerImpl sm, Transaction tx)
    {
        sm.clearFields();
        return changeState(sm, TRANSIENT);
    }

    /**
     * Method to transition to rollback state.
     * @param sm StateManager.
     * @return new LifeCycle state.
     **/
    public LifeCycleState transitionRollback(StateManagerImpl sm)
    {
        Transaction tx = sm.getPersistenceManager().currentTransaction();
        if (tx.getRestoreValues())
        {
            sm.restoreFields();
        }

        return changeState(sm, TRANSIENT);
    }

    /**
     * Method to transition to read-field state.
     * @param sm StateManager.
     * @param isLoaded if the field was previously loaded. 
     * @return new LifeCycle state.
     **/
    public LifeCycleState transitionReadField(StateManagerImpl sm, boolean isLoaded)
    {
        throw new JDOUserException(LOCALISER.msg("LifeCycle.PersistentNewDeletedToReadField"),sm.getInternalObjectId());
    }

    /**
     * Method to transition to write-field state.
     * @param sm StateManager.
     * @return new LifeCycle state.
     **/
    public LifeCycleState transitionWriteField(StateManagerImpl sm)
    {
        throw new JDOUserException(LOCALISER.msg("LifeCycle.PersistentNewDeletedToWriteField"),sm.getInternalObjectId());
    }

    /**
     * Method to return a string version of this object.
     * @return The string "P_NEW_DELETED".
     **/
    public String toString()
    {
        return "P_NEW_DELETED";
    }
}
