/**********************************************************************
Copyright (c) 2004 Erik Bengtson and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
    ...
**********************************************************************/
package org.jpox.state;

import javax.jdo.Transaction;

/**
 * This class represents TransientClean state specific state transitions as 
 * requested by StateManagerImpl. This state is the result of a call to
 * makeTransactional on a Transient instance, or commit or rollback of a
 * TransientDirty instance.
 *
 * @version $Revision: 1.1 $ 
 */
class TransientClean extends LifeCycleState
{
    /**
     * Constructor.
     **/
    TransientClean()
    {
        // these flags are set only in the constructor 
        // and shouldn't be changed afterwards
        // (cannot make them final since they are declared in superclass 
        // but their values are specific to subclasses)
        isPersistent = false;
        isTransactional = true;
        isDirty = false;
        isNew = false;
        isDeleted = false;

        stateType = T_CLEAN;
    }

    /**
     * @param sm The StateManager 
     * @see LifeCycleState#transitionMakeTransient(StateManagerImpl sm)
     */
    public LifeCycleState transitionMakeTransient(StateManagerImpl sm)
    {
        return this;
    }

    /**
     * @param sm The StateManager 
     * @see LifeCycleState#transitionMakeNontransactional(StateManagerImpl sm)
     */
    public LifeCycleState transitionMakeNontransactional(StateManagerImpl sm)
    {  
        sm.disconnect();
        return changeTransientState(sm,TRANSIENT);
    }

    /**
     * @param sm The StateManager 
     * @see LifeCycleState#transitionMakePersistent(StateManagerImpl sm)
     */
    public LifeCycleState transitionMakePersistent(StateManagerImpl sm)
    {    
        sm.registerTransactional();
        return changeState(sm,P_NEW);
    }
    
    /**
     * @param sm The StateManager
     * @param isLoaded if the field was previously loaded.
     * @see LifeCycleState#transitionReadField(StateManagerImpl sm, boolean isLoaded)
     */
    public LifeCycleState transitionReadField(StateManagerImpl sm, boolean isLoaded)
    {
        return this;
    }

    /**
     * @param sm The StateManager
     * @see LifeCycleState#transitionWriteField(StateManagerImpl sm)
     */
    public LifeCycleState transitionWriteField(StateManagerImpl sm)
    {
        Transaction tx = sm.getPersistenceManager().currentTransaction();
        if (tx.isActive())
        {
            sm.saveFields();
            return changeTransientState(sm,T_DIRTY);
        }
        else
        {
            return this;
        }
    }

    /**
     * Method to transition to commit state.
     * This is a no-op.
     * @param sm StateManager.
     * @param tx the Transaction been committed.
     * @return new LifeCycle state.
     **/
    public LifeCycleState transitionCommit(StateManagerImpl sm, Transaction tx)
    {
        return this;
    }

    /**
     * @param sm The StateManager
     * @see LifeCycleState#transitionRollback(StateManagerImpl sm)
     */
    public LifeCycleState transitionRollback(StateManagerImpl sm)
    {
        return this;
    }
}
