/**********************************************************************
Copyright (c) 2004 Erik Bengtson and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
    ...
**********************************************************************/
package org.jpox.state;

import javax.jdo.Transaction;

/**
 * This class represents TransientDirty state specific state transitions as
 * requested by StateManagerImpl. This state is the result of a wrie operation
 * on a TransientClean instance
 *
 * @version $Revision: 1.1 $ 
 */
class TransientDirty extends LifeCycleState
{
    /**
     * Constructor
     **/
    TransientDirty()
    {
        // these flags are set only in the constructor 
        // and shouldn't be changed afterwards
        // (cannot make them final since they are declared in superclass 
        // but their values are specific to subclasses)
        isPersistent = false;
        isTransactional = true;
        isDirty = true;
        isNew = false;
        isDeleted = false;

        stateType =  T_DIRTY;
    }

    /**
     * @param sm The StateManager 
     * @see LifeCycleState#transitionMakeTransient(StateManagerImpl sm)
     */
    public LifeCycleState transitionMakeTransient(StateManagerImpl sm)
    {
        return this;
    }
 
    /**
     * @param sm The StateManager 
     * @see LifeCycleState#transitionMakePersistent(StateManagerImpl sm)
     */
    public LifeCycleState transitionMakePersistent(StateManagerImpl sm)
    {
        sm.registerTransactional();
        return changeState(sm,P_NEW);
    }

    /**
     * Method to transition to commit state.
     * @param sm StateManager.
     * @param tx the Transaction been committed.
     * @return new LifeCycle state.
     **/
    public LifeCycleState transitionCommit(StateManagerImpl sm, Transaction tx)
    {
    	sm.clearSavedFields();
        return changeTransientState(sm,T_CLEAN);
    }
 
    /**
     * @param sm The StateManager 
     * @see LifeCycleState#transitionRollback(StateManagerImpl sm)
     */
    public LifeCycleState transitionRollback(StateManagerImpl sm)
    {
        Transaction tx = sm.getPersistenceManager().currentTransaction();
        if (tx.getRestoreValues() || sm.isRestoreValues())
        {
            sm.restoreFields();
        } // else do nothing.
        return changeTransientState(sm,T_CLEAN); 
    }
}
