/**********************************************************************
Copyright (c) 2003 Andy Jefferson and others. All rights reserved. 
This program and the accompanying materials are made available under 
the terms of the JPOX License v1.0 which accompanies this distribution.

Contributors:
2004 Andy Jefferson - changed to use StoreData instead of AutoStartData
2004 Erik Bengtson - added open()/close()/isOpen()
    ...
**********************************************************************/
package org.jpox.store;

import java.util.Collection;

import org.jpox.store.exceptions.DatastoreInitialisationException;

/**
 * Interface defining an Auto-Start Mechanism.
 * An Auto-Start Mechanism is a means of auto-populating the classes supported
 * by a StoreManager.
 *
 * @version $Revision: 1.1 $
 **/
public interface AutoStartMechanism
{
    public static final String NONE = "None";
    
    public static final String MODE_QUIET = "Quiet";
    public static final String MODE_CHECKED = "Checked";
    public static final String MODE_IGNORED = "Ignored";
    
    /**
     * Accessor for the mode of operation.
     * @return The mode of operation
     **/
    String getMode();

    /**
     * Mutator for the mode of operation.
     * @param mode The mode of operation
     **/
    void setMode(String mode);

    /**
     * Accessor for the data for the classes that are supported.
     * @return Collection of class data (AutoStartData).
     **/
    Collection getAllClassData() throws DatastoreInitialisationException;

    /**
     * Starts a transaction for writing (add/delete) classes to the auto start
     * mechanism 
     */
    void open();
    
    /**
     * Closes a transaction for writing (add/delete) classes to the auto start
     * mechanism 
     */
    void close();
    
    /**
     * Whether it's open for writing (add/delete) classes to the auto start
     * mechanism
     * @return whether this is open for writting 
     */
    public boolean isOpen();
    
    /**
     * Method to add a class/field (with its data) to the currently-supported list.
     * @param data The data for the class.
     **/
    void addClass(StoreData data);

    /**
     * Method to delete a class/field that is currently listed as supported in
     * the internal storage.
     * TODO Rename this method to allow for deleting fields
     * @param name The name of the class/field
     **/
    void deleteClass(String name);

    /**
     * Method to delete all classes that are currently listed as supported in
     * the internal storage.
     **/
    void deleteAllClasses();

    /**
     * Utility to return a description of the storage for this mechanism.
     * @return The storage description.
     **/
    String getStorageDescription();
}