/**********************************************************************
Copyright (c) 2005 Erik Bengtson and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
    ...
**********************************************************************/
package org.jpox.store;

import org.jpox.metadata.DiscriminatorMetaData;
import org.jpox.metadata.ExtendableMetaData;
import org.jpox.store.mapping.JavaTypeMapping;

/**
 * This represents a container of fields. Such as table (container of columns), file, etc.
 * @version $Revision: 1.3 $
 */
public interface DatastoreContainerObject extends DatastoreObject
{
    /**
     * Method to add a new column to the internal representation.
     * @param type The type of the Java field to store
     * @param name The name of the column
     * @param role The role of the column (e.g FK?)
     * @param mapping The type mapping for this column
     * @param colmd The ExtendableMetaData (ColumnMetaData for RDBMS)
     * @return The new Column
     */
    DatastoreField addDatastoreField(Class type, DatastoreIdentifier name, Role role, JavaTypeMapping mapping, ExtendableMetaData colmd);

    /**
     * Accessor for Discriminator MetaData
     * @return Returns the Discriminator MetaData.
     */
    DiscriminatorMetaData getDiscriminatorMetaData();
    
    /**
     * Accessor for the discriminator mapping specified .
     * @return The mapping for the discriminator datastore field
     **/
    JavaTypeMapping getDiscriminatorMapping();    
}
