/**********************************************************************
Copyright (c) 2004 Andy Jefferson and others. All rights reserved. 
This program and the accompanying materials are made available under 
the terms of the JPOX License v1.0 which accompanies this distribution. 

Contributors:
    ...
**********************************************************************/
package org.jpox.store;

import org.jpox.store.mapping.JavaTypeMapping;

/**
 * Representation of a Java field in a datastore.
 * In the case of RDBMS this will be a column.
 * In the case of a file-based structure this may be a file.
 * In the case of an XML-based structure this may be an node. 
 *
 * @version $Revision: 1.4 $
 **/ 
public interface DatastoreField extends DatastoreObject
{
    /**
     * Accessor for the type of data stored in this field.
     * @return The type of data in the field.
     **/
    Class getType();

    /**
     * Mutator to make the field the primary key.
     **/
    void setAsPrimaryKey();

    /**
     * Accessor for whether the field is the primary key
     * @return whether the field is (part of) the primary key
     **/
    boolean isPrimaryKey();

    /**
     * Accessor for whether the field is nullable
     * @return whether the field is nullable
     **/
    boolean isNullable();

    /**
     * Accessor for the Mapping for this field. 
     * @return The Mapping
     */
    JavaTypeMapping getMapping();
    
    /**
     * Accessor for the DatastoreContainerObject container of this field
     * @return The DatastoreContainerObject
     */
    DatastoreContainerObject getDatastoreContainerObject();
    
    /**
     * Wraps the column name with a FUNCTION.
     * <PRE>example: SQRT(?) generates: SQRT(columnName)</PRE>
     * @param replacementValue the replacement to ?. Probably it's a column name, that may be fully qualified name or not
     * @return a String with function taking as parameter the replacementValue
     */
    String applySelectFunction(String replacementValue);

    /**
     * Copy the configuration of this field to another field
     * @param col the datastore field
     */
	void copyConfigurationTo(DatastoreField col);
	
    /**
     * Mutator for the nullability of the datastore field.
     *
     * @return The datastore field with the updated info
     **/
    DatastoreField setNullable();
    
    /**
     * Mutator for the defaultability of the datastore field.
     *
     * @return The datastore field with the updated info
     **/
    DatastoreField setDefaultable();    

}
