/**********************************************************************
 Copyright (c) 2005 Andy Jefferson and others. All rights reserved.
 This program and the accompanying materials are made available under 
 the terms of the JPOX License v1.0 which accompanies this distribution.

 Contributors:
 ...
 **********************************************************************/
package org.jpox.store;

/**
 * Representation of a statement to fetch a particular object.
 * @version $Revision: 1.7 $
 */
public interface FetchStatement
{
    /**
     * Add a column to the SELECT clause.
     * @param col The column to add
     * @return The parameter position of this column in the statement.
     */
    public abstract int select(DatastoreField col);

    /**
     * Adds an expression to the SELECT clause.
     * @param expr the expr to add to the select clause
     * @return The parameter position of this expression in the statement.
     */
    public abstract int select(String expr);

    /**
     * Method to specify a column to be referenced.
     * @param col The column
     * @return The column statement text
     */
    public abstract String referenceColumn(DatastoreField col);

    /**
     * Method to add an extra condition to the WHERE clause.
     * @param condition The condition
     */
    public abstract void andCondition(String condition);

    /**
     * Method to return the statement assuming no locking of rows.
     * @return The statement
     */
    public abstract String toString();

    /**
     * Method to return the statement.
     * @param lock Whether to lock the rows found by this SELECT
     * @return The statement
     **/
    public abstract String toString(boolean lock);
}