/**********************************************************************
Copyright (c) 2002 Kelly Grizzle (TJDO) and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
2002 Mike Martin (TJDO)
2003 Erik Bengtson - added custom for user specified
2003 Andy Jefferson - coding standards
    ...
**********************************************************************/
package org.jpox.store;


/**
 * Representation of a role for a Column. It represents the type of column we
 * are handling ... key, index, owner, element, etc. It also provides a
 * "custom" option which is for use where the user has overridden the name for
 * example.
 *
 * @version $Revision: 1.2 $
 **/
public class Role
{
    private static final int ROLE_TYPE_CUSTOM   = -1;    
    private static final int ROLE_TYPE_NONE    = 0;
    private static final int ROLE_TYPE_OWNER   = 1;
    private static final int ROLE_TYPE_ELEMENT = 2;
    private static final int ROLE_TYPE_KEY     = 3;
    private static final int ROLE_TYPE_VALUE   = 4;
    private static final int ROLE_TYPE_INDEX   = 5;    

    public static final Role CUSTOM    = new Role(ROLE_TYPE_CUSTOM);
    public static final Role NONE    = new Role(ROLE_TYPE_NONE);
    public static final Role OWNER   = new Role(ROLE_TYPE_OWNER);
    public static final Role ELEMENT = new Role(ROLE_TYPE_ELEMENT);
    public static final Role KEY     = new Role(ROLE_TYPE_KEY);
    public static final Role VALUE   = new Role(ROLE_TYPE_VALUE);
    public static final Role INDEX   = new Role(ROLE_TYPE_INDEX);
    
    public static final int MAX_SUFFIX_LENGTH = 4;

    private final int type;

    /**
     * Constructor - private to prevent instantiation.
     * @param type The type of role.
     **/
    private Role(int type)
    {
        this.type = type;
    }

    /**
     * Accessor for the suffix to add to any SQL identifier, based on the
     * role type.
     * @param dba The database adapter
     * @param columnType The type of data in the column
     * @return The suffix (e.g _ID for id columns).
     **/
    public String getIdentifierSuffix(DatastoreAdapter dba, Class columnType)
    {
        boolean isIDType = !dba.isEmbeddedType(columnType);

        String suffix;
        
        switch (type)
        {
            case ROLE_TYPE_NONE:
            default:
                suffix = isIDType ? "_ID" : "";
                break;

            case ROLE_TYPE_CUSTOM:
                suffix = "";
                break;          

            case ROLE_TYPE_OWNER:
                suffix = isIDType ? "_OID" : "_OWN";
                break;

            case ROLE_TYPE_ELEMENT:
                suffix = isIDType ? "_EID" : "_ELE";
                break;

            case ROLE_TYPE_KEY:
                suffix = isIDType ? "_KID" : "_KEY";
                break;

            case ROLE_TYPE_VALUE:
                suffix = isIDType ? "_VID" : "_VAL";
                break;

            case ROLE_TYPE_INDEX:
                suffix = isIDType ? "_XID" : "_IDX";
                break;                
        }

        return suffix;
    }
}
