/**********************************************************************
Copyright (c) 2002 Mike Martin (TJDO) and others. All rights reserved. 
This program and the accompanying materials are made available under
the terms of the JPOX License v1.0 which accompanies this distribution. 

Contributors:
2003 Andy Jefferson - coding standards
    ...
**********************************************************************/
package org.jpox.store;


/**
 * A "second-class" object identifier.  SCOIDs are used as object identifiers
 * for classes that have no database extent, such as JPOXSQL result objects.
 *
 * <p>The only thing a SCOID tracks is the class of the instance being
 * identified.  Every SCOID is unique within the JVM, which effectively means
 * universally unique since Serializable is not implemented.
 *
 * @see OID
 * @version $Revision: 1.1 $
 */
public final class SCOID
{
    private String objClass;

    /**
     * Constructs a new SCOID to identify an object of the given class.
     *
     * @param objClass  The class of the instance being identified.
     */
    public SCOID(String objClass)
    {
        this.objClass = objClass;
    }

    /**
     * Returns the class of the object identified by this SCOID.
     *
     * @return  The class of the object identified by this SCOID.
     */
    public String getSCOClass()
    {
        return objClass;
    }
}
