/**********************************************************************
Copyright (c) 2003 Erik Bengtson and others. All rights reserved. 
This program and the accompanying materials are made available under 
the terms of the JPOX License v1.0 which accompanies this distribution. 

Contributors:
    ...
**********************************************************************/
package org.jpox.store;

import org.jpox.store.mapping.JavaTypeMapping;

/**
 * Maintain an index for the mapping field vs columns in a JDBC statement.
 * Each field for a Request (in a Statement) must have one instance of this
 * containing the field Number + Mapping + the columns
 * <PRE> 
 * e.g.
 * CLASS                 TABLE                MAPPING           FIELD NUMBER    EXPRESSION INDEX
 * -----------------     ----------------     --------------    ------------    ----------------
 * class A           --> TABLE_A            
 * {                                       
 *    int fieldA;    --> COL_FIELDA       --> IntegerMapping --> 1           --> 1
 *    String fieldB; --> COL_FIELDB_PART1 --> StringMapping  --> 2           --> 2
 *                   --> COL_FIELDB_PART2 -->                                --> 3
 *    ...             
 * } 
 * </PRE> 
 *
 * @version $Revision: 1.1 $ 
 */
public class StatementExpressionIndex
{
    /** the mapping to the field **/
    JavaTypeMapping mapping;

    /** the index for a column in the statement **/
    int[] expressionIndex;

    /** the index for a parameter in the statement **/
    int[] parameterIndex;

    /**
     * @return The expression index
     */
    public int[] getExpressionIndex()
    {
        return expressionIndex;
    }

    /**
     * @return The mapping
     */
    public JavaTypeMapping getMapping()
    {
        return mapping;
    }

    /**
     * @param is The expression index
     */
    public void setExpressionIndex(int[] is)
    {
        expressionIndex = is;
    }

    /**
     * @param mapping The mapping
     */
    public void setMapping(JavaTypeMapping mapping)
    {
        this.mapping = mapping;
    }

    /**
     * @return The parameter index
     */
    public int[] getParameterIndex()
    {
        return parameterIndex;
    }

    /**
     * @param is The parameter index
     */
    public void setParameterIndex(int[] is)
    {
        parameterIndex = is;
    }
    
    /**
     * Method to return a string version of this object.
     * @return String version
     **/
    public String toString()
    {
        StringBuffer buffer = new StringBuffer();
        buffer.append("mapping: "+mapping);
        buffer.append("\n");
        buffer.append("parameterIndex: "+parameterIndex);
        buffer.append("\n");
        buffer.append("expressionIndex: "+expressionIndex);
        buffer.append("\n");
        return buffer.toString();
    }
}
