/**********************************************************************
Copyright (c) 2002 Mike Martin and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
    Andy Jefferson - coding standards
    ...
**********************************************************************/
package org.jpox.store.exceptions;

import org.jpox.store.rdbms.Column;
import org.jpox.store.rdbms.typeinfo.TypeInfo;
import org.jpox.util.Localiser;

/**
 * A <tt>IncompatibleDataTypeException</tt> is thrown if a column is detected to
 * have an incompatible type in the database during schema validation.
 *
 * @see Column
 * @version $Revision: 1.1 $ 
 */
public class IncompatibleDataTypeException extends SchemaValidationException
{
    private static final Localiser LOCALISER=Localiser.getInstance("org.jpox.store.exceptions.Localisation");

    /**
     * Constructs an incompatible data type exception.
     *
     * @param column        The column having an incompatible data type.
     * @param expectedType  The expected java.sql.Type of the column.
     * @param actualType    The actual java.sql.Type of the column.
     */
    public IncompatibleDataTypeException(Column column, int expectedType, int actualType)
    {
        super(LOCALISER.msg("Exception.IncompatibleDataType",column,TypeInfo.getJDBCTypeName(actualType),TypeInfo.getJDBCTypeName(expectedType)));
    }
}
