/**********************************************************************
Copyright (c) 2002 Mike Martin and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
    Andy Jefferson - coding standards
    ...
**********************************************************************/
package org.jpox.store.exceptions;

import javax.jdo.JDOUserException;
import org.jpox.util.Localiser;

/**
 * A <tt>IncompatibleQueryElementTypeException</tt> is thrown if a variable used
 * in a query is detected to have an type incompatible with the element type of
 * the collection to which it is being applied.
 *
 * @see org.jpox.store.query.Query
 * @see org.jpox.store.scostore.CollectionStore
 * @version $Revision: 1.2 $ 
 */
public class IncompatibleQueryElementTypeException extends JDOUserException
{
    private static final Localiser LOCALISER=Localiser.getInstance("org.jpox.store.exceptions.Localisation");

    /**
     * Constructs an incompatible query element type exception.
     *
     * @param expectedType  The expected element base type.
     * @param actualType    The actual element type.
     */
    public IncompatibleQueryElementTypeException(String expectedType, String actualType)
    {
        super(LOCALISER.msg("Exception.IncompatibleQueryElementType",actualType,expectedType));
    }
}
