/**********************************************************************
Copyright (c) 2002 Mike Martin and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
    Andy Jefferson - coding standards
    ...
**********************************************************************/
package org.jpox.store.exceptions;

import javax.jdo.JDOUserException;

import org.jpox.metadata.FieldMetaData;
import org.jpox.util.Localiser;

/**
 * A <tt>InvalidMetaDataRelationshipException</tt> is thrown if the metadata for
 * a persistent field declares a relationship to another field, but the field on
 * the other side has no complementary declaration.
 *
 * @version $Revision: 1.1 $
 */
public class InvalidMetaDataRelationshipException extends JDOUserException
{
    private static final Localiser LOCALISER=Localiser.getInstance("org.jpox.store.exceptions.Localisation");

    /**
     * Constructs an invalid metadata relationship exception.
     *
     * @param from      The metadata of the field declaring a relationship.
     * @param fromRel   The type of relationship declared by 'from'.
     * @param to        The metadata of the field referred to by the
     *                  relationship.
     * @param toRel     The type of relationship missing from 'to'.
     */
    public InvalidMetaDataRelationshipException(FieldMetaData from, String fromRel, FieldMetaData to, String toRel)
    {
        super(LOCALISER.msg("Exception.InvalidMetaDataRelationship",from.toString(),to.toString(),fromRel,toRel));
    }
}
