/**********************************************************************
Copyright (c) 2002 Mike Martin and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
    Andy Jefferson - coding standards
    ...
**********************************************************************/
package org.jpox.store.exceptions;

import java.util.Collection;
import java.util.Iterator;

import org.jpox.store.rdbms.Column;
import org.jpox.store.rdbms.table.Table;

/**
 * A <tt>MissingColumnException</tt> is thrown if an expected column is
 * not found in the database during schema validation.
 *
 * @see Table
 * @see Column
 * @version $Revision: 1.1 $ 
 */
public class MissingColumnException extends SchemaValidationException
{
    /**
     * Constructs a missing column exception.
     *
     * @param table     The table in which column(s) were missing.
     * @param columns   The collection of Column(s) that were missing.
     */
    public MissingColumnException(Table table, Collection columns)
    {
        super(LOCALISER.msg("Exception.MissingColumn",table.toString(),getColumnNameList(columns)));
    }

    private static String getColumnNameList(Collection columns)
    {
        StringBuffer list = new StringBuffer();
        Iterator i = columns.iterator();

        while (i.hasNext())
        {
            if (list.length() > 0)
            {
                list.append(", ");
            }

            list.append(((Column)i.next()).getName());
        }

        return list.toString();
    }
}
