/**********************************************************************
Copyright (c) 2002 Mike Martin and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
    Andy Jefferson - coding standards
    ...
**********************************************************************/
package org.jpox.store.exceptions;

import java.util.Collection;
import java.util.Iterator;

import org.jpox.store.rdbms.table.Table;

/**
 * A <tt>MissingIndicesException</tt> is thrown if an expected index is not
 * found in the database during schema validation.
 *
 * @see Table
 * @version $Revision: 1.1 $ 
 */
public class MissingIndicesException extends SchemaValidationException
{
    /**
     * Constructs a missing index(s) exception.
     *
     * @param table The table in which index(s) were missing.
     * @param stmts The statements that would be used to create the missing
     * indices.
     */
    public MissingIndicesException(Table table, Collection stmts)
    {
        super(LOCALISER.msg("Exception.MissingIndices",table.toString(),listOnePerLine(stmts)));
    }

    private static String listOnePerLine(Collection stmts)
    {
        StringBuffer list = new StringBuffer();
        Iterator i = stmts.iterator();

        while (i.hasNext())
        {
            list.append(i.next()).append('\n');
        }

        return list.toString();
    }
}
