/**********************************************************************
Copyright (c) 2002 Mike Martin and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
    Andy Jefferson - coding standards
    Erik Bengtson - renamed to NoDatastoreMappingException
    ...
**********************************************************************/
package org.jpox.store.exceptions;

import javax.jdo.JDOUserException;
import org.jpox.util.Localiser;

/**
 * A <tt>NoDatastoreMappingException</tt> is thrown if an operation is performed that
 * assumes that a particular persistent field is stored in a single datastore
 * field when it is not (such as if the field is a Collection or a Map).
 *
 * @link org.jpox.store.mapping.AbstractListMapping
 * @link org.jpox.store.mapping.AbstractSetMapping
 * @link org.jpox.store.mapping.MapMapping
 * @version $Revision: 1.1 $ 
 */
public class NoDatastoreMappingException extends JDOUserException
{
    protected static final Localiser LOCALISER=Localiser.getInstance("org.jpox.store.exceptions.Localisation");

    /**
     * Constructs a no datastore mapping exception.
     *
     * @param fieldName The name of the field.
     */
    public NoDatastoreMappingException(String fieldName)
    {
        super(LOCALISER.msg("Exception.NoColumn",fieldName));
    }
}
