/**********************************************************************
Copyright (c) 2002 Mike Martin and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
    Andy Jefferson - coding standards
    ...
**********************************************************************/
package org.jpox.store.exceptions;

import javax.jdo.JDOUserException;
import org.jpox.util.Localiser;

/**
 * A <tt>NoSuchPersistentFieldException</tt> is thrown if a reference is made
 * somewhere, such as in a query filter string, to a field that either doesn't
 * exist or is not persistent.
 * 
 * @version $Revision: 1.1 $
 */
public class NoSuchPersistentFieldException extends JDOUserException
{
    protected static final Localiser LOCALISER=Localiser.getInstance("org.jpox.store.exceptions.Localisation");

    /**
     * Constructs a no such persistent field exception.
     *
     * @param className The class in which the field was not found.
     * @param fieldName The name of the field.
     */
    public NoSuchPersistentFieldException(String className, String fieldName)
    {
        super(LOCALISER.msg("Exception.NoSuchPersistentField",fieldName,className));
    }

    /**
     * Constructs a no such persistent field exception.
     *
     * @param className     The class in which the field was not found.
     * @param fieldNumber   The field number  of the field.
     */
    public NoSuchPersistentFieldException(String className, int fieldNumber)
    {
        super(LOCALISER.msg("Exception.NoSuchPersistentFieldNo","" + fieldNumber,className));
    }
}
