/**********************************************************************
Copyright (c) 2002 Mike Martin and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
    Andy Jefferson - coding standards
    ...
**********************************************************************/
package org.jpox.store.exceptions;

import javax.jdo.JDOFatalInternalException;

import org.jpox.store.DatastoreContainerObject;
import org.jpox.store.DatastoreField;
import org.jpox.util.Localiser;

/**
 * A <tt>TableMismatchException</tt> is thrown if the query statement generator
 * attempts to reference a column in a table expression but the column's table
 * is not present in the expression, nor can it be sensibly joined to the
 * expression.
 * This indicates a bug in the query statement generator.
 *
 * @see org.jpox.store.expression.TableExpression
 * @version $Revision: 1.4 $ 
 */
public class TableMismatchException extends JDOFatalInternalException
{
    protected static final Localiser LOCALISER=Localiser.getInstance("org.jpox.store.exceptions.Localisation");

    /**
     * Constructs a table mismatch exception.
     *
     * @param column    The column that was being referenced.
     * @param mainTable The main table of the table expression.
     */
    public TableMismatchException(DatastoreField column, DatastoreContainerObject mainTable)
    {
        super(LOCALISER.msg("Exception.TableMismatch",column.getName(),mainTable,column.getDatastoreContainerObject()));
    }
}
