/**********************************************************************
Copyright (c) 2002 Mike Martin and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
    Andy Jefferson - coding standards
    ...
**********************************************************************/
package org.jpox.store.exceptions;

import javax.jdo.JDOFatalDataStoreException;

import org.jpox.store.rdbms.sqlidentifier.ForeignKeyIdentifier;
import org.jpox.util.Localiser;

/**
 * A <tt>TooManyForeignKeysException</tt> is thrown when trying to add a foreign
 * key to a table and the table already has the maximum allowed number of
 * foreign keys.
 *
 * @see ForeignKeyIdentifier
 * @version $Revision: 1.1 $ 
 */
public class TooManyForeignKeysException extends JDOFatalDataStoreException
{
    protected static final Localiser LOCALISER=Localiser.getInstance("org.jpox.store.exceptions.Localisation");

    /**
     * Constructs a too-many-foreign-keys exception.
     *
     * @param table_name Name of the table that already has the maximum allowed
     *                   number of foreign keys.
     */
    public TooManyForeignKeysException(String table_name)
    {
        super(LOCALISER.msg("Exception.TooManyForeignKeys","" + ForeignKeyIdentifier.MAX_FOREIGN_KEYS,table_name));
    }
}
