/**********************************************************************
Copyright (c) 2004 Erik Bengtson and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
    ...
**********************************************************************/
package org.jpox.store.expression;

import java.util.ArrayList;
import java.util.List;

import org.jpox.store.mapping.JavaTypeMapping;
import org.jpox.store.query.QueryStatement;

/**
 * Representation of aggregate functions in JDOQL
 * @version $Revision: 1.4 $
 */
public class AggregateExpression extends NumericExpression
{

    /**
     * @param qs
     */
    public AggregateExpression(QueryStatement qs)
    {
        super(qs);
    }

    protected AggregateExpression(JavaTypeMapping mapping,  String functionName, List args)
    {
        super(functionName, args);
        this.expressionList.addExpression(this);
        this.mapping = mapping;
    }    
    
    /**
     * Returns the max value of the argument.
     * @param expr the expression
     * @return the result in a ScalarExpression instance
     */
    public ScalarExpression maxMethod(ScalarExpression expr)
    {
        return getFunctionExpression("MAX",expr,expr.getMapping());
    }
    
    /**
     * Returns the max value of the argument.
     * @param expr the expression
     * @return the result in a ScalarExpression instance
     */
    public ScalarExpression minMethod(ScalarExpression expr)
    {
        return getFunctionExpression("MIN",expr,expr.getMapping());
    }
    
    /**
     * Returns the sum value of the argument.
     * @param expr the expression
     * @return the result in a ScalarExpression instance
     */
    public ScalarExpression sumMethod(ScalarExpression expr)
    {
        return getFunctionExpression("SUM",expr,expr.getMapping());
    }
    
    /**
     * Returns the average value of the argument.
     * @param expr the expression
     * @return the result in a ScalarExpression instance
     */
    public ScalarExpression avgMethod(ScalarExpression expr)
    {
        return getFunctionExpression("AVG",expr,expr.getMapping());
    }
    
    /**
     * Returns the count value of the argument.
     * @param expr the expression
     * @return the result in a ScalarExpression instance
     */
    public ScalarExpression countMethod(ScalarExpression expr)
    {
        return getFunctionExpression("COUNT",expr,qs.getStoreManager().getDatastoreAdapter().getMapping(Long.class,qs.getStoreManager()));
    }    
    
    /**
     * Creates a function expression. e.g. MAX(expr)
     * @param functionName the name of the function
     * @param expr the expression
     * @param mapping the mapping to be used for converting from db to java and vice-versa
     * @return the function expression. a NullLiteral is returned if the argument expr is null 
     */
    private ScalarExpression getFunctionExpression(String functionName, ScalarExpression expr, JavaTypeMapping mapping)
    {
        if( expr == null )
        {
            return new NullLiteral(qs);
        }
        else
        {
            ArrayList args = new ArrayList();
            args.add(expr);

            ScalarExpression numExpr = new AggregateExpression(mapping, functionName, args);
            return numExpr;
        }        
    }
}
