/**********************************************************************
Copyright (c) 2003 Andy Jefferson and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
    ...
**********************************************************************/
package org.jpox.store.expression;

import java.util.Arrays;

import org.jpox.store.DatastoreAdapter;
import org.jpox.store.mapping.JavaTypeMapping;
import org.jpox.store.query.QueryStatement;

/**
 * Literal for binary object.
 *
 * @version $Revision: 1.4 $
 **/
public class BinaryLiteral extends BinaryExpression implements Literal
{
    private final byte[] value;

    public BinaryLiteral(QueryStatement qs, JavaTypeMapping mapping, byte[] value)
    {
        super(qs);
        this.mapping = mapping;
        this.value = value;
        DatastoreAdapter dba = qs.getStoreManager().getDatastoreAdapter();
        st.appendParameter(dba.getMapping(byte[].class, qs.getStoreManager()), value);
    }

    public BooleanExpression eq(ScalarExpression expr)
    {
        if (expr instanceof BinaryLiteral)
        {
            return new BooleanLiteral(qs, mapping, Arrays.equals(value, ((BinaryLiteral)expr).value));
        }
        else
        {
            return super.eq(expr);
        }
    }

    public BooleanExpression noteq(ScalarExpression expr)
    {
        if (expr instanceof BinaryLiteral)
        {
            return new BooleanLiteral(qs, mapping, !Arrays.equals(value, ((BinaryLiteral)expr).value));
        }
        else
        {
            return super.noteq(expr);
        }
    }

    public Object getValue()
    {
        return value;
    }
}