/**********************************************************************
Copyright (c) 2002 Kelly Grizzle (TJDO) and others.
All rights reserved.   This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
2002 Mike Martin (TJDO)
2003 Andy Jefferson - coding standards
    ...
**********************************************************************/
package org.jpox.store.expression;

import org.jpox.store.mapping.JavaTypeMapping;
import org.jpox.store.query.QueryStatement;

/**
 * Representation of BooleanBit column expression in a Query.
 *
 * @version $Revision: 1.2 $
 **/
public class BooleanBitColumnExpression extends BooleanExpression
{
    private boolean truthTest;

    public BooleanBitColumnExpression(QueryStatement qs, JavaTypeMapping mapping, TableExpression te)
    {
        this(qs, mapping, te, true);
    }

    public BooleanBitColumnExpression(QueryStatement qs, JavaTypeMapping mapping, TableExpression te, boolean truthTest)
    {
        super(qs, mapping, te);

        this.truthTest = truthTest;
        st.append(truthTest ? " = 1" : " = 0");

        lowestOperator = OP_EQ;
    }

    public BooleanExpression not()
    {
        return new BooleanBitColumnExpression(qs, mapping, te, !truthTest);
    }

    public BooleanExpression eq(ScalarExpression expr)
    {
        if (expr instanceof NullLiteral)
        {
            return expr.eq(new NumericExpression(qs, mapping, te));
        }
        else if (expr instanceof BooleanBitColumnExpression)
        {
            return new BooleanExpression(new NumericExpression(qs, mapping, te),
                                         OP_EQ,
                                         new NumericExpression(qs, expr.mapping, expr.te));
        }
        else
        {
            return super.eq(expr);
        }
    }

    public BooleanExpression noteq(ScalarExpression expr)
    {
        if (expr instanceof NullLiteral)
        {
            return expr.noteq(new NumericExpression(qs, mapping, te));
        }
        else if (expr instanceof BooleanBitColumnExpression)
        {
            return new BooleanExpression(new NumericExpression(qs, mapping, te),
                                         OP_NOTEQ,
                                         new NumericExpression(qs, expr.mapping, expr.te));
        }
        else
        {
            return super.noteq(expr);
        }
    }
}
