/**********************************************************************
Copyright (c) 2002 Kelly Grizzle (TJDO) and others.
All rights reserved.   This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
2002 Mike Martin (TJDO)
2003 Andy Jefferson - coding standards
    ...
**********************************************************************/
package org.jpox.store.expression;

import org.jpox.store.mapping.JavaTypeMapping;
import org.jpox.store.query.QueryStatement;

/**
 * Representation of a BooleanChar column expression.
 *
 * @version $Revision: 1.2 $
 **/
public class BooleanCharColumnExpression extends BooleanExpression
{
    private boolean truthTest;

    public BooleanCharColumnExpression(QueryStatement qs, JavaTypeMapping mapping, TableExpression te)
    {
        this(qs, mapping, te, true);
    }

    public BooleanCharColumnExpression(QueryStatement qs, JavaTypeMapping mapping, TableExpression te, boolean truthTest)
    {
        super(qs, mapping, te);

        this.truthTest = truthTest;
        st.append(" = ").append(new BooleanCharColumnLiteral(qs, mapping, truthTest));

        lowestOperator = OP_EQ;
    }

    public BooleanExpression not()
    {
        return new BooleanCharColumnExpression(qs, mapping, te, !truthTest);
    }

    public BooleanExpression eq(ScalarExpression expr)
    {
        if (expr instanceof NullLiteral)
        {
            return expr.eq(new CharacterExpression(qs, mapping, te));
        }
        else if (expr instanceof BooleanCharColumnExpression)
        {
            return new BooleanExpression(new CharacterExpression(qs, mapping, te),
                                         OP_EQ,
                                         new CharacterExpression(qs, expr.mapping, expr.te));
        }
        else
        {
            return super.eq(expr);
        }
    }

    public BooleanExpression noteq(ScalarExpression expr)
    {
        if (expr instanceof NullLiteral)
        {
            return expr.noteq(new CharacterExpression(qs, mapping, te));
        }
        else if (expr instanceof BooleanCharColumnExpression)
        {
            return new BooleanExpression(new CharacterExpression(qs, mapping, te),
                                         OP_NOTEQ,
                                         new CharacterExpression(qs, expr.mapping, expr.te));
        }
        else
        {
            return super.noteq(expr);
        }
    }

    public BooleanExpression in(ScalarExpression expr)
    {
        return new BooleanExpression(new CharacterExpression(qs, mapping, te), OP_IN, expr);
    }
}
