/**********************************************************************
Copyright (c) 2002 Kelly Grizzle (TJDO) and others.
All rights reserved.   This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
2002 Mike Martin (TJDO)
2003 Andy Jefferson - coding standards
    ...
**********************************************************************/
package org.jpox.store.expression;

import java.util.List;

import org.jpox.store.mapping.JavaTypeMapping;
import org.jpox.store.query.QueryStatement;
import org.jpox.store.rdbms.adapter.DatabaseAdapter;

/**
 * Representation of a Boolean expression in a Query.
 *
 * @version $Revision: 1.3 $
 **/
public class BooleanExpression extends ScalarExpression
{
    public BooleanExpression(QueryStatement qs)
    {
        super(qs);
    }

    public BooleanExpression(QueryStatement qs, JavaTypeMapping mapping, TableExpression te)
    {
        super(qs, mapping, te);
    }

    public BooleanExpression(String functionName, List args)
    {
        super(functionName, args);
    }

    public BooleanExpression(MonadicOperator op, ScalarExpression operand)
    {
        super(op, operand);
    }

    public BooleanExpression(ScalarExpression operand1, DyadicOperator op, ScalarExpression operand2)
    {
        super(operand1, op, operand2);
    }

    public BooleanExpression and(ScalarExpression expr)
    {
        if (expr instanceof BooleanLiteral)
        {
            return expr.and(this);
        }
        else if (expr instanceof BooleanExpression)
        {
            return new BooleanExpression(this, OP_AND, expr);
        }
        else
        {
            return super.and(expr);
        }
    }

    public BooleanExpression eor(ScalarExpression expr)
    {
        if (expr instanceof BooleanLiteral)
        {
            return expr.eor(this);
        }
        else if (expr instanceof BooleanExpression)
        {
            if (((DatabaseAdapter) qs.getStoreManager().getDatastoreAdapter()).supportsBooleanComparison())
            {
                return new BooleanExpression(this, OP_NOTEQ, expr);
            }
            else
            {
                return and(expr.not()).ior(not().and(expr));
            }
        }
        else
        {
            return super.eor(expr);
        }
    }

    public BooleanExpression ior(ScalarExpression expr)
    {
        if (expr instanceof BooleanLiteral)
        {
            return expr.ior(this);
        }
        else if (expr instanceof BooleanExpression)
        {
            return new BooleanExpression(this, OP_OR, expr);
        }
        else
        {
            return super.ior(expr);
        }
    }

    public BooleanExpression not()
    {
        return new BooleanExpression(OP_NOT, this);
    }

    public BooleanExpression eq(ScalarExpression expr)
    {
        if (expr instanceof BooleanLiteral || expr instanceof NullLiteral)
        {
            return expr.eq(this);
        }
        else if (expr instanceof BooleanExpression)
        {
            if (((DatabaseAdapter) qs.getStoreManager().getDatastoreAdapter()).supportsBooleanComparison())
            {
                return new BooleanExpression(this, OP_EQ, expr);
            }
            else
            {
                return and(expr).ior(not().and(expr.not()));
            }
        }
        else
        {
            return super.eq(expr);
        }
    }

    public BooleanExpression noteq(ScalarExpression expr)
    {
        if (expr instanceof BooleanLiteral || expr instanceof NullLiteral)
        {
            return expr.noteq(this);
        }
        else if (expr instanceof BooleanExpression)
        {
            if (((DatabaseAdapter) qs.getStoreManager().getDatastoreAdapter()).supportsBooleanComparison())
            {
                return new BooleanExpression(this, OP_NOTEQ, expr);
            }
            else
            {
                return and(expr.not()).ior(not().and(expr));
            }
        }
        else
        {
            return super.noteq(expr);
        }
    }
    
    public BooleanExpression in(ScalarExpression expr)
    {
        return new BooleanExpression(this, OP_IN, expr);
    }
    
    public ScalarExpression as(ScalarExpression expr)
    {
        throw new IllegalOperationException("BooleanExpressions can't evaluate as operations.", this);
    }    
}
