/**********************************************************************
Copyright (c) 2003 Erik Bengtson and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
2003 Andy Jefferson - coding standards
    ...
**********************************************************************/
package org.jpox.store.expression;

import org.jpox.store.mapping.JavaTypeMapping;
import org.jpox.store.query.QueryStatement;

/**
 * Representation of a Boolean literal in a Query.
 *
 * @version $Revision: 1.2 $
 **/
public class BooleanLiteral extends BooleanExpression implements Literal
{
    private final boolean value;

    public BooleanLiteral(QueryStatement qs, JavaTypeMapping mapping, boolean value)
    {
        super(qs);
        this.mapping = mapping;
        this.value = value;
        st.append(value ? getBooleanTrueValue() : getBooleanFalseValue());
    }

    public Object getValue()
    {
        return new Boolean(value);
    }
    
    /**
     * Return the String value for TRUE in the database.  This should be
     * overriden by subclasses for special cases of booleans.
     * 
     * @return  The String value for TRUE in the database.
     */
    protected String getBooleanTrueValue()
    {
        return "TRUE";
    }

    /**
     * Return the String value for FALSE in the database.  This should be
     * overriden by subclasses for special cases of booleans.
     * 
     * @return  The String value for FALSE in the database.
     */
    protected String getBooleanFalseValue()
    {
        return "(1=0)";
    }

    public BooleanExpression and(ScalarExpression expr)
    {
        if (expr instanceof BooleanExpression)
        {
            return value ? (BooleanExpression)expr : this;
        }
        else
        {
            return super.and(expr);
        }
    }

    public BooleanExpression eor(ScalarExpression expr)
    {
        if (expr instanceof BooleanExpression)
        {
            return value ? expr.not() : (BooleanExpression)expr;
        }
        else
        {
            return super.eor(expr);
        }
    }

    public BooleanExpression ior(ScalarExpression expr)
    {
        if (expr instanceof BooleanExpression)
        {
            return value ? this : (BooleanExpression)expr;
        }
        else
        {
            return super.ior(expr);
        }
    }

    public BooleanExpression not()
    {
        return new BooleanLiteral(qs, mapping, !value);
    }

    public BooleanExpression eq(ScalarExpression expr)
    {
        if (expr instanceof BooleanExpression)
        {
            return value ? (BooleanExpression)expr : expr.not();
        }
        else
        {
            return super.eq(expr);
        }
    }

    public BooleanExpression noteq(ScalarExpression expr)
    {
        if (expr instanceof BooleanExpression)
        {
            return value ? expr.not() : (BooleanExpression)expr;
        }
        else
        {
            return super.noteq(expr);
        }
    }
}
