/**********************************************************************
Copyright (c) 2002 Kelly Grizzle (TJDO) and others.
All rights reserved.   This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
2002 Mike Martin (TJDO)
2003 Andy Jefferson - coding standards
    ...
**********************************************************************/
package org.jpox.store.expression;

import java.math.BigInteger;

import org.jpox.store.DatastoreAdapter;
import org.jpox.store.mapping.JavaTypeMapping;
import org.jpox.store.query.QueryStatement;

/**
 * Representation of a Byte literal.
 *
 * @version $Revision: 1.3 $
 **/
public class ByteLiteral extends NumericExpression implements Literal
{
    private final BigInteger value;

    private JavaTypeMapping mapping;
    
    public ByteLiteral(QueryStatement qs, JavaTypeMapping mapping, BigInteger value)
    {
        super(qs);
        this.mapping = mapping;

        this.value = value;

        DatastoreAdapter dba = qs.getStoreManager().getDatastoreAdapter();
        mapping = dba.getMapping(Byte.class, qs.getStoreManager());        
        st.appendParameter(mapping, value);
        
    }

    public Object getValue()
    {
        return value;
    }

    public BooleanExpression eq(ScalarExpression expr)
    {
        if (expr instanceof ByteLiteral)
        {
            return new BooleanLiteral(qs, mapping, value.compareTo(((ByteLiteral)expr).value) == 0);
        }
        else if (expr instanceof CharacterExpression)
        {
            return new BooleanExpression(this, OP_EQ, expr);
        }
        else
        {
            return super.eq(expr);
        }
    }

    public BooleanExpression noteq(ScalarExpression expr)
    {
        if (expr instanceof ByteLiteral)
        {
            return new BooleanLiteral(qs, mapping, value.compareTo(((ByteLiteral)expr).value) != 0);
        }
        else if (expr instanceof CharacterExpression)
        {
            return new BooleanExpression(this, OP_NOTEQ, expr);
        }
        else
        {
            return super.noteq(expr);
        }
    }

    public BooleanExpression lt(ScalarExpression expr)
    {
        if (expr instanceof ByteLiteral)
        {
            return new BooleanLiteral(qs, mapping, value.compareTo(((ByteLiteral)expr).value) < 0);
        }
        else if (expr instanceof CharacterExpression)
        {
            return new BooleanExpression(this, OP_LT, expr);
        }            
        else
        {
            return super.lt(expr);
        }
    }

    public BooleanExpression lteq(ScalarExpression expr)
    {
        if (expr instanceof ByteLiteral)
        {
            return new BooleanLiteral(qs, mapping, value.compareTo(((ByteLiteral)expr).value) <= 0);
        }
        else if (expr instanceof IndexOfExpression)
        {
            return expr.gteq(this);
        }
        else if (expr instanceof CharacterExpression)
        {
            return new BooleanExpression(this, OP_LTEQ, expr);
        }            
        else
        {
            return super.lteq(expr);
        }
    }

    public BooleanExpression gt(ScalarExpression expr)
    {
        if (expr instanceof ByteLiteral)
        {
            return new BooleanLiteral(qs, mapping, value.compareTo(((ByteLiteral)expr).value) > 0);
        }
        else if (expr instanceof CharacterExpression)
        {
            return new BooleanExpression(this, OP_GT, expr);
        }            
        else
        {
            return super.gt(expr);
        }
    }

    public BooleanExpression gteq(ScalarExpression expr)
    {
        if (expr instanceof ByteLiteral)
        {
            return new BooleanLiteral(qs, mapping, value.compareTo(((ByteLiteral)expr).value) >= 0);
        }
        else if (expr instanceof CharacterExpression)
        {
            return new BooleanExpression(this, OP_GTEQ, expr);
        }            
        else
        {
            return super.gteq(expr);
        }
    }

    public ScalarExpression add(ScalarExpression expr)
    {
        if (expr instanceof ByteLiteral)
        {
            return new ByteLiteral(qs, mapping, value.add(((ByteLiteral)expr).value));
        }
        else
        {
            return super.add(expr);
        }
    }

    public ScalarExpression sub(ScalarExpression expr)
    {
        if (expr instanceof ByteLiteral)
        {
            return new ByteLiteral(qs, mapping, value.subtract(((ByteLiteral)expr).value));
        }
        else
        {
            return super.sub(expr);
        }
    }

    public ScalarExpression mul(ScalarExpression expr)
    {
        if (expr instanceof ByteLiteral)
        {
            return new ByteLiteral(qs, mapping, value.multiply(((ByteLiteral)expr).value));
        }
        else
        {
            return super.mul(expr);
        }
    }

    public ScalarExpression div(ScalarExpression expr)
    {
        if (expr instanceof ByteLiteral)
        {
            return new ByteLiteral(qs, mapping, value.divide(((ByteLiteral)expr).value));
        }
        else
        {
            return super.div(expr);
        }
    }

    public ScalarExpression mod(ScalarExpression expr)
    {
        if (expr instanceof ByteLiteral)
        {
            return new ByteLiteral(qs, mapping, value.mod(((ByteLiteral)expr).value));
        }
        else
        {
            return super.mod(expr);
        }
    }

    public ScalarExpression neg()
    {
        return new ByteLiteral(qs, mapping, value.negate());
    }
}
