/**********************************************************************
Copyright (c) 2002 Mike Martin (TJDO) and others.
All rights reserved.   This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
2003 Andy Jefferson - coding standards
    ...
**********************************************************************/
package org.jpox.store.expression;

import org.jpox.store.mapping.JavaTypeMapping;
import org.jpox.store.query.QueryStatement;

/**
 * Representation of a Character literal in a Query.
 *
 * @version $Revision: 1.3 $
 **/
public class CharacterLiteral extends CharacterExpression implements Literal
{
    private final String value;
    private JavaTypeMapping mapping;

    public CharacterLiteral(QueryStatement qs, JavaTypeMapping mapping, char value)
    {
        super(qs);

        this.mapping = mapping;
        this.value = String.valueOf(value);
        st.append('\'').append(value).append('\'');
    }

    public CharacterLiteral(QueryStatement qs, JavaTypeMapping mapping, String value)
    {
        super(qs);

        this.mapping = mapping;
        this.value = value;
        st.appendParameter(mapping, value);
    }

    public Object getValue()
    {
        return value;
    }
    
    public BooleanExpression eq(ScalarExpression expr)
    {
        if (expr instanceof CharacterLiteral)
        {
            return new BooleanLiteral(qs, mapping, value.equals(((CharacterLiteral)expr).value));
        }
        else
        {
            return super.eq(expr);
        }
    }

    public BooleanExpression noteq(ScalarExpression expr)
    {
        if (expr instanceof CharacterLiteral)
        {
            return new BooleanLiteral(qs, mapping, !value.equals(((CharacterLiteral)expr).value));
        }
        else
        {
            return super.noteq(expr);
        }
    }

    public BooleanExpression lt(ScalarExpression expr)
    {
        if (expr instanceof CharacterLiteral)
        {
            return new BooleanLiteral(qs, mapping, value.compareTo(((CharacterLiteral)expr).value) < 0);
        }
        else
        {
            return super.lt(expr);
        }
    }

    public BooleanExpression lteq(ScalarExpression expr)
    {
        if (expr instanceof CharacterLiteral)
        {
            return new BooleanLiteral(qs, mapping, value.compareTo(((CharacterLiteral)expr).value) <= 0);
        }
        else
        {
            return super.lteq(expr);
        }
    }

    public BooleanExpression gt(ScalarExpression expr)
    {
        if (expr instanceof CharacterLiteral)
        {
            return new BooleanLiteral(qs, mapping, value.compareTo(((CharacterLiteral)expr).value) > 0);
        }
        else
        {
            return super.gt(expr);
        }
    }

    public BooleanExpression gteq(ScalarExpression expr)
    {
        if (expr instanceof CharacterLiteral)
        {
            return new BooleanLiteral(qs, mapping, value.compareTo(((CharacterLiteral)expr).value) >= 0);
        }
        else
        {
            return super.gteq(expr);
        }
    }

    public ScalarExpression add(ScalarExpression expr)
    {
        if (expr instanceof CharacterLiteral)
        {
            return new CharacterLiteral(qs, mapping, value.concat(((CharacterLiteral)expr).value));
        }
        else
        {
            return super.add(expr);
        }
    }

    public CharacterExpression toLowerCase()
    {
        return new CharacterLiteral(qs, mapping, value.toLowerCase());
    }

    public CharacterExpression toUpperCase()
    {
        return new CharacterLiteral(qs, mapping, value.toUpperCase());
    }

    public CharacterExpression trim()
    {
        return new CharacterLiteral(qs, mapping, value.trim());
    }
}
