/**********************************************************************
Copyright (c) 2002 Mike Martin (TJDO) and others. All rights reserved. 
This program and the accompanying materials are made available under 
the terms of the JPOX License v1.0 which accompanies this distribution. 

Contributors:
2003 Andy Jefferson - coding standards
2004 Andy Jefferson - renamed from SubquerySetExpression
    ...
**********************************************************************/
package org.jpox.store.expression;

import javax.jdo.JDOFatalInternalException;

import org.jpox.store.query.QueryStatement;

/**
 * An expression for collections whose contents are expressed as a SQL subquery.
 *
 * @version $Revision: 1.3 $
 */
public class CollectionSubqueryExpression extends ScalarExpression
{
    private final QueryStatement subquery;

    /**
     * Constructor.
     * @param qs The main Query Statement
     * @param subquery The subquery Query Statement.
     **/
    public CollectionSubqueryExpression(QueryStatement qs,
                                        QueryStatement subquery)
    {
        super(qs);

        this.subquery = subquery;

        if (subquery.columnsSelected() != 1)
        {
            throw new JDOFatalInternalException("Cannot use as set expression, # of selected columns != 1: " + 
                subquery.toStatementText(false));
        }

        st.append("(").append(subquery.toStatementText(false)).append(')');
    }

    public BooleanExpression containsMethod(ScalarExpression expr)
    {
        return expr.in(this);
    }

    public BooleanExpression isEmptyMethod()
    {
        return new ExistsExpression(qs, subquery, false);
    }
}