/**********************************************************************
Copyright (c) 2002 Mike Martin (TJDO) and others.
All rights reserved.   This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
2003 Andy Jefferson - coding standards
    ...
**********************************************************************/
package org.jpox.store.expression;

import java.math.BigInteger;

import javax.jdo.JDOUserException;

import org.jpox.store.StatementText;

/**
 * Representation of an IndexOf expression in a Query.
 *
 * @version $Revision: 1.3 $
 **/
class IndexOfExpression extends NumericExpression
{
    private StringExpression str;
    private StringExpression substr;

    public IndexOfExpression(StringExpression str, StringExpression substr)
    {
        super(str.qs);

        this.str = str;
        this.substr = substr;
    }

    public BooleanExpression gteq(ScalarExpression expr)
    {
        if (expr instanceof IntegerLiteral)
        {
            BigInteger idx = (BigInteger) ((IntegerLiteral)expr).getValue();

            if (idx.compareTo(BigInteger.ZERO) != 0)
            {
                throw new JDOUserException("String.indexOf() can only be compared >= 0");
            }

            StringLiteral pct = new StringLiteral(qs, mapping, '%');

            return new BooleanExpression(str, OP_LIKE, pct.add(substr).add(pct));
        }
        else
        {
            return super.gteq(expr);
        }
    }

    public StatementText toStatementText()
    {
        throw new JDOUserException("String.indexOf() can only be compared >= 0");
    }
}
