/**********************************************************************
Copyright (c) 2004 Erik Bengtson and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
    ...
**********************************************************************/
package org.jpox.store.expression;

import javax.jdo.JDOHelper;

import org.jpox.store.DatastoreAdapter;
import org.jpox.store.mapping.JavaTypeMapping;
import org.jpox.store.query.QueryStatement;

/**
 * Representation of JDOHelper in JDOQL
 * @version $Revision: 1.3 $
 */
public class JDOHelperExpression extends ScalarExpression
{

    /**
     * @param qs
     */
    public JDOHelperExpression(QueryStatement qs)
    {
        super(qs);
    }

    /**
     * Returns an object expression for the argument.
     * @param expr the expression
     * @return the returned value for an object expression argument is the
     * argument itself. The returned for a literal argument is a
     * ObjectLiteral or NullLiteral
     */
    public ScalarExpression getObjectIdMethod(ScalarExpression expr)
    {
        if( expr == null )
        {
            return new NullLiteral(qs);
        }
        if( expr instanceof Literal )
        {
            Object id = JDOHelper.getObjectId(((Literal)expr).getValue());
            if( id == null )
            {
                return new NullLiteral(qs);
            }
            else
            {
                DatastoreAdapter dba = qs.getStoreManager().getDatastoreAdapter();	                
                JavaTypeMapping m = dba.getMapping(id.getClass(), qs.getStoreManager());
                return new ObjectLiteral(qs,m,id);
            }
        }
        else if( ObjectExpression.class.isAssignableFrom(expr.getClass()) )
        {
            return expr;
        }
        throw new IllegalOperationException(this, "getObjectId", expr);
    }
}
