/**********************************************************************
Copyright (c) 2004 Erik Bengtson and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
    ...
**********************************************************************/
package org.jpox.store.expression;

import java.text.CharacterIterator;
import java.text.StringCharacterIterator;

/**
 * Parser for a Match Expression.
 *
 * @version $Revision: 1.4 $
 **/
class MatchExpressionParser
{
    private final String input;
    private final CharacterIterator ci;
    private final String zeroOrMoreChar;
    private final String anyChar;

    /**
     * Constructor
     * @param input The input string
     **/
    public MatchExpressionParser(String input, String zeroOrMoreChar, String anyChar)
    {
        this.input = input;
        this.zeroOrMoreChar = zeroOrMoreChar;
        this.anyChar = anyChar;
        ci = new StringCharacterIterator(input);
    }

    private boolean parseNextChar(char c)
    {
        int savedIdx = ci.getIndex();
        if (ci.next() == c)
        {
            return true;
        }
        else
        {
            ci.setIndex(savedIdx);
            return false;
        }
    }

    public String parsePattern()
    {
        StringBuffer lit = new StringBuffer();
        char c;

        while ( (c = ci.current()) != CharacterIterator.DONE)
        {

            if (c == '\\')
            {
                lit.append( parseEscapedCharacter() );
            }
            else if( c == '.' )
            {
                if( parseNextChar('*') )
                {
                    lit.append(zeroOrMoreChar);
                }
                else
                {
                    lit.append(anyChar);                    
                }
            }
            else
            {
                lit.append(c);
            }
            ci.next();
        }

        return lit.toString();
    }

    private String parseEscapedCharacter()
    {
        char c;

        c = ci.next();
        switch (c)
        {
            case '.':   return ".";

            // Creating a new String with \\ will swallow the escape \, so we add it back on
            // Added as a specific case as a reminder that we need to retain the escaping
            case '\\':  return "\\" + c;

            default:    return "\\" + c;
        }
    }

    public String toString()
    {
        return input;
    }
}