/**********************************************************************
Copyright (c) 2004 Erik Bengtson and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
    ...
**********************************************************************/
package org.jpox.store.expression;

import java.math.BigDecimal;
import java.math.BigInteger;

import org.jpox.store.query.QueryStatement;
import org.jpox.store.rdbms.adapter.DatabaseAdapter;

/**
 * Representation of Math functions in JDOQL
 * @version $Revision: 1.4 $
 */
public class MathExpression extends ScalarExpression
{

    /**
     * @param qs
     */
    public MathExpression(QueryStatement qs)
    {
        super(qs);
    }

    /**
     * Returns the absolute value of the argument.
     * @param expr the expression
     * @return the result in a ScalarExpression instance
     */
    public ScalarExpression absMethod(ScalarExpression expr)
    {
        if( expr == null )
        {
            return new NullLiteral(qs);
        }
        if( expr instanceof Literal )
        {
            if( expr instanceof ByteLiteral )
            {
                int originalValue = ((BigInteger)((ByteLiteral)expr).getValue()).intValue();
                BigInteger absValue = new BigInteger(String.valueOf(Math.abs( originalValue )));
                return new ByteLiteral(qs, expr.mapping, absValue);
            }
            else if( expr instanceof IntegerLiteral )
            {
                int originalValue = ((BigInteger)((IntegerLiteral)expr).getValue()).intValue();
                BigInteger absValue = new BigInteger(String.valueOf(Math.abs( originalValue )));
                return new IntegerLiteral(qs, expr.mapping, absValue);
            }            
            else if( expr instanceof FloatingPointLiteral )
            {
                double originalValue = ((BigDecimal)((FloatingPointLiteral)expr).getValue()).doubleValue();
                Double absValue = new Double(Math.abs( originalValue ));
                return new FloatingPointLiteral(qs, expr.mapping, absValue);
            }
            throw new IllegalOperationException(this, "absMethod", expr);
        }
        else
        {
            return ((DatabaseAdapter) qs.getStoreManager().getDatastoreAdapter()).absMethod(expr);
        }
    }
    /**
     * Returns the square root of the argument.
     * @param expr the expression
     * @return the result in a ScalarExpression instance 
     */
    public ScalarExpression sqrtMethod(ScalarExpression expr)
    {
        if( expr == null )
        {
            return new NullLiteral(qs);
        }
        if( expr instanceof Literal )
        {
            if( expr instanceof ByteLiteral )
            {
                int originalValue = ((BigInteger)((ByteLiteral)expr).getValue()).intValue();
                Double absValue = new Double(Math.sqrt( originalValue ));
                return new FloatingPointLiteral(qs, expr.mapping, absValue);
            }
            else if( expr instanceof IntegerLiteral )
            {
                int originalValue = ((BigInteger)((IntegerLiteral)expr).getValue()).intValue();
                Double absValue = new Double(Math.sqrt( originalValue ));
                return new FloatingPointLiteral(qs, expr.mapping, absValue);
            }            
            else if( expr instanceof FloatingPointLiteral )
            {
                double originalValue = ((BigDecimal)((FloatingPointLiteral)expr).getValue()).doubleValue();
                Double absValue = new Double(Math.sqrt( originalValue ));
                return new FloatingPointLiteral(qs, expr.mapping, absValue);
            }
            throw new IllegalOperationException(this, "sqrtMethod", expr);
        }
        else
        {
            return ((DatabaseAdapter) qs.getStoreManager().getDatastoreAdapter()).sqrtMethod(expr);
        }
    }    
}
