/**********************************************************************
Copyright (c) 2002 Mike Martin (TJDO) and others.
All rights reserved.   This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
2003 Andy Jefferson - coding standards
    ...
**********************************************************************/
package org.jpox.store.expression;

import java.util.List;

import org.jpox.store.mapping.JavaTypeMapping;
import org.jpox.store.query.QueryStatement;

/**
 * Representation of a Numeric expression in a Query.
 *
 * @version $Revision: 1.3 $
 **/
public class NumericExpression extends ScalarExpression
{
    public NumericExpression(QueryStatement qs)
    {
        super(qs);
    }

    public NumericExpression(QueryStatement qs, JavaTypeMapping mapping, TableExpression te)
    {
        super(qs, mapping, te);
    }

    public NumericExpression(String functionName, List args)
    {
        super(functionName, args);
    }

    public NumericExpression(MonadicOperator op, ScalarExpression operand)
    {
        super(op, operand);
    }

    public NumericExpression(ScalarExpression operand1, DyadicOperator op, ScalarExpression operand2)
    {
        super(operand1, op, operand2);
    }
    
    public BooleanExpression eq(ScalarExpression expr)
    {
        if (expr instanceof NullLiteral)
        {
            return expr.eq(this);
        }
        else if (expr instanceof NumericExpression)
        {
            return new BooleanExpression(this, OP_EQ, expr);
        }
        else if (expr instanceof CharacterLiteral)
        {
            return new BooleanExpression(this, OP_EQ, expr);
        }
        else
        {
            return super.eq(expr);
        }
    }

    public BooleanExpression noteq(ScalarExpression expr)
    {
        if (expr instanceof NullLiteral)
        {
            return expr.noteq(this);
        }
        else if (expr instanceof NumericExpression)
        {
            return new BooleanExpression(this, OP_NOTEQ, expr);
        }
        else if (expr instanceof CharacterLiteral)
        {
            return new BooleanExpression(this, OP_NOTEQ, expr);
        }
        else
        {
            return super.noteq(expr);
        }
    }

    public BooleanExpression lt(ScalarExpression expr)
    {
        if (expr instanceof NumericExpression)
        {
            return new BooleanExpression(this, OP_LT, expr);
        }
        else if (expr instanceof CharacterLiteral)
        {
            return new BooleanExpression(this, OP_LT, expr);
        }            
        else
        {
            return super.lt(expr);
        }
    }

    public BooleanExpression lteq(ScalarExpression expr)
    {
        if (expr instanceof NumericExpression)
        {
            return new BooleanExpression(this, OP_LTEQ, expr);
        }
        else if (expr instanceof CharacterLiteral)
        {
            return new BooleanExpression(this, OP_LTEQ, expr);
        }              
        else
        {
            return super.lteq(expr);
        }
    }

    public BooleanExpression gt(ScalarExpression expr)
    {
        if (expr instanceof NumericExpression)
        {
            return new BooleanExpression(this, OP_GT, expr);
        }
        else if (expr instanceof CharacterLiteral)
        {
            return new BooleanExpression(this, OP_GT, expr);
        }             
        else
        {
            return super.gt(expr);
        }
    }

    public BooleanExpression gteq(ScalarExpression expr)
    {
        if (expr instanceof NumericExpression)
        {
            return new BooleanExpression(this, OP_GTEQ, expr);
        }
        else if (expr instanceof CharacterLiteral)
        {
            return new BooleanExpression(this, OP_GTEQ, expr);
        }              
        else
        {
            return super.gteq(expr);
        }
    }

    public BooleanExpression in(ScalarExpression expr) 
    { 
        return new BooleanExpression(this, OP_IN, expr); 
    } 

    public ScalarExpression add(ScalarExpression expr)
    {
        if (expr instanceof NumericExpression)
        {
            return new NumericExpression(this, OP_ADD, expr);
        }
        else if (expr instanceof StringExpression )
        {
            return new StringExpression(new StringExpression(qs, this), OP_CONCAT, expr);
        }        
        else if (expr instanceof NullLiteral)
        {
            return expr;
        }        
        else
        {
            return super.add(expr);
        }
    }

    public ScalarExpression sub(ScalarExpression expr)
    {
        if (expr instanceof NumericExpression)
        {
            return new NumericExpression(this, OP_SUB, expr);
        }
        else
        {
            return super.sub(expr);
        }
    }

    public ScalarExpression mul(ScalarExpression expr)
    {
        if (expr instanceof NumericExpression)
        {
            return new NumericExpression(this, OP_MUL, expr);
        }
        else
        {
            return super.mul(expr);
        }
    }

    public ScalarExpression div(ScalarExpression expr)
    {
        if (expr instanceof NumericExpression)
        {
            return new NumericExpression(this, OP_DIV, expr);
        }
        else
        {
            return super.div(expr);
        }
    }

    public ScalarExpression neg()
    {
        return new NumericExpression(OP_NEG, this);
    }
}
