/**********************************************************************
Copyright (c) 2002 Mike Martin (TJDO) and others. All rights reserved.
This program and the accompanying materials are made available under 
the terms of the JPOX License v1.0 which accompanies this distribution. 

Contributors:
2003 Andy Jefferson - coding standards
2003 Andy Jefferson - addition of equalsMethod()
    ...
**********************************************************************/
package org.jpox.store.expression;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.List;

import org.jpox.store.mapping.JavaTypeMapping;
import org.jpox.store.query.QueryStatement;
import org.jpox.store.rdbms.adapter.DatabaseAdapter;

/**
 * Representation of an expression with a string.
 *
 * @version $Revision: 1.6 $
 **/
public class StringExpression extends ScalarExpression
{
    public StringExpression(QueryStatement qs, NumericExpression expr)
    {
        super(qs);
        st.append(((DatabaseAdapter) qs.getStoreManager().getDatastoreAdapter()).convertExprToString(expr));
    }
    
    public StringExpression(QueryStatement qs)
    {
        super(qs);
    }

    public StringExpression(QueryStatement qs, JavaTypeMapping mapping, TableExpression te)
    {
        super(qs, mapping, te);
    }

    public StringExpression(String functionName, List args)
    {
        super(functionName, args);
    }

    public StringExpression(ScalarExpression operand1, DyadicOperator op, ScalarExpression operand2)
    {
        super(operand1, op, operand2);
    }

    public BooleanExpression eq(ScalarExpression expr)
    {
        if (expr instanceof NullLiteral)
        {
            return expr.eq(this);
        }
        else if (expr instanceof CharacterLiteral)
        {
            return new BooleanExpression(this, OP_EQ, expr);            
        }
        else if (expr instanceof StringExpression)
        {
            return new BooleanExpression(this, OP_EQ, expr);   
        }
        else if (expr instanceof ByteLiteral)
        {
            int value = ((BigInteger)((ByteLiteral)expr).getValue()).intValue(); 
            CharacterLiteral literal = new CharacterLiteral(qs, mapping, String.valueOf((char)value));    
            return new BooleanExpression(this, OP_EQ, literal);
        }                   
        else if (expr instanceof IntegerLiteral)
        {
            int value = ((BigInteger)((IntegerLiteral)expr).getValue()).intValue(); 
            CharacterLiteral literal = new CharacterLiteral(qs, mapping, String.valueOf((char)value));                
            return new BooleanExpression(this, OP_EQ, literal);
        }
        else if (expr instanceof FloatingPointLiteral)
        {
            int value = ((BigDecimal)((FloatingPointLiteral)expr).getValue()).intValue(); 
            CharacterLiteral literal = new CharacterLiteral(qs, mapping, String.valueOf((char)value));
            return new BooleanExpression(this, OP_EQ, literal);
        }
        else if (expr instanceof NumericExpression)
        {
            return new BooleanExpression(this, OP_EQ, expr);
        }
        else
        {
            return super.eq(expr);
        }
    }
    
    public BooleanExpression noteq(ScalarExpression expr)
    {
        if (expr instanceof NullLiteral)
        {
            return expr.noteq(this);
        }
        else if (expr instanceof CharacterLiteral)
        {
            return new BooleanExpression(this, OP_NOTEQ, expr);            
        }
        else if (expr instanceof StringExpression)
        {
            return new BooleanExpression(this, OP_NOTEQ, expr);   
        }
        else if (expr instanceof ByteLiteral)
        {
            int value = ((BigInteger)((ByteLiteral)expr).getValue()).intValue(); 
            CharacterLiteral literal = new CharacterLiteral(qs, mapping, String.valueOf((char)value));
            return new BooleanExpression(this, OP_NOTEQ, literal);
        }                   
        else if (expr instanceof IntegerLiteral)
        {
            int value = ((BigInteger)((IntegerLiteral)expr).getValue()).intValue(); 
            CharacterLiteral literal = new CharacterLiteral(qs, mapping, String.valueOf((char)value));
            return new BooleanExpression(this, OP_NOTEQ, literal);
        }
        else if (expr instanceof FloatingPointLiteral)
        {
            int value = ((BigDecimal)((FloatingPointLiteral)expr).getValue()).intValue(); 
            CharacterLiteral literal = new CharacterLiteral(qs, mapping, String.valueOf((char)value));
            return new BooleanExpression(this, OP_NOTEQ, literal);
        }
        else if (expr instanceof NumericExpression)
        {
            return new BooleanExpression(this, OP_NOTEQ, expr);
        }
        else
        {
            return super.noteq(expr);
        }
    }

    public BooleanExpression lt(ScalarExpression expr)
    {
        if (expr instanceof NullLiteral)
        {
            return expr.lt(this);
        }
        else if (expr instanceof CharacterLiteral)
        {
            return new BooleanExpression(this, OP_LT, expr);            
        }
        else if (expr instanceof StringExpression)
        {
            return new BooleanExpression(this, OP_LT, expr);   
        }
        else if (expr instanceof ByteLiteral)
        {
            int value = ((BigInteger)((ByteLiteral)expr).getValue()).intValue(); 
            CharacterLiteral literal = new CharacterLiteral(qs, mapping, String.valueOf((char)value));
            return new BooleanExpression(this, OP_LT, literal);
        }                   
        else if (expr instanceof IntegerLiteral)
        {
            int value = ((BigInteger)((IntegerLiteral)expr).getValue()).intValue(); 
            CharacterLiteral literal = new CharacterLiteral(qs, mapping, String.valueOf((char)value));
            return new BooleanExpression(this, OP_LT, literal);
        }
        else if (expr instanceof FloatingPointLiteral)
        {
            int value = ((BigDecimal)((FloatingPointLiteral)expr).getValue()).intValue(); 
            CharacterLiteral literal = new CharacterLiteral(qs, mapping, String.valueOf((char)value));
            return new BooleanExpression(this, OP_LT, literal);
        }
        else if (expr instanceof NumericExpression)
        {
            return new BooleanExpression(this, OP_LT, expr);
        }
        else
        {
            return super.lt(expr);
        }
    }

    public BooleanExpression lteq(ScalarExpression expr)
    {
        if (expr instanceof NullLiteral)
        {
            return expr.lteq(this);
        }
        else if (expr instanceof CharacterLiteral)
        {
            return new BooleanExpression(this, OP_LTEQ, expr);            
        }
        else if (expr instanceof StringExpression)
        {
            return new BooleanExpression(this, OP_LTEQ, expr);   
        }
        else if (expr instanceof ByteLiteral)
        {
            int value = ((BigInteger)((ByteLiteral)expr).getValue()).intValue(); 
            CharacterLiteral literal = new CharacterLiteral(qs, mapping, String.valueOf((char)value));
            return new BooleanExpression(this, OP_LTEQ, literal);
        }                   
        else if (expr instanceof IntegerLiteral)
        {
            int value = ((BigInteger)((IntegerLiteral)expr).getValue()).intValue(); 
            CharacterLiteral literal = new CharacterLiteral(qs, mapping, String.valueOf((char)value));
            return new BooleanExpression(this, OP_LTEQ, literal);
        }
        else if (expr instanceof FloatingPointLiteral)
        {
            int value = ((BigDecimal)((FloatingPointLiteral)expr).getValue()).intValue(); 
            CharacterLiteral literal = new CharacterLiteral(qs, mapping, String.valueOf((char)value));
            return new BooleanExpression(this, OP_LTEQ, literal);
        }
        else if (expr instanceof NumericExpression)
        {
            return new BooleanExpression(this, OP_LTEQ, expr);
        }
        else
        {
            return super.lteq(expr);
        }
    }

    public BooleanExpression gt(ScalarExpression expr)
    {
        if (expr instanceof NullLiteral)
        {
            return expr.gt(this);
        }
        else if (expr instanceof CharacterLiteral)
        {
            return new BooleanExpression(this, OP_GT, expr);            
        }
        else if (expr instanceof StringExpression)
        {
            return new BooleanExpression(this, OP_GT, expr);   
        }
        else if (expr instanceof ByteLiteral)
        {
            int value = ((BigInteger)((ByteLiteral)expr).getValue()).intValue(); 
            CharacterLiteral literal = new CharacterLiteral(qs, mapping, String.valueOf((char)value));
            return new BooleanExpression(this, OP_GT, literal);
        }                   
        else if (expr instanceof IntegerLiteral)
        {
            int value = ((BigInteger)((IntegerLiteral)expr).getValue()).intValue(); 
            CharacterLiteral literal = new CharacterLiteral(qs, mapping, String.valueOf((char)value));
            return new BooleanExpression(this, OP_GT, literal);
        }
        else if (expr instanceof FloatingPointLiteral)
        {
            int value = ((BigDecimal)((FloatingPointLiteral)expr).getValue()).intValue(); 
            CharacterLiteral literal = new CharacterLiteral(qs, mapping, String.valueOf((char)value));
            return new BooleanExpression(this, OP_GT, literal);
        }
        else if (expr instanceof NumericExpression)
        {
            return new BooleanExpression(this, OP_GT, expr);
        }
        else
        {
            return super.gt(expr);
        }
    }

    public BooleanExpression gteq(ScalarExpression expr)
    {
        if (expr instanceof NullLiteral)
        {
            return expr.gteq(this);
        }
        else if (expr instanceof CharacterLiteral)
        {
            return new BooleanExpression(this, OP_GTEQ, expr);            
        }
        else if (expr instanceof StringExpression)
        {
            return new BooleanExpression(this, OP_GTEQ, expr);   
        }
        else if (expr instanceof ByteLiteral)
        {
            int value = ((BigInteger)((ByteLiteral)expr).getValue()).intValue(); 
            CharacterLiteral literal = new CharacterLiteral(qs, mapping, String.valueOf((char)value));
            return new BooleanExpression(this, OP_GTEQ, literal);
        }                   
        else if (expr instanceof IntegerLiteral)
        {
            int value = ((BigInteger)((IntegerLiteral)expr).getValue()).intValue(); 
            CharacterLiteral literal = new CharacterLiteral(qs, mapping, String.valueOf((char)value));
            return new BooleanExpression(this, OP_GTEQ, literal);
        }
        else if (expr instanceof FloatingPointLiteral)
        {
            int value = ((BigDecimal)((FloatingPointLiteral)expr).getValue()).intValue(); 
            CharacterLiteral literal = new CharacterLiteral(qs, mapping, String.valueOf((char)value));
            return new BooleanExpression(this, OP_GTEQ, literal);
        }
        else if (expr instanceof NumericExpression)
        {
            return new BooleanExpression(this, OP_GTEQ, expr);
        }
        else
        {
            return super.gteq(expr);
        }
    }

    public ScalarExpression add(ScalarExpression expr)
    {
        if (expr instanceof StringExpression)
        {
            return new StringExpression(this, OP_CONCAT, expr);
        }
        else if (expr instanceof NumericExpression )
        {
            return new StringExpression(this, OP_CONCAT, new StringExpression(qs,(NumericExpression) expr));
        }      
        else if (expr instanceof NullLiteral)
        {
            return expr;
        }         
        else
        {
            return super.add(expr);
        }
    }
    
    public BooleanExpression in(ScalarExpression expr)
    {
        return new BooleanExpression(this, OP_IN, expr);
    }

    /**
     * Returns whether this string is equal to the expression. This is provided
     * as an operation on String.
     * @param expr The expression to compare against.
     * @return Expression whether they are equal.
     **/
    public BooleanExpression equalsMethod(ScalarExpression expr)
    {
        return eq(expr);
    }

    /**
     * Returns the character at a position.
     * @param index The position of the character required.
     * @return The expression.
     **/
    public StringExpression charAtMethod(ScalarExpression index)
    {
        if (!(index instanceof NumericExpression))
        {
            throw new IllegalArgumentTypeException(index);
        }

        NumericExpression begin = (NumericExpression)index;
        NumericExpression end   = (NumericExpression)begin.add(new IntegerLiteral(qs, mapping, BigInteger.ONE));

        return substringMethod(begin, end);
    }

    /**
     * Returns whether this string ends with the specified string.
     * @param str The string to compare against.
     * @return Whether it ends with the string.
     **/
    public BooleanExpression endsWithMethod(ScalarExpression str)
    {
        if (!(str instanceof StringExpression))
        {
            throw new IllegalArgumentTypeException(str);
        }

        StringLiteral pct = new StringLiteral(qs, mapping, '%');
        
        return new BooleanExpression(this, OP_LIKE, pct.add(((DatabaseAdapter) qs.getStoreManager().getDatastoreAdapter()).getEscapedPatternExpression(str)));
    }

    /**
     * Returns the index within this string of the first occurrence of the
     * specified string.
     * @param str The string to find the index of
     * @return The index of the string.
     **/
    public NumericExpression indexOfMethod(ScalarExpression str)
    {
        if (!(str instanceof StringExpression))
        {
            throw new IllegalArgumentTypeException(str);
        }

        return new IndexOfExpression(this, (StringExpression)str);
    }
    
    /**
     * Returns the index within this string of the first occurrence of the
     * specified character, starting the search at the specified index. 
     * @param str The string to find the index of
     * @param fromIndex the index to start the search from. 
     * @return The index of the string.
     **/
    public NumericExpression indexOfMethod(ScalarExpression str, ScalarExpression fromIndex)
    {
        if (!(str instanceof StringExpression))
        {
            throw new IllegalArgumentTypeException(str);
        }
        if (!(fromIndex instanceof NumericExpression))
        {
            throw new IllegalArgumentTypeException(fromIndex);
        }
        
        return new IndexOfExpression(
                ((DatabaseAdapter) qs.getStoreManager().getDatastoreAdapter()).substringMethod(
                        this,
                        (NumericExpression)fromIndex),
                (StringExpression)str);
    }

    /**
     * Method to handle the starts with operation.
     * @return The expression.
     **/
    public NumericExpression lengthMethod()
    {
        return ((DatabaseAdapter) qs.getStoreManager().getDatastoreAdapter()).lengthMethod(this);
    }

    /**
     * Method to handle the starts with operation.
     * @param str The string to compare against 
     * @return The expression.
     **/
    public BooleanExpression startsWithMethod(ScalarExpression str)
    {
        if (!(str instanceof StringExpression))
        {
            throw new IllegalArgumentTypeException(str);
        }
            
        return ((DatabaseAdapter) qs.getStoreManager().getDatastoreAdapter()).startsWithMethod(this, str);
    }

    /**
     * Method to handle the starts with operation.
     * @param str The string to compare against 
     * @param toffset The offset
     * @return The expression.
     **/
    public BooleanExpression startsWithMethod(ScalarExpression str, ScalarExpression toffset)
    {
        return substringMethod(toffset).startsWithMethod(str);
    }

    /**
     * Method to handle the substring operation.
     * @param begin The start position
     * @return The expression.
     **/
    public StringExpression substringMethod(ScalarExpression begin)
    {
        if (!(begin instanceof NumericExpression))
        {
            throw new IllegalArgumentTypeException(begin);
        }

        return ((DatabaseAdapter) qs.getStoreManager().getDatastoreAdapter()).substringMethod(this,
                                                                         (NumericExpression)begin);
    }

    /**
     * Method to handle the substring operation.
     * @param begin The start position
     * @param end The end position
     * @return The expression.
     **/
    public StringExpression substringMethod(ScalarExpression begin, ScalarExpression end)
    {
        if (!(begin instanceof NumericExpression))
        {
            throw new IllegalArgumentTypeException(begin);
        }
        if (!(end   instanceof NumericExpression))
        {
            throw new IllegalArgumentTypeException(end);
        }

        return ((DatabaseAdapter) qs.getStoreManager().getDatastoreAdapter()).substringMethod(this,
                                                                         (NumericExpression)begin,
                                                                         (NumericExpression)end);
    }

    /**
     * Method to handle the lower case operation.
     * @return The expression.
     **/
    public StringExpression toLowerCaseMethod()
    {
        ArrayList args = new ArrayList();
        args.add(this);

        return new StringExpression("LOWER", args);
    }

    /**
     * Method to handle the upper case operation.
     * @return The expression.
     **/
    public StringExpression toUpperCaseMethod()
    {
        ArrayList args = new ArrayList();
        args.add(this);

        return new StringExpression("UPPER", args);
    }

    /**
     * Matches this to the argument expression pattern. Use "." to find any
     * character and ".*" for wildcard matches. A global case-insensitive flag
     * "(?i)" can be set for the pattern. If used, the global case-insensitive
     * flag must prefix the pattern. The pattern passed to matches must be a
     * literal or parameter.
     * @param expr The literal expression with the pattern.
     * @return the match expression.
     */
    public BooleanExpression matchesMethod(ScalarExpression expr)
    {
        if (expr instanceof StringLiteral)
        {
            expr = ((DatabaseAdapter) qs.getStoreManager().getDatastoreAdapter()).getEscapedPatternExpression(expr);
            String pattern = (String) ((Literal) expr).getValue();
            boolean caseSensitive = false;
            if (pattern.startsWith("(?i)"))
            {
                caseSensitive = true;
                pattern = pattern.substring(4);
            }
            MatchExpressionParser parser = new MatchExpressionParser(pattern, ((DatabaseAdapter) qs.getStoreManager().getDatastoreAdapter())
                    .getPatternExpressionZeroMoreCharacters(), ((DatabaseAdapter) qs.getStoreManager().getDatastoreAdapter()).getPatternExpressionAnyCharacter());
            String parsedPattern = parser.parsePattern();
            if (caseSensitive)
            {
                return new BooleanExpression(this.toLowerCaseMethod(), ScalarExpression.OP_LIKE, new StringLiteral(qs, mapping, parsedPattern).toLowerCase());
            }
            else
            {
                return new BooleanExpression(this, ScalarExpression.OP_LIKE, new StringLiteral(qs, mapping, parsedPattern));
            }
        }
        throw new IllegalOperationException(this, "matches", expr);
    }
    
    /**
     * Method to handle trimming of a string.
     * @return The expression.
     */
    public StringExpression trimMethod()
    {
        return ((DatabaseAdapter) qs.getStoreManager().getDatastoreAdapter()).trimMethod(this);
    }
}