/**********************************************************************
Copyright (c) 2002} Mike Martin (TJDO) and others.
All rights reserved.   This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
2003 Andy Jefferson - coding standards
    ...
**********************************************************************/
package org.jpox.store.expression;

import java.util.HashMap;

import org.jpox.store.DatastoreClass;
import org.jpox.store.DatastoreContainerObject;
import org.jpox.store.DatastoreField;
import org.jpox.store.DatastoreIdentifier;
import org.jpox.store.exceptions.TableMismatchException;
import org.jpox.store.query.QueryStatement;

/**
 * A SQL table expression that joins superclass tables by joining them directly
 * to the surrounding QueryStatement.
 *
 * @see QueryStatement
 * @version $Revision: 1.8 $ 
 */
public class TableExprAsJoins extends TableExpression
{
    protected final HashMap rangeVarsByTable = new HashMap();

    public TableExprAsJoins(QueryStatement qs, DatastoreContainerObject mainTable, DatastoreIdentifier mainRangeVar)
    {
        super(qs, mainTable, mainRangeVar);

        rangeVarsByTable.put(mainTable, mainRangeVar);
    }

    public String referenceColumn(DatastoreField col)
    {
        assertNotFrozen();

        DatastoreContainerObject table = col.getDatastoreContainerObject();
        DatastoreIdentifier rangeVar = (DatastoreIdentifier)rangeVarsByTable.get(table);

        if (rangeVar == null)
        {
            if (!(mainTable instanceof DatastoreClass) || !(table instanceof DatastoreClass))
            {
                new TableMismatchException(col, mainTable);
            }

            /*
             * Since both tables are ClassTables we assume that the column
             * is a superclass field, meaning 'table' is a supertable of
             * 'mainTable'.  We join the supertable to the query statement using
             * range variable derived from the main range variable (an
             * underscore and a counter are appended).
             *
             * So even though the caller may have been looking for column FOO
             * relative to rangevar THIS he may actually get back something like
             * THIS_1.FOO.
             */
            rangeVar = mainRangeVar.getNewIdentifier(String.valueOf(rangeVarsByTable.size()));
            rangeVarsByTable.put(table, rangeVar);

            /* mt... = "main table" */
            /* st... = "supertable" */
            DatastoreClass mt = (DatastoreClass)mainTable;
            DatastoreClass st = (DatastoreClass)table;
            TableExpression stTblExpr = qs.newTableExpression(st, rangeVar);

            ScalarExpression mtExpr = mt.getIDMapping().newScalarExpression(qs,this);
            ScalarExpression stExpr = st.getIDMapping().newScalarExpression(qs,stTblExpr);

            qs.innerJoin(mtExpr,stExpr, stTblExpr, true);
        }
        if (rangeVar.toString().length() > 0)
        {
            return col.applySelectFunction(rangeVar + "." + col.getName());
        }
        else
        {
            return col.applySelectFunction(col.getName().toString());
        }
    }

    public String toString()
    {
        if (sqlText == null)
        {
            DatastoreIdentifier mainTableName = mainTable.getName();
            StringBuffer sb = new StringBuffer(mainTable.toString());
            if (!mainRangeVar.equals(mainTableName))
            {
                sb.append(' ').append(mainRangeVar);
            }

            sqlText = sb.toString();
        }

        return sqlText;
    }
}