/**********************************************************************
Copyright (c) 2002 Mike Martin (TJDO) and others.
All rights reserved.   This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
2003 Andy Jefferson - coding standards
    ...
**********************************************************************/
package org.jpox.store.expression;

import java.util.ArrayList;
import java.util.Iterator;

import org.jpox.store.DatastoreClass;
import org.jpox.store.DatastoreContainerObject;
import org.jpox.store.DatastoreField;
import org.jpox.store.DatastoreIdentifier;
import org.jpox.store.DatastoreObject;
import org.jpox.store.exceptions.TableMismatchException;
import org.jpox.store.query.QueryStatement;

/**
 * A SQL table expression that joins superclass tables by constructing a
 * parenthesized set of subjoins.
 *
 * @see QueryStatement
 * @version $Revision: 1.7 $ 
 */
public class TableExprAsSubjoins extends TableExpression
{
    protected final ArrayList supertables = new ArrayList();

    public TableExprAsSubjoins(QueryStatement qs, DatastoreContainerObject mainTable, DatastoreIdentifier mainRangeVar)
    {
        super(qs, mainTable, mainRangeVar);
    }

    public String referenceColumn(DatastoreField col)
    {
        assertNotFrozen();

        DatastoreObject table = col.getDatastoreContainerObject();

        if (!table.equals(mainTable))
        {
            if (!(mainTable instanceof DatastoreClass) || !(table instanceof DatastoreClass))
            {
                throw new TableMismatchException(col, mainTable);
            }

            /*
             * Since both tables are ClassTables we assume that the column
             * is a superclass field, meaning 'table' is a supertable of
             * 'mainTable'.  We add it to the list of supertables that will be
             * joined in when the statement text is constructed.
             */
            if (!(supertables.contains(table)))
            {
                supertables.add(table);
            }
        }

        if( mainRangeVar.toString().length() > 0)
        {
            return col.applySelectFunction(mainRangeVar + "." + col.getName());
        }
        else
        {
            return col.applySelectFunction(col.getName().toString());
        }
    }

    public String toString()
    {
        if (sqlText == null)
        {
            StringBuffer sb = new StringBuffer();
            DatastoreIdentifier mainTableName = mainTable.getName();

            Iterator i = supertables.iterator();
            if (i.hasNext())
            {
                sb.append('(').append(mainTable.toString());

                while (i.hasNext())
                {
                    DatastoreClass supertable = (DatastoreClass)i.next();

					sb.append(" INNER JOIN ").append(supertable.toString());
					sb.append(" ON ");
					for (int j=0; j<((DatastoreClass) mainTable).getIDMapping().getNumberOfDatastoreFields(); j++)
					{
						DatastoreIdentifier mainTableIDColumnName = ((DatastoreClass)mainTable).getIDMapping().getDataStoreMapping(j).getDatastoreField().getName();
						if (j > 0)
						{
							sb.append(" AND ");
						}
						sb.append(mainTable.toString()).append('.').append(mainTableIDColumnName);
						sb.append(" = ");
						sb.append(supertable.toString()).append('.').append(supertable.getIDMapping().getDataStoreMapping(j).getDatastoreField().getName());
					}
                }

                sb.append(") ").append(mainRangeVar);
            }
            else
            {
                sb.append(mainTable.toString());
                if (!mainRangeVar.equals(mainTableName))
                {
                    sb.append(' ').append(mainRangeVar);
                }
            }

            sqlText = sb.toString();
        }

        return sqlText;
    }
}