/**********************************************************************
Copyright (c) 2002 Mike Martin (TJDO) and others.
All rights reserved.   This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
2003 Andy Jefferson - coding standards
    ...
**********************************************************************/
package org.jpox.store.expression;

import java.util.ArrayList;
import java.util.Iterator;

import org.jpox.store.DatastoreClass;
import org.jpox.store.DatastoreContainerObject;
import org.jpox.store.DatastoreField;
import org.jpox.store.DatastoreIdentifier;
import org.jpox.store.DatastoreObject;
import org.jpox.store.FetchStatement;
import org.jpox.store.exceptions.TableMismatchException;
import org.jpox.store.query.QueryStatement;
import org.jpox.store.rdbms.RDBMSFetchStatement;

/**
 * A SQL table expression that joins superclass tables by constructing a
 * parenthesized sub-SELECT statement.
 *
 * @see QueryStatement
 * @version $Revision: 1.8 $ 
 */
public class TableExprAsSubquery extends TableExpression
{
    protected final ArrayList columns = new ArrayList();
    protected boolean multipleTablesReferenced = false;

    public TableExprAsSubquery(QueryStatement qs, DatastoreContainerObject mainTable, DatastoreIdentifier mainRangeVar)
    {
        super(qs, mainTable, mainRangeVar);
    }

    public String referenceColumn(DatastoreField col)
    {
        assertNotFrozen();

        DatastoreObject table = col.getDatastoreContainerObject();

        if (!table.equals(mainTable))
        {
            if (!(mainTable instanceof DatastoreClass) || !(table instanceof DatastoreClass))
            {
                throw new TableMismatchException(col, mainTable);
            }

            /*
             * Since both tables are DatastoreClass we assume that the column
             * is a superclass field, meaning 'table' is a supertable of
             * 'mainTable'.  We set the flag indicating that this expression
             * must become a subquery that joins the necessary tables.
             */

            multipleTablesReferenced = true;
        }

        if (!columns.contains(col))
        {
            columns.add(col);
        }

        if (mainRangeVar.toString().length() > 0)
        {
            return col.applySelectFunction(mainRangeVar + "." + col.getName());
        }
        else
        {
            return col.applySelectFunction(col.getName().toString());
        }
    }

    public String toString()
    {
        if (sqlText == null)
        {
            StringBuffer sb = new StringBuffer();
            DatastoreIdentifier mainTableName = mainTable.getName();

            if (!multipleTablesReferenced)
            {
                sb.append(mainTable.toString());
                if (!mainRangeVar.equals(mainTableName))
                {
                    sb.append(' ').append(mainRangeVar);
                }
            }
            else
            {
                FetchStatement subQuery = new RDBMSFetchStatement((DatastoreClass) mainTable);

                Iterator i = columns.iterator();
                while (i.hasNext())
                {
                    subQuery.select((DatastoreField)i.next());
                }

                // Assumed to not want any locking in this statement
                sb.append('(').append(subQuery.toString(false)).append(") ").append(mainRangeVar);
            }

            sqlText = sb.toString();
        }

        return sqlText;
    }
}