/**********************************************************************
Copyright (c) 2002 Mike Martin (TJDO) and others.
All rights reserved.   This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
2003 Erik Bengtson - fixed bug [833915] QueryResult passed as parameter
    				for another Query
2003 Andy Jefferson - coding standards
    ...
**********************************************************************/
package org.jpox.store.expression;

import javax.jdo.JDOFatalInternalException;
import javax.jdo.JDOUserException;

import org.jpox.store.DatastoreClass;
import org.jpox.store.DatastoreContainerObject;
import org.jpox.store.DatastoreField;
import org.jpox.store.DatastoreIdentifier;
import org.jpox.store.DatastoreObject;
import org.jpox.store.StoreManager;
import org.jpox.store.mapping.JavaTypeMapping;
import org.jpox.store.query.QueryStatement;

/**
 * Represents a SQL table expression as might be listed in the FROM clause of
 * a SELECT statement.
 * A table expression is a fragment of a larger containing QueryStatement.
 * <p>
 * A table expression has a base "main" table.
 * If that table serves as backing for a Java class, and that class has
 * persistence-capable superclasses, then the table expression may include
 * joins to superclass tables, or may cause such joins to occur in its
 * surrounding QueryStatement.
 *
 * @see QueryStatement
 * @version $Revision: 1.8 $ 
 */
public abstract class TableExpression
{
    protected final QueryStatement qs;
    protected final DatastoreContainerObject mainTable;
    protected final DatastoreIdentifier mainRangeVar;
    protected final StoreManager storeMgr;
    protected String sqlText = null;

    protected TableExpression(QueryStatement qs, DatastoreContainerObject mainTable, DatastoreIdentifier mainRangeVar)
    {
        this.qs           = qs;
        this.mainTable    = mainTable;
        this.mainRangeVar = mainRangeVar;

        storeMgr = mainTable.getStoreManager();
    }

    protected void assertNotFrozen()
    {
        if (sqlText != null)
        {
            throw new JDOFatalInternalException("A table expression cannot be modified after being output");
        }
    }

    public final DatastoreObject getMainTable()
    {
        return mainTable;
    }

    public final DatastoreIdentifier getRangeVariable()
    {
        return mainRangeVar;
    }

    public ScalarExpression newFieldExpression(String fieldName)
    {
        if (!(mainTable instanceof DatastoreClass))
        {
            throw new JDOUserException("'" + fieldName + "' can't be referenced in " +
                   mainTable.toString() + ": table does not store a persistence-capable class");
        }

        DatastoreClass ct = (DatastoreClass) mainTable;
        JavaTypeMapping m;

        if (fieldName.equals("this"))
        {
            m = ct.getIDMapping();
            return m.newScalarExpression(qs, this);
        }
        else
        {
            m = ct.getFieldMapping(fieldName);
            ScalarExpression expr = m.newScalarExpression(qs, this);
            if( expr instanceof ObjectExpression )
            {
                ((ObjectExpression)expr).setFieldName(fieldName);
            }
            return expr;
        }
    }

    public abstract String referenceColumn(DatastoreField col);

    public abstract String toString();
}
