/**********************************************************************
Copyright (c) 2002 Mike Martin (TJDO) and others.
All rights reserved.   This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
2003 Andy Jefferson - coding standards
    ...
**********************************************************************/
package org.jpox.store.expression;

import javax.jdo.JDOUserException;

import org.jpox.store.DatastoreAdapter;
import org.jpox.store.DatastoreClass;
import org.jpox.store.StatementText;
import org.jpox.store.mapping.JavaTypeMapping;
import org.jpox.store.query.JDOQLQuery;
import org.jpox.store.query.QueryStatement;
import org.jpox.store.rdbms.sqlidentifier.TableIdentifier;

/**
 * Representation of an unbound variable in a Query.
 *
 * @version $Revision: 1.6 $
 **/
public class UnboundVariable extends ScalarExpression
{
    private final String name;
    private final Class type;
    private final JDOQLQuery.Compiler compiler;

    public UnboundVariable(QueryStatement qs, String name, Class type, JDOQLQuery.Compiler compiler)
    {
        super(qs);

        this.name = name;
        this.type = type;
        this.compiler = compiler;
    }

    public String getVariableName()
    {
        return name;
    }

    public Class getVariableType()
    {
        return type;
    }

    public void bindTo(ScalarExpression qsc)
    {
        compiler.bindVariable(name, qsc);
    }

    public StatementText toStatementText()
    {
        throw new JDOUserException("Unconstrained variable referenced: " + name);
    }
    
    /**
     * Checks if one expression is equals the other expression
     * @param expr the expression to check if this is equals to the expression
     */
    public BooleanExpression eq(ScalarExpression expr)
    {
        ScalarExpression boundExpr = getExpressionBindedToThis();
        qs.innerJoin(expr,boundExpr,boundExpr.te, true);
        
        return expr.eq(boundExpr);
    }
    
    /**
     * Checks if one expression is not equals the other expression
     * @param expr the expression to check if this is not equals to the expression
     */
    public BooleanExpression noteq(ScalarExpression expr)
    {
        ScalarExpression boundExpr = getExpressionBindedToThis();
        qs.innerJoin(expr,boundExpr,boundExpr.te, true);
        
        return expr.noteq(boundExpr);
    }
    
    private ScalarExpression getExpressionBindedToThis()
    {
        UnboundVariable var = this;
        //String etJavaName = qs.getDefaultTableExpression().getRangeVariable().getJavaName() + '.' + var.getVariableName();
        String jtJavaName = "UNBOUND" + '.' + var.getVariableName();

        DatastoreAdapter dba = qs.getStoreManager().getDatastoreAdapter();
        //ClassMetaData cmd = storeMgr.getMetaDataManager().getMetaDataForClass(type);
        DatastoreClass cbt = qs.getStoreManager().getDatastoreClass(type.getName());

        TableIdentifier jtRangeVar = new TableIdentifier(dba, jtJavaName);
        TableExpression jtExpr = qs.getTableExpression(jtRangeVar);

        if( jtExpr == null )
        {
            jtExpr = qs.newTableExpression(cbt, jtRangeVar);
            qs.addAlias(jtExpr, true);
        }

        //bind unbond variable
        JavaTypeMapping mb = cbt.getIDMapping();
        
        ScalarExpression exprBindTo = mb.newScalarExpression(qs, jtExpr);
        bindTo(exprBindTo);
        
        return exprBindTo;
    }
    
    public ScalarExpression accessField(String fieldName, boolean innerJoin)
    {
        UnboundVariable var = this;
        //String etJavaName = qs.getDefaultTableExpression().getRangeVariable().getJavaName() + '.' + var.getVariableName();
        String jtJavaName = "UNBOUND" + '.' + var.getVariableName();

        DatastoreAdapter dba = qs.getStoreManager().getDatastoreAdapter();
        //ClassMetaData cmd = storeMgr.getMetaDataManager().getMetaDataForClass(type);
        DatastoreClass cbt = qs.getStoreManager().getDatastoreClass(type.getName());

        TableIdentifier jtRangeVar = new TableIdentifier(dba, jtJavaName);
        TableExpression jtExpr = qs.getTableExpression(jtRangeVar);

        if( jtExpr == null )
        {
            jtExpr = qs.newTableExpression(cbt, jtRangeVar);
            qs.addAlias(jtExpr, true);
        }

        //bind unbound variable
        JavaTypeMapping mb = cbt.getIDMapping();
        
        ScalarExpression exprBindTo = mb.newScalarExpression(qs, jtExpr);
        bindTo(exprBindTo);

        return exprBindTo.accessField(fieldName,innerJoin);
    }
        
}
