/**********************************************************************
Copyright (c) 2003 Erik Bengtson and others. All rights reserved. 
This program and the accompanying materials are made available under
the terms of the JPOX License v1.0 which accompanies this distribution. 

Contributors:
2003 Andy Jefferson - coding standards
	...
**********************************************************************/
package org.jpox.store.mapping;

import java.sql.PreparedStatement;
import java.sql.SQLException;

import javax.jdo.JDODataStoreException;
import javax.jdo.spi.PersistenceCapable;
import javax.jdo.spi.PersistenceCapable.ObjectIdFieldConsumer;

import org.jpox.util.Localiser;

/**
 * Parameter setter class to deal with application identity.
 * Parameters must be in the same order as the fields.
 * 
 * TODO to support other types of fields in the application id fields, 
 * we must delegate the store method tasks to a Mapping...
 *
 * @version $Revision: 1.1 $ 
 */
class AppIDObjectIdFieldConsumer implements ObjectIdFieldConsumer
{
	protected static final Localiser LOCALISER=Localiser.getInstance("org.jpox.store.rdbms.mapping.Localisation");
	
    private PreparedStatement ps;
    /**
     * the position of the parameter in the ps. 
     * Having composed keys, we should just increase it
     */
    private int[] param;
    // the next parameter index
    private int next;
    public AppIDObjectIdFieldConsumer(PreparedStatement ps, int[] param)
    {
        this.ps = ps;
        this.param = param;
        next = 0;
    }
    public void storeBooleanField(int fieldNumber, boolean value)
    {
        try
        {
            ps.setBoolean(param[next], value);
        }
        catch (SQLException e)
        {
            throw new JDODataStoreException(LOCALISER.msg("Mapping.CannotSetAppIDParameter", String.valueOf(value)), e);
        }
    }

    public void storeCharField(int fieldNumber, char value)
    {
        try
        {
            ps.setString(param[next], String.valueOf(value));
        }
        catch (SQLException e)
        {
            throw new JDODataStoreException(LOCALISER.msg("Mapping.CannotSetAppIDParameter", String.valueOf(value)), e);
        }
    }

    public void storeByteField(int fieldNumber, byte value)
    {
        try
        {
            ps.setByte(param[next++], value);
        }
        catch (SQLException e)
        {
            throw new JDODataStoreException(LOCALISER.msg("Mapping.CannotSetAppIDParameter", String.valueOf(value)), e);
        }
    }

    public void storeShortField(int fieldNumber, short value)
    {
        try
        {
            ps.setShort(param[next++], value);
        }
        catch (SQLException e)
        {
            throw new JDODataStoreException(LOCALISER.msg("Mapping.CannotSetAppIDParameter", String.valueOf(value)), e);
        }
    }

    public void storeIntField(int fieldNumber, int value)
    {
        try
        {
            ps.setInt(param[next++], value);
        }
        catch (SQLException e)
        {
            throw new JDODataStoreException(LOCALISER.msg("Mapping.CannotSetAppIDParameter", String.valueOf(value)), e);
        }
    }

    public void storeLongField(int fieldNumber, long value)
    {
        try
        {
            ps.setLong(param[next++], value);
        }
        catch (SQLException e)
        {
            throw new JDODataStoreException(LOCALISER.msg("Mapping.CannotSetAppIDParameter", String.valueOf(value)), e);
        }
    }

    public void storeFloatField(int fieldNumber, float value)
    {
        try
        {
            ps.setFloat(param[next++], value);
        }
        catch (SQLException e)
        {
            throw new JDODataStoreException(LOCALISER.msg("Mapping.CannotSetAppIDParameter", String.valueOf(value)), e);
        }
    }

    public void storeDoubleField(int fieldNumber, double value)
    {
        try
        {
            ps.setDouble(param[next++], value);
        }
        catch (SQLException e)
        {
            throw new JDODataStoreException(LOCALISER.msg("Mapping.CannotSetAppIDParameter", String.valueOf(value)), e);
        }
    }

    public void storeStringField(int fieldNumber, String value)
    {
        try
        {
            ps.setString(param[next++], value);
        }
        catch (SQLException e)
        {
            throw new JDODataStoreException(LOCALISER.msg("Mapping.CannotSetAppIDParameter", String.valueOf(value)), e);
        }
    }

    public void storeObjectField(int fieldNumber, Object value)
    {
        try
        {
            //this is used in indentifying relationships
            if( value instanceof PersistenceCapable )
            {
                PersistenceCapable pc = (PersistenceCapable) value;
				pc.jdoCopyKeyFieldsFromObjectId(this, pc.jdoGetObjectId());
            }
            else
            {
                ps.setObject(param[next++], value);
            }
        }
        catch (SQLException e)
        {
            throw new JDODataStoreException(LOCALISER.msg("Mapping.CannotSetAppIDParameter", String.valueOf(value)), e);
        }
    }
}
