/**********************************************************************
Copyright (c) 2004 Erik Bengtson and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution.

Contributors:
    ...
**********************************************************************/
package org.jpox.store.mapping;

import java.math.BigDecimal;

import org.jpox.metadata.FieldMetaData;
import org.jpox.store.DatastoreAdapter;
import org.jpox.store.DatastoreClass;
import org.jpox.store.expression.FloatingPointLiteral;
import org.jpox.store.expression.NumericExpression;
import org.jpox.store.expression.ScalarExpression;
import org.jpox.store.expression.TableExpression;
import org.jpox.store.query.QueryStatement;

/**
 * Mapping for BigDecimal type.
 *
 * @version $Revision: 1.2 $
 **/
public class BigDecimalMapping extends SingleFieldMapping implements SimpleDatastoreRepresentation
{
	private static BigDecimal mappingSampleValue = new BigDecimal(0);
	
    public BigDecimalMapping(DatastoreAdapter dba, Class type)
    {
        super(dba, type);
    }

    public BigDecimalMapping(DatastoreAdapter dba, FieldMetaData fmd, DatastoreClass table)
    {
        super(dba, fmd, table);
    }

	public Object getSampleValue()
	{
		return mappingSampleValue;
	}

    public ScalarExpression newLiteral(QueryStatement qs, Object value)
    {
        ScalarExpression expr = new FloatingPointLiteral(qs, (BigDecimal)value);
        return expr; 
    }

    public ScalarExpression newScalarExpression(QueryStatement qs, TableExpression te)
    {
        ScalarExpression expr = new NumericExpression(qs, this, te);
        return expr; 
    }

    public Class getJavaType()
    {
        return BigDecimal.class;
    }
}
