/**********************************************************************
Copyright (c) 27-Sep-2004 Erik Bengtson and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution.

Contributors:
    ...
**********************************************************************/
package org.jpox.store.mapping;

import java.math.BigInteger;

import org.jpox.metadata.FieldMetaData;
import org.jpox.store.DatastoreAdapter;
import org.jpox.store.DatastoreClass;
import org.jpox.store.expression.IntegerLiteral;
import org.jpox.store.expression.NumericExpression;
import org.jpox.store.expression.ScalarExpression;
import org.jpox.store.expression.TableExpression;
import org.jpox.store.query.QueryStatement;

/**
 * Mapping for BigInteger type.
 *
 * @version $Revision: 1.2 $
 **/
public class BigIntegerMapping extends SingleFieldMapping implements SimpleDatastoreRepresentation
{
	private static BigInteger mappingSampleValue = new BigInteger("0");
		
    public BigIntegerMapping(DatastoreAdapter dba, Class type)
    {
        super(dba, type);
    }

    public BigIntegerMapping(DatastoreAdapter dba, FieldMetaData fmd, DatastoreClass table)
    {
        super(dba, fmd, table);
    }

	public Object getSampleValue()
	{
		return mappingSampleValue;
	}
	
    public ScalarExpression newLiteral(QueryStatement qs, Object value)
    {
        ScalarExpression expr = new IntegerLiteral(qs, this, (BigInteger)value);
        return expr; 
    }

    public ScalarExpression newScalarExpression(QueryStatement qs, TableExpression te)
    {
        ScalarExpression expr = new NumericExpression(qs, this, te);
        return expr; 
    }

    public Class getJavaType()
    {
        return BigInteger.class;
    }
}
