/**********************************************************************
Copyright (c) 2002 Kelly Grizzle (TJDO) and others. All rights reserved. 
This program and the accompanying materials are made available under 
the terms of the JPOX License v1.0 which accompanies this distribution. 

Contributors:
2002 Mike Martin (TJDO)
2004 Andy Jefferson - removed dependency on JDK 1.4 (Boolean.valueOf(boolean))
2004 Andy Jefferson - updated type handling to use datastoreMappings
    ...
**********************************************************************/
package org.jpox.store.mapping;

import java.sql.Types;

import org.jpox.metadata.FieldMetaData;
import org.jpox.store.DatastoreAdapter;
import org.jpox.store.DatastoreClass;
import org.jpox.store.expression.BooleanBitColumnExpression;
import org.jpox.store.expression.BooleanBitColumnLiteral;
import org.jpox.store.expression.BooleanCharColumnExpression;
import org.jpox.store.expression.BooleanCharColumnLiteral;
import org.jpox.store.expression.BooleanExpression;
import org.jpox.store.expression.BooleanLiteral;
import org.jpox.store.expression.ScalarExpression;
import org.jpox.store.expression.TableExpression;
import org.jpox.store.query.QueryStatement;
import org.jpox.store.rdbms.mapping.RDBMSMapping;

/**
 * Mapping of Java Boolean object.
 *
 * @version $Revision: 1.3 $
 **/
public class BooleanMapping extends SingleFieldMapping implements SimpleDatastoreRepresentation
{
    /** Some datastores (e.g Postgresql) treat BIT types as boolean internally. */
    private boolean bitIsReallyBoolean = getDatabaseAdapter().isBitReallyBoolean();

    private static Boolean mappingSampleValue = Boolean.TRUE;

    public BooleanMapping(DatastoreAdapter dba, Class type)
    {
        super(dba, type);
    }

    public BooleanMapping(DatastoreAdapter dba, FieldMetaData fmd, DatastoreClass table)
    {
        super(dba, fmd, table);
    }

    public Object getSampleValue()
    {
        return mappingSampleValue;
    }

    public ScalarExpression newLiteral(QueryStatement qs, Object value)
    {
        ScalarExpression expr;
        RDBMSMapping data_mapping = (RDBMSMapping)datastoreMappings.get(0);
        switch (data_mapping.getTypeInfo().dataType)
        {
            case Types.BIT:
                if (bitIsReallyBoolean)
                {
                    expr = new BooleanLiteral(qs, this, ((Boolean)value).booleanValue());
                }
                else
                {
                    expr = new BooleanBitColumnLiteral(qs, this, ((Boolean)value).booleanValue());
                }
            	break;
            case Types.BOOLEAN: // JDBC 3.0
                expr = new BooleanLiteral(qs, this, ((Boolean)value).booleanValue());
                break;
            case Types.CHAR:
                expr = new BooleanCharColumnLiteral(qs, this, ((Boolean)value).booleanValue());
            	break;
            default:
                expr = new BooleanLiteral(qs, this, ((Boolean)value).booleanValue());
        }

        return expr;
    }

    public ScalarExpression newScalarExpression(QueryStatement qs, TableExpression te)
    {
        ScalarExpression expr;
        RDBMSMapping data_mapping = (RDBMSMapping)datastoreMappings.get(0);
        switch (data_mapping.getTypeInfo().dataType)
        {
            case Types.BIT:
                if (bitIsReallyBoolean)
                {
                    expr = new BooleanExpression(qs, this, te);
                }
                else
                {
                    expr = new BooleanBitColumnExpression(qs, this, te);
                }
            	break;
            case Types.BOOLEAN: // JDBC 3.0
                expr = new BooleanExpression(qs, this, te);
                break;
            case Types.CHAR:
                expr = new BooleanCharColumnExpression(qs, this, te);
            	break;
            default:
                expr = new BooleanExpression(qs, this, te);
        }

        return expr; 
    }

    /**
     * Accessor for the Java type being represented here.
     * @return The Java type
     */
    public Class getJavaType()
    {
        return Boolean.class;
    }
}