/**********************************************************************
Copyright (c) 2002 Kelly Grizzle (TJDO) and others. All rights reserved. 
This program and the accompanying materials are made available under 
the terms of the JPOX License v1.0 which accompanies this distribution. 

Contributors:
2002 Mike Martin (TJDO)
2003 Andy Jefferson - coding standards
2004 Erik Bengtson - included a check "is null" on getObject method 
    ...
**********************************************************************/
package org.jpox.store.mapping;

import org.jpox.metadata.FieldMetaData;
import org.jpox.store.DatastoreAdapter;
import org.jpox.store.DatastoreClass;
import org.jpox.store.expression.CharacterExpression;
import org.jpox.store.expression.CharacterLiteral;
import org.jpox.store.expression.ScalarExpression;
import org.jpox.store.expression.TableExpression;
import org.jpox.store.query.QueryStatement;

/**
 * Mapping for Character type.
 * Note, this mapping uses INT within the datastore instead of CHAR.
 * This is to facilitate easier greater than, less than operations within
 * queries etc. 
 *
 * @version $Revision: 1.2 $
 **/
public class CharacterMapping extends SingleFieldMapping implements SimpleDatastoreRepresentation
{
    private static Character mappingSampleValue = new Character('0');
    
    public CharacterMapping(DatastoreAdapter dba, Class type)
    {
        super(dba, type);
    }

    public CharacterMapping(DatastoreAdapter dba, FieldMetaData fmd, DatastoreClass table)
    {
		super(dba, fmd, table);
	}

    public Object getSampleValue()
    {
        return mappingSampleValue;
    }

    public ScalarExpression newLiteral(QueryStatement qs, Object value)
    {
        ScalarExpression expr = new CharacterLiteral(qs, this, ((Character)value).toString());
        return expr; 
    }

    public ScalarExpression newScalarExpression(QueryStatement qs, TableExpression te)
    {
        ScalarExpression expr = new CharacterExpression(qs, this, te);
        return expr; 
    }

    public Class getJavaType()
    {
        return Character.class;
    }
}
