/**********************************************************************
Copyright (c) 2004 Erik Bengtson and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
    Andy Jefferson - removed getUpdateInputParameter,getInsertionInputParameter
    ...
**********************************************************************/
package org.jpox.store.mapping;

import java.math.BigInteger;

import org.jpox.metadata.FieldMetaData;
import org.jpox.store.DatastoreAdapter;
import org.jpox.store.DatastoreClass;
import org.jpox.store.expression.IntegerLiteral;
import org.jpox.store.expression.NumericExpression;
import org.jpox.store.expression.ScalarExpression;
import org.jpox.store.expression.TableExpression;
import org.jpox.store.query.QueryStatement;

/**
 * Mapping for Index Columns.
 *
 * @version $Revision: 1.3 $
 **/
public class IndexMapping extends SingleFieldMapping implements SimpleDatastoreRepresentation
{
    private static Integer mappingSampleValue = new Integer(0);

    /**
     * Constructor.
     * @param dba Datastore Adapter
     * @param type Type of index field
     */
    public IndexMapping(DatastoreAdapter dba, Class type)
    {
        super(dba, type);
    }

    /**
     * Constructor.
     * @param dba Datastore Adapter
     * @param fmd FieldMetaData for the field
     * @param table The table
     */
    public IndexMapping(DatastoreAdapter dba, FieldMetaData fmd, DatastoreClass table)
    {
        super(dba, fmd, table);
    }

    /**
     * Accessor for a sample value for this field
     * @return Sample field value
     */
    public Object getSampleValue()
    {
        return mappingSampleValue;
    }

    /**
     * Accessor for a new literal for this mapping.
     * @param qs The QueryStatement
     * @param value The value of the object
     * @return The new literal
     */
    public ScalarExpression newLiteral(QueryStatement qs, Object value)
    {
        return new IntegerLiteral(qs, this, BigInteger.valueOf(((Integer)value).longValue()));
    }

    /**
     * Accessor for a new literal for this mapping.
     * @param qs The QueryStatement
     * @param te The table Expression
     * @return The new literal
     */
    public ScalarExpression newScalarExpression(QueryStatement qs, TableExpression te)
    {
        return new NumericExpression(qs, this, te);
    }

    /**
     * Accessor for whether to include this column in any fetch statement
     * @return Whether to include the column when fetching.
     */
    public boolean includeInFetchStatement()
    {
        return false;
    }

    /**
     * Accessor for the type represented here, returning the class itself
     * @return This class.
     */
    public Class getJavaType()
    {
        return Integer.class;
    }
}