/**********************************************************************
Copyright (c) 2002 Kelly Grizzle (TJDO) and others. All rights reserved. 
This program and the accompanying materials are made available under 
the terms of the JPOX License v1.0 which accompanies this distribution. 

Contributors:
2002 Mike Martin (TJDO)
2003 Andy Jefferson - coding standards
    ...
**********************************************************************/
package org.jpox.store.mapping;

import java.math.BigInteger;

import org.jpox.metadata.FieldMetaData;
import org.jpox.store.DatastoreAdapter;
import org.jpox.store.DatastoreClass;
import org.jpox.store.expression.IntegerLiteral;
import org.jpox.store.expression.NumericExpression;
import org.jpox.store.expression.ScalarExpression;
import org.jpox.store.expression.TableExpression;
import org.jpox.store.query.QueryStatement;

/**
 * Mapping for Integer type.
 *
 * @version $Revision: 1.2 $
 **/
public class IntegerMapping extends SingleFieldMapping implements SimpleDatastoreRepresentation
{
    private static Integer mappingSampleValue = new Integer(0);

    public IntegerMapping(DatastoreAdapter dba, Class type)
    {
        super(dba, type);
    }

    public IntegerMapping(DatastoreAdapter dba, FieldMetaData fmd, DatastoreClass table)
    {
		super(dba, fmd, table);
	}

    public Object getSampleValue()
    {
        return mappingSampleValue;
    }

    public ScalarExpression newLiteral(QueryStatement qs, Object value)
    {
        ScalarExpression expr = new IntegerLiteral(qs, this, BigInteger.valueOf(((Integer)value).longValue()));
        return expr; 
    }

    public ScalarExpression newScalarExpression(QueryStatement qs, TableExpression te)
    {
        ScalarExpression expr = new NumericExpression(qs, this, te);
        return expr; 
    }

    /* (non-Javadoc)
     * @see org.jpox.store.mapping.JavaTypeMapping#getJavaType()
     */
    public Class getJavaType()
    {
        return Integer.class;
    }
}
