/**********************************************************************
Copyright (c) 2002 Kelly Grizzle (TJDO) and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
2002 Mike Martin (TJDO)
2003 Erik Bengtson - removed unused import
2003 Andy Jefferson - coding standards
2003 Andy Jefferson - updated setObject to use all input "params"
2004 Andy Jefferson - fixes to allow full use of Long/String OIDs
    ...
**********************************************************************/
package org.jpox.store.mapping;

import javax.jdo.spi.PersistenceCapable;

import org.jpox.PersistenceManager;
import org.jpox.metadata.FieldMetaData;
import org.jpox.store.DatastoreAdapter;
import org.jpox.store.DatastoreClass;
import org.jpox.store.OID;
import org.jpox.store.expression.ObjectExpression;
import org.jpox.store.expression.ObjectLiteral;
import org.jpox.store.expression.ScalarExpression;
import org.jpox.store.expression.TableExpression;
import org.jpox.store.query.QueryStatement;
import org.jpox.util.JPOXLogger;

/**
 * Mapping for OID type.
 *
 * @version $Revision: 1.2 $
 **/
public class  OIDMapping extends SingleFieldMapping implements SimpleDatastoreRepresentation
{
    /**
     * Constructor.
     * @param dba Datastore Adapter
     * @param type Type represented
     */
    public OIDMapping(DatastoreAdapter dba, Class type)
    {
        super(dba, type);
    }

    /**
     * Constructor.
     * @param dba Datastore Adapter
     * @param fmd FieldMetaData for the field
     * @param table Table for persisting this field
     */
    public OIDMapping(DatastoreAdapter dba, FieldMetaData fmd, DatastoreClass table)
    {
        super(dba, fmd, table);
    }

    /**
     * Mutator for the OID in the datastore
     * @param pm The Persistence Manager managing this object
     * @param ps The Prepared Statement
     * @param param Param numbers in the PreparedStatement for this object
     * @param value The OID value to use
     */
    public void setObject(PersistenceManager pm, Object ps, int[] param, Object value)
    {
        if (value == null)
        {
            getDataStoreMapping(0).setObject(ps,param[0],null);
        }
        else
        {
            OID oid;
            if (value instanceof PersistenceCapable)
            {
                oid = (OID) ((PersistenceCapable)value).jdoGetObjectId();
                if (oid == null)
                {
                    pm.makePersistent(value);
                    pm.flush();
                }
                oid = (OID) ((PersistenceCapable)value).jdoGetObjectId();
            }
            else
            {
                oid = (OID) value;
            }

            try
            {
                // Try as a Long
                getDataStoreMapping(0).setObject(ps,param[0],new Long(oid.keyValue()));
            }
            catch (Exception e)
            {
                // Must be a String
                getDataStoreMapping(0).setObject(ps,param[0],new String(oid.keyValue()));
            }
        }
    }

    /**
     * Accessor for the OID object from the result set
     * @param pm Persistence Manager managing this object
     * @param rs The ResultSet
     * @param param Array of param numbers for this object
     * @return The OID object
     */
    public Object getObject(PersistenceManager pm, Object rs, int[] param)
    {
        Object value;
        value = getDataStoreMapping(0).getObject(rs,param[0]);
        if (value != null)
        {
            value = new OID(type.getName(), value);
            if (JPOXLogger.RDBMS.isDebugEnabled())
            {
                JPOXLogger.RDBMS.debug(LOCALISER.msg("OIDMapping.GetObject",value));
            }
        }
        else
        {
            JPOXLogger.RDBMS.warn(LOCALISER.msg("OIDMapping.GetObjectError"));
        }

        return value;
    }

    public Object getSampleValue()
    {
        return "NO SAMPLE AVAILABLE";
    }

    public ScalarExpression newLiteral(QueryStatement qs, Object value)
    {
       ScalarExpression expr = new ObjectLiteral(qs, this, value, this.type);
       return expr;        
    }

    public ScalarExpression newScalarExpression(QueryStatement qs, TableExpression te)
    {
        ScalarExpression expr = new ObjectExpression(qs, this, te);
        return expr;        
    }

    public Class getJavaType()
    {
        return OID.class;
    }
}