/**********************************************************************
Copyright (c) 02-Aug-2004 Brendan de Beer and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
    ...
**********************************************************************/
package org.jpox.store.mapping;

import java.sql.ResultSet;
import java.sql.SQLException;

import javax.jdo.JDODataStoreException;

import org.jpox.PersistenceManager;
import org.jpox.metadata.FieldMetaData;
import org.jpox.store.DatastoreAdapter;
import org.jpox.store.DatastoreClass;

/**
 * Mapping for an array of shorts. 
 * This Mapping converts the short[] to and from a byte[] and stores the byte[]
 * array in the datastore.
 *
 * @version $Revision: 1.1 $
 */
public class ShortArrayMapping extends ByteArrayMapping
{
    /**
     * Creates a new ShortArrayMapping object.
     *
     * @param dba Datastore adapter
     * @param type The type of the field
     */
    public ShortArrayMapping(DatastoreAdapter dba, Class type)
    {
        super(dba, type);
    }

    /**
     * Creates a new ShortArrayMapping object.
     * @param dba Database Adapter
     * @param fmd FieldMetaData for the field
     * @param table Table for persisting this field
     */
    public ShortArrayMapping(DatastoreAdapter dba, FieldMetaData fmd, DatastoreClass table)
    {
        super(dba, fmd, table);
    }

    /**
     * Overides to convert a byte array back to a short array.
     * @param pm The PersistenceManager
     * @param rs The ResultSet
     * @param param The param number in the ResultSet
     * @return The value retrieved.
     * @throws JDODataStoreException Thrown if an error occurs retrieving the object
     */
    public Object getObject(PersistenceManager pm, Object rs, int[] param)
    {
        Object value;

        try
        {
            byte[] ba = ((ResultSet) rs).getBytes(param[0]);
            value = ((ResultSet) rs).wasNull() ? null : fromByteArray(ba);
        }
        catch (SQLException e)
        {
            throw new JDODataStoreException(LOCALISER.msg("ShortArrayMapping.GetError", param), e);
        }

        return value;
    }

    /**
     * Convert a byte[] into an instance of our value class.
     * @param buf byte array to be converted
     * @return converted short array as object
     */
    protected Object fromByteArray(byte[] buf)
    {
        int n = buf.length / 2;
        short[] a = new short[n];
        int i = 0;
        int j = 0;

        for (; i < n;)
        {
            a[i++] = (short) (((buf[j++] & 0xFF) << 8) + (buf[j++] & 0xFF));
        }

        return a;
    }

    /**
     * Convert an instance of our value class into a byte[].
     * @param value Object to be converted
     * @return converted byte array
     */
    protected byte[] toByteArray(Object value)
    {
        if (value == null)
        {
            return null;
        }

        short[] a = (short[]) value;
        int n = a.length;
        byte[] buf = new byte[n * 2];
        int i = 0;
        int j = 0;

        for (; i < n;)
        {
            short x = a[i++];
            buf[j++] = (byte) ((x >>> 8) & 0xFF);
            buf[j++] = (byte) (x & 0xFF);
        }

        return buf;
    }
    
    /**
     * Accessor for the java type
     * @return The Java type
     */
    public Class getJavaType()
    {
        return short[].class;
    }    
}