/**********************************************************************
Copyright (c) 2002 Mike Martin (TJDO) and others. All rights reserved. 
This program and the accompanying materials are made available under
the terms of the JPOX License v1.0 which accompanies this distribution.

Contributors:
2003 Andy Jefferson - coding standards
    ...
**********************************************************************/
package org.jpox.store.mapping;

import org.jpox.PersistenceManager;
import org.jpox.metadata.FieldMetaData;
import org.jpox.store.DatastoreAdapter;
import org.jpox.store.DatastoreClass;
import org.jpox.store.rdbms.typeinfo.TypeInfo;

/**
 * SCO Mapping. Mapping of a Java type that maps one java field to one datastore field.
 *
 * @version $Revision: 1.2 $
 **/
abstract class SingleFieldMapping extends JavaTypeMapping
{
    protected TypeInfo typeInfo;
    
    protected DatastoreClass datastoreClass;

    /**
     * Constructor.
     * @param dba Datastore Adapter
     * @param type Type of the field
     */
    public SingleFieldMapping(DatastoreAdapter dba, Class type)
    {
        super(null, dba, type);
    }

	/**
	 * Constructor.
	 * @param dba Datastore Adapter
	 * @param fmd FieldMetaData for the field being mapped
	 * @param table Table containing the mapped object
	 */
	public SingleFieldMapping(DatastoreAdapter dba, FieldMetaData fmd, DatastoreClass table)
	{
		super(fmd, dba, fmd.getType());
		datastoreClass = table;
		table.prepareDatastoreMapping(this,fmd);
	}

	/**
	 * Constructor.
	 * @param dba Datastroe Adapter
	 * @param fmd FieldMetaData for the field to be mapped
	 * @param table Table containing the mapped field
	 * @param initDatastoreMappings Whether to initialise the datastore mappings (create the columns etc)
	 */
	public SingleFieldMapping(DatastoreAdapter dba, FieldMetaData fmd, DatastoreClass table, boolean initDatastoreMappings)
	{
		super(fmd, dba, fmd.getType());
        datastoreClass = table;
        if (initDatastoreMappings)
        {
            table.prepareDatastoreMapping(this, fmd);
        }
	}
	
    /**
     * Accessor for the datastore class (e.g in an RDBMS context, the Table)
     * @return The datastore class containing this mapped field.
     */
    public DatastoreClass getDatastoreClass()
    {
        return datastoreClass;
    }
    
    /**
     * Accessor for the string to put in any retrieval datastore statement for this field.
     * In RDBMS, this is typically a ? to be used in JDBC statements.
     * @return The input parameter
     */
    public boolean includeInFetchStatement()
    {
        return true;
    }

    /**
     * This mapping is included in the update statement.
     * @return Whether to include in update statement
     */
    public boolean includeInUpdateStatement()
    {
        return true;
    }

    /**
     * This mapping is included in the insert statement.
     * @return Whether to include in insert statement
     */
    public boolean includeInInsertStatement()
    {
        return true;
    }
    
    /**
     * Equality operator
     * @param obj The object to compare with
     * @return Whether the objects are equal
     */
    public boolean equals(Object obj)
    {
        if (this == obj)
        {
            return true;
        }

        if (!(obj instanceof SingleFieldMapping))
        {
            return false;
        }

        SingleFieldMapping cm = (SingleFieldMapping)obj;

        return getClass().equals(cm.getClass()) &&
            dba.equals(cm.dba) &&
            (typeInfo == null ? cm.typeInfo == null : typeInfo.equals(cm.typeInfo));
    }
    
    public void setBoolean(PersistenceManager pm, Object preparedStatement, int[] exprIndex, boolean value)
    {
        getDataStoreMapping(0).setBoolean(preparedStatement, exprIndex[0], value);
    }

    public boolean getBoolean(PersistenceManager pm, Object resultSet, int[] exprIndex)
    {
        return getDataStoreMapping(0).getBoolean(resultSet, exprIndex[0]);
    }

    public void setChar(PersistenceManager pm, Object preparedStatement, int[] exprIndex, char value)
    {
        getDataStoreMapping(0).setChar(preparedStatement, exprIndex[0], value);
    }

    public char getChar(PersistenceManager pm, Object resultSet, int[] exprIndex)
    {
        return getDataStoreMapping(0).getChar(resultSet, exprIndex[0]);
    }

    public void setByte(PersistenceManager pm, Object preparedStatement, int[] exprIndex, byte value)
    {
        getDataStoreMapping(0).setByte(preparedStatement, exprIndex[0], value);
    }

    public byte getByte(PersistenceManager pm, Object resultSet, int[] exprIndex)
    {
        return getDataStoreMapping(0).getByte(resultSet, exprIndex[0]);
    }

    public void setShort(PersistenceManager pm, Object preparedStatement, int[] exprIndex, short value)
    {
        getDataStoreMapping(0).setShort(preparedStatement, exprIndex[0], value);
    }

    public short getShort(PersistenceManager pm, Object resultSet, int[] exprIndex)
    {
        return getDataStoreMapping(0).getShort(resultSet, exprIndex[0]);
    }

    public void setInt(PersistenceManager pm, Object preparedStatement, int[] exprIndex, int value)
    {
        getDataStoreMapping(0).setInt(preparedStatement, exprIndex[0], value);
    }

    public int getInt(PersistenceManager pm, Object resultSet, int[] exprIndex)
    {
        return getDataStoreMapping(0).getInt(resultSet, exprIndex[0]);
    }

    public void setLong(PersistenceManager pm, Object preparedStatement, int[] exprIndex, long value)
    {
        getDataStoreMapping(0).setLong(preparedStatement, exprIndex[0], value);
    }

    public long getLong(PersistenceManager pm, Object resultSet, int[] exprIndex)
    {
        return getDataStoreMapping(0).getLong(resultSet, exprIndex[0]);
    }

    public void setFloat(PersistenceManager pm, Object preparedStatement, int[] exprIndex, float value)
    {
        getDataStoreMapping(0).setFloat(preparedStatement, exprIndex[0], value);
    }

    public float getFloat(PersistenceManager pm, Object resultSet, int[] exprIndex)
    {
        return getDataStoreMapping(0).getFloat(resultSet, exprIndex[0]);
    }

    public void setDouble(PersistenceManager pm, Object preparedStatement, int[] exprIndex, double value)
    {
        getDataStoreMapping(0).setDouble(preparedStatement, exprIndex[0], value);
    }

    public double getDouble(PersistenceManager pm, Object resultSet, int[] exprIndex)
    {
        return getDataStoreMapping(0).getDouble(resultSet, exprIndex[0]);
    }

    public void setString(PersistenceManager pm, Object preparedStatement, int[] exprIndex, String value)
    {
        getDataStoreMapping(0).setString(preparedStatement, exprIndex[0], value);
    }

    public String getString(PersistenceManager pm, Object resultSet, int[] exprIndex)
    {
        return getDataStoreMapping(0).getString(resultSet, exprIndex[0]);
    }

    public void setObject(PersistenceManager pm, Object preparedStatement, int[] exprIndex, Object value)
    {
        getDataStoreMapping(0).setObject(preparedStatement, exprIndex[0], value);
    }

    public Object getObject(PersistenceManager pm, Object resultSet, int[] exprIndex)
    {
        return getDataStoreMapping(0).getObject(resultSet, exprIndex[0]);
    }    
}