/**********************************************************************
Copyright (c) 2002 TJDO and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
    Andy Jefferson - coding standards and javadocs
    ...
**********************************************************************/
package org.jpox.store.mapping;

import java.sql.Date;

import org.jpox.metadata.FieldMetaData;
import org.jpox.store.DatastoreAdapter;
import org.jpox.store.DatastoreClass;
import org.jpox.store.expression.ScalarExpression;
import org.jpox.store.expression.SqlDateLiteral;
import org.jpox.store.expression.SqlTemporalExpression;
import org.jpox.store.expression.TableExpression;
import org.jpox.store.query.QueryStatement;

/**
 * SCO Mapping for an SQLDate type.
 * Maps between a java.sql.Date object and an SQL "DATE" type in the datatsore. 
 *
 * @version $Revision: 1.2 $
 **/
public class SqlDateMapping extends SingleFieldMapping implements SimpleDatastoreRepresentation
{
    private static java.sql.Date mappingSampleValue = new java.sql.Date(new java.util.Date().getTime());

    public SqlDateMapping(DatastoreAdapter dba, Class type)
    {
        super(dba, type);
    }

    /**
     * Creates a new SqlDateMapping object.
     * @param dba Database Adapter
     * @param fmd FieldMetaData for the field
     * @param table Table for persisting this field
     */
    public SqlDateMapping(DatastoreAdapter dba, FieldMetaData fmd, DatastoreClass table)
    {
        super(dba, fmd, table);
    }

    public Object getSampleValue()
    {
        return mappingSampleValue;
    }

    public ScalarExpression newLiteral(QueryStatement qs, Object value)
    {
        ScalarExpression expr = new SqlDateLiteral(qs, this, (Date)value);
        return expr; 
    }

    public ScalarExpression newScalarExpression(QueryStatement qs, TableExpression te)
    {
        ScalarExpression expr = new SqlTemporalExpression(qs, this, te);
        return expr; 
    }

    public Class getJavaType()
    {
        return java.sql.Date.class;
    }
}