/**********************************************************************
Copyright (c) 2003 Andy Jefferson and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
    ...
**********************************************************************/
package org.jpox.store.mapping;

import java.sql.Time;

import org.jpox.metadata.FieldMetaData;
import org.jpox.store.DatastoreAdapter;
import org.jpox.store.DatastoreClass;
import org.jpox.store.expression.ScalarExpression;
import org.jpox.store.expression.SqlTemporalExpression;
import org.jpox.store.expression.SqlTimeLiteral;
import org.jpox.store.expression.TableExpression;
import org.jpox.store.query.QueryStatement;

/**
 * SCO Mapping for an SQLTime type.
 *
 * @version $Revision: 1.2 $
 **/
public class SqlTimeMapping extends SingleFieldMapping implements SimpleDatastoreRepresentation
{
    private static java.sql.Time mappingSampleValue = new java.sql.Time(new java.util.Date().getTime());

    public SqlTimeMapping(DatastoreAdapter dba, Class type)
    {
        super(dba, type);
    }

    /**
     * Creates a new SqlTimeMapping object.
     * @param dba Database Adapter
     * @param fmd FieldMetaData for the field
     * @param table Table for persisting this field
     */
    public SqlTimeMapping(DatastoreAdapter dba, FieldMetaData fmd, DatastoreClass table)
    {
        super(dba, fmd, table);
    }

    public Object getSampleValue()
    {
        return mappingSampleValue;
    }

    public ScalarExpression newLiteral(QueryStatement qs, Object value)
    {
        ScalarExpression expr = new SqlTimeLiteral(qs, this, (Time)value);
        return expr; 
    }

    public ScalarExpression newScalarExpression(QueryStatement qs, TableExpression te)
    {
        ScalarExpression expr = new SqlTemporalExpression(qs, this, te);
        return expr; 
    }

    public Class getJavaType()
    {
        return java.sql.Time.class;
    }
}
