/**********************************************************************
Copyright (c) 2002 Mike Martin (TJDO) and others. All rights reserved. 
This program and the accompanying materials are made available under 
the terms of the JPOX License v1.0 which accompanies this distribution. 

Contributors:
2003 Andy Jefferson - coding standards and javadocs
    ...
**********************************************************************/
package org.jpox.store.mapping;

import java.sql.Timestamp;

import org.jpox.metadata.FieldMetaData;
import org.jpox.store.DatastoreAdapter;
import org.jpox.store.DatastoreClass;
import org.jpox.store.expression.ScalarExpression;
import org.jpox.store.expression.SqlTemporalExpression;
import org.jpox.store.expression.SqlTimestampLiteral;
import org.jpox.store.expression.TableExpression;
import org.jpox.store.query.QueryStatement;

/**
 * SCO Mapping for an SQLTimestamp type.
 *
 * @version $Revision: 1.3 $
 **/
public class SqlTimestampMapping extends SingleFieldMapping implements SimpleDatastoreRepresentation
{
    private static java.sql.Timestamp mappingSampleValue = new java.sql.Timestamp(new java.util.Date().getTime());

    public SqlTimestampMapping(DatastoreAdapter dba, Class type)
    {
        super(dba, type);
    }

    /**
     * Creates a new SqlTimestampMapping object.
     * @param dba Database Adapter
     * @param fmd FieldMetaData for the field
     * @param table Table for persisting this field
     */
    public SqlTimestampMapping(DatastoreAdapter dba, FieldMetaData fmd, DatastoreClass table)
    {
        super(dba, fmd, table);
    }
    
    public Object getSampleValue()
    {
        return mappingSampleValue;
    }

    public ScalarExpression newLiteral(QueryStatement qs, Object value)
    {
        ScalarExpression expr = new SqlTimestampLiteral(qs, this, (Timestamp)value);
        return expr; 
    }

    public ScalarExpression newScalarExpression(QueryStatement qs, TableExpression te)
    {
        ScalarExpression expr = new SqlTemporalExpression(qs, this, te);
        return expr; 
    }

    public Class getJavaType()
    {
        return java.sql.Timestamp.class;
    }
}