/**********************************************************************
Copyright (c) 2002 Kelly Grizzle (TJDO) and others. All rights reserved. 
This program and the accompanying materials are made available under 
the terms of the JPOX License v1.0 which accompanies this distribution. 

Contributors:
2003 Erik Bengtson - removed subclasses operation
2003 Andy Jefferson - comments, and update to returned class name
2003 Erik Bengtson - added getObjectByAid
2004 Erik Bengtson - throws an JDOObjectNotFoundException
2004 Erik Bengtson - removed unused variable and import
2004 Erik Bengtson - added support for ignoreCache
2004 Andy Jefferson - coding standards
    ...
**********************************************************************/
package org.jpox.store.query;

import java.sql.ResultSet;

import org.jpox.PersistenceManager;
import org.jpox.StateManager;
import org.jpox.metadata.ClassMetaData;
import org.jpox.metadata.IdentityType;
import org.jpox.store.DatastoreClass;
import org.jpox.store.FieldValues;
import org.jpox.store.StatementExpressionIndex;
import org.jpox.store.mapping.JavaTypeMapping;
import org.jpox.store.mapping.OIDMapping;
import org.jpox.store.rdbms.fieldmanager.ResultSetGetter;

/**
 * From a ResultSet (JDBC) creates a PersistenceCapable object instance.
 *
 * @version $Revision: 1.3 $
 */
public final class PersistentIDROF implements Query.ResultObjectFactory
{
    private final DatastoreClass table;

    private final boolean ignoreCache;

    protected int[] fieldNumbers;

    protected StatementExpressionIndex[] statementExpressionIndex;

    protected final int[] datastoreIdentityExpressionIndex;

    public PersistentIDROF(DatastoreClass table, int[] fieldNumbers, StatementExpressionIndex[] statementExpressionIndex,
            int[] datastoreIdentityExpressionIndex, boolean ignoreCache)
    {
        this.statementExpressionIndex = statementExpressionIndex;
        this.fieldNumbers = fieldNumbers;
        this.table = table;
        this.datastoreIdentityExpressionIndex = datastoreIdentityExpressionIndex;
        this.ignoreCache = ignoreCache;
    }

    /**
     * Method to convert the ResultSet row into an Object.
     * @param rs The ResultSet from the Query.
     * @param pcClass The Class that we are looking for.
     * @return The (persisted) object.
     **/
    public Object getObject(final PersistenceManager pm, final ResultSet rs, Class pcClass)
    {
        Class pcClassForObject = pcClass;
        boolean hasInheritanceMetadata = false;
        try
        {
            pcClassForObject = pm.getClassLoaderResolver().classForName(rs.getString("JPOXMETADATA").trim());
            hasInheritanceMetadata = true;
        }
        catch (Exception e)
        {
            //do nothing
        }
        ClassMetaData cmd = pm.getMetaDataManager().getMetaDataForClass(pcClassForObject);
        if (cmd.getIdentityType() == IdentityType.APPLICATION)
        {
            return getObjectByAID(pm, rs, pcClassForObject);
        }
        else
        {
            JavaTypeMapping mapping;
            mapping = new OIDMapping(table.getStoreManager().getDatastoreAdapter(), pcClassForObject);
            mapping.addDataStoreMapping(table.getDataStoreObjectIdMapping().getDataStoreMapping(0));
            Object id = mapping.getObject(pm, rs, datastoreIdentityExpressionIndex);
            if (fieldNumbers == null)
            {
                return pm.getObjectById(id, false, !hasInheritanceMetadata);
            }
            else
            {
                return getObjectById(pm, id, rs, pcClassForObject);
            }
        }
    }

    /**
     * Returns a PC instance from an AID 
     * @param rs JDBC ResultSet
     * @param pcClass PersistenceCapable class
     * @return The object with this AID
     */
    private Object getObjectByAID(final PersistenceManager pm, final ResultSet rs, Class pcClass)
    {
        return pm.getObjectByAID(pcClass, new FieldValues()
        {
            // StateManager calls the fetchFields method
            public void fetchFields(StateManager sm)
            {
                sm.replaceFields(fieldNumbers, new ResultSetGetter(sm, rs, statementExpressionIndex));
            }
        }, ignoreCache, false);
    }

    /**
     * Returns a PC instance from an Datastore ID 
     * @param oid The object id
     * @param rs The ResultSet
     * @param pcClass The PersistenceCapable class
     * @return The Object
     */
    private Object getObjectById(final PersistenceManager pm, Object oid, final ResultSet rs, Class pcClass)
    {
        return pm.getObjectById(oid, new FieldValues()
        {
            // StateManager calls the fetchFields method
            public void fetchFields(StateManager sm)
            {
                sm.replaceFields(fieldNumbers, new ResultSetGetter(sm, rs, statementExpressionIndex));
            }
        }, pcClass, ignoreCache);
    }
}