/**********************************************************************
Copyright (c) 2002 Mike Martin (TJDO) and others. All rights reserved. 
This program and the accompanying materials are made available under 
the terms of the JPOX License v1.0 which accompanies this distribution. 

Contributors:
2003 Andy Jefferson - coding standards
    ...
**********************************************************************/
package org.jpox.store.query;

/**
 * Indicates an object that can be queried, such as an Extent or persistent
 * collection.
 *
 * @see QueryStatement
 * @version $Revision: 1.2 $
 */
public interface Queryable
{
    /**
     * Returns a prototypical query statement over the underlying collection.
     * <p>
     * The returned query statement selects all applicable rows from the
     * relevant base table(s) and the column that represents the element or
     * element ID.
     * The statement can then be modified to join additional tables, select more
     * columns, add WHERE conditions, etc.
     * @return The new prototypical query statement.
     */
    QueryStatement newQueryStatement();

    /**
     * Returns a prototypical query statement over the underlying collection,
     * pre-filtered to include only objects of the specified class.
     *
     * @param candidateClass
     * @return
     *      The new prototypical query statement.
     *
     * @exception javax.jdo.JDOUserException
     *      If <var>candidateClass</var> is not the same as or a subclass of
     *      the defined element type for this collection.
     *
     * @see #newQueryStatement()
     */
    QueryStatement newQueryStatement(Class candidateClass);

    /**
     * Returns a suitable query result factory for results produced by the
     * specified query.
     * <p>
     * The <var>stmt</var> argument must have been obtained by a previous call
     * to newQueryStatement() on the same Queryable.
     * @param stmt The query statement
     * @param ignoreCache Whether to ignore the cache
     * @param resultClass Create objects of a particular type
     * @return A factory for creating PersistenceCapable objects from query results.
     */
    Query.ResultObjectFactory newResultObjectFactory(QueryStatement stmt,boolean ignoreCache,Class resultClass);

    /**
     * Returns <tt>true</tt> if this collection contains no elements.<p>
     * @return <tt>true</tt> if this collection contains no elements.
     */
    boolean isEmpty();
}