/**********************************************************************
Copyright (c) 2002 Kelly Grizzle (TJDO) and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
2002 Mike Martin (TJDO)
2003 Andy Jefferson - commented
    ...
**********************************************************************/
package org.jpox.store.rdbms;

import org.jpox.AbstractPersistenceManagerFactory;
import org.jpox.ClassLoaderResolver;

/**
 * Provides a Factory for RDBMSManager objects. This class provides a caching
 * mechanism for retrieval of RDBMSManagers. This class maintains a cache of 
 * RDBMSManager instances grouped by PMF. 
 *
 * @version $Revision: 1.1 $
 */
public class RDBMSManagerFactory
{
    /**
     * Default Constructor. Private to prevent outside instantiation.
     **/
    private RDBMSManagerFactory()
    {
    }

    /**
     * Accessor for the RDBMSManager.
     * 
     * @param clr the ClassLoaderResolver
     * @param pmf Persistence Manager Factory
     * @param userName The user name for this store
     * @param password The password for this store.
     * @return The RDBMSManager.
     **/
    public static synchronized RDBMSManager getStoreManager(
    					ClassLoaderResolver clr,
                        AbstractPersistenceManagerFactory pmf,
                        String userName,
                        String password)
    {
    	if (userName == null)
		{
			userName = pmf.getPMFConfiguration().getConnectionUserName();
			password = pmf.getPMFConfiguration().getConnectionPassword();
		}
        String key = "" + userName + ';' + password;
        
        RDBMSManager storeMgr = (RDBMSManager)pmf.getPMFContext().getStoreMgrs().get(key);

        if (storeMgr == null)
        {
            storeMgr = new RDBMSManager(clr, pmf, userName, password);

            pmf.getPMFContext().getStoreMgrs().put(key, storeMgr);
        }

        return storeMgr;
    }
}
