/**********************************************************************
Copyright (c) 2004 Andy Jefferson and others. All rights reserved. 
This program and the accompanying materials are made available under 
the terms of the JPOX License v1.0 which accompanies this distribution.

Contributors:
    ...
**********************************************************************/
package org.jpox.store.rdbms;

import org.jpox.metadata.ClassMetaData;
import org.jpox.metadata.FieldMetaData;
import org.jpox.store.StoreData;
import org.jpox.store.rdbms.sqlidentifier.SQLIdentifier;
import org.jpox.store.rdbms.table.JDOTable;
import org.jpox.store.rdbms.table.ViewImpl;
import org.jpox.util.Localiser;

/**
 * Representation of a class (FCO) / field (SCO) that is persisted to an RDBMS table.
 * Extends the basic Store representation to contain the actual table/view object used.
 *
 * @version $Revision: 1.3 $ 
 */
public class RDBMSStoreData extends StoreData
{
    /** Localiser for messages */
    protected static final Localiser LOCALISER_RDBMS=Localiser.getInstance("org.jpox.store.rdbms.Localisation");

    /** The table/view where this class/field is persisted. */
    protected JDOTable table;

    /**
     * Constructor, taking the meta data for the class, and the table it is mapped to.
     * @param cmd MetaData for the class.
     * @param table Table definition
     * @param tableOwner Whether it is the owner of the table
     **/
    public RDBMSStoreData(ClassMetaData cmd, JDOTable table, boolean tableOwner)
    {
        super(cmd, table != null ? table.toString() : null, tableOwner);
        this.table = table;
    }

    /**
     * Constructor, taking the meta data for the field, and the table it is mapped to.
     * @param fmd MetaData for the field.
     * @param table Table definition
     **/
    public RDBMSStoreData(FieldMetaData fmd, JDOTable table)
    {
        super(fmd, table != null ? table.toString() : null);
        this.table = table;
    }
 
    /**
     * Utility to return whether this table is a view.
     * @return Whether it is for a view.
     **/
    public boolean mapsToView()
    {
        if (table == null)
        {
            return false;
        }
        return (table instanceof ViewImpl);
    }

    /**
     * Accessor for the Table details.
     * @return The Table
     **/
    public JDOTable getTable()
    {
        return table;
    }

    /**
     * Accessor for the identifier for the table.
     * @return The table identifier
     **/
    public SQLIdentifier getTableIdentifier()
    {
        if (table != null)
        {
            return (SQLIdentifier) table.getName();
        }

        // If the class doesn't map to a table return null
        return null;
    }

    /**
     * Convenience to set the table. To be used in cases where the table isn't known
     * until after the initial create
     * @param table The table
     */
    public void setTable(JDOTable table)
    {
        if (table != null)
        {
            tableName = table.toString();
            this.table = table;
        }
    }
}