/**********************************************************************
Copyright (c) 2003 Erik Bengtson and others. All rights reserved.
This program and the accompanying materials are made available under
the terms of the JPOX License v1.0 which accompanies this distribution.

Contributors:
    ...
**********************************************************************/
package org.jpox.store.rdbms.adapter;

import java.sql.DatabaseMetaData;
import java.sql.ResultSet;

import org.jpox.store.rdbms.typeinfo.InformixTypeInfo;
import org.jpox.store.rdbms.typeinfo.TypeInfo;

/**
 * Provides methods for adapting SQL language elements to the Informix
 * database. Overrides some methods in DatabaseAdapter where Informix behaviour
 * differs.
 * 
 * Informix databases must be created WITH LOG MODE ANSI, otherwise
 * errors like "Transaction Not Supported", "Not in transaction" will appear.
 * See the informix info
 *
 * @version $Revision: 1.2 $ 
 */
class InformixAdapter extends DatabaseAdapter
{

    /**
     * Constructor.
     * @param metadata MetaData for the DB
     **/
    public InformixAdapter(DatabaseMetaData metadata)
    {
        super(metadata);

    }

    public String getVendorID()
    {
        return "informix";
    }

    /**
     * Factory for TypeInfo objects.
     * @param rs The ResultSet from DatabaseMetaData.getTypeInfo(). 
     * @return A TypeInfo object.
     **/
    public TypeInfo newTypeInfo(ResultSet rs)
    {
        return new InformixTypeInfo(rs);
    }
    
    /**
     * Accessor for an identifier quote string.
     * @return Identifier quote string.
     **/
    public String getIdentifierQuoteString()
    {
    	/*
    	 * the documentation says a double quote, but it seems to not work
    	 */
        return "";
    }   
    
	/**
	 * Accessor for the auto-increment sql statement for this datastore.
	 * @return The statement for getting the latest auto-increment key
	 **/
	public String getAutoIncrementStmt()
	{
	    return "SELECT first 1 dbinfo('sqlca.sqlerrd1') from systables";
	}    
}
