/**********************************************************************
Copyright (c) 2004 Andy Jefferson and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
    ...
**********************************************************************/
package org.jpox.store.rdbms.adapter;

import java.sql.DatabaseMetaData;
import java.sql.Types;

import org.jpox.store.DatastoreContainerObject;
import org.jpox.store.DatastoreIdentifier;
import org.jpox.store.expression.TableExprAsJoins;
import org.jpox.store.expression.TableExpression;
import org.jpox.store.query.QueryStatement;
import org.jpox.store.rdbms.typeinfo.TypeInfo;

/**
 * Provides methods for adapting SQL language elements to the Pointbase
 * database.
 *
 * @version $Revision: 1.4 $ 
 */
class PointbaseAdapter extends DatabaseAdapter
{
    public PointbaseAdapter(DatabaseMetaData metadata)
    {
        super(metadata);
    }

    /**
     * Accessor for the vendor id.
     * @return The vendor id.
     **/
    public String getVendorID()
    {
        return "pointbase";
    }

    public TableExpression newTableExpression(QueryStatement qs, DatastoreContainerObject table, DatastoreIdentifier rangeVar)
    {
        return new TableExprAsJoins(qs,table,rangeVar);
    }

    public boolean supportsBooleanComparison()
    {
        return false;
    }

    public int getUnlimitedLengthPreciisionValue(TypeInfo typeinfo)
    {
        if (typeinfo.dataType == Types.BLOB ||
            typeinfo.dataType == Types.CLOB)
        {
            return 1 << 31;
        }
        else
        {
            return super.getUnlimitedLengthPrecisionValue(typeinfo);
        }
    }
}
