/**********************************************************************
Copyright (c) 2002 Mike Martin (TJDO) and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
2003 Andy Jefferson - coding standards
    ...
**********************************************************************/
package org.jpox.store.rdbms.columninfo;

import java.sql.ResultSet;
import java.sql.Types;

import org.jpox.store.rdbms.typeinfo.PostgreSQLTypeInfo;

/**
 * Represents the metadata of a specific table column in PostgreSQL.
 *
 * @version $Revision: 1.1 $
 */
public class PostgreSQLColumnInfo extends ColumnInfo
{
    /**
     * Constructs a column information object from the current row of the given
     * result set.  The {@link ResultSet} object passed must have been obtained
     * from a call to DatabaseMetaData.getColumns().
     *
     * <p>This method only retrieves the values from the current row; the caller
     * is required to advance to the next row with {@link ResultSet#next}.
     *
     * @param rs    The result set returned from DatabaseMetaData.getColumns().
     */
    public PostgreSQLColumnInfo(ResultSet rs)
    {
        super(rs);

        // Equate "text" to Types.LONGVARCHAR
        if (typeName.equalsIgnoreCase("text"))
        {
            dataType = Types.LONGVARCHAR;
        }
        // Equate "bytea" to Types.LONGVARBINARY
        else if (typeName.equalsIgnoreCase("bytea"))
        {
            dataType = Types.LONGVARBINARY;
        }

        /*
         * The PostgreSQL drivers sometimes produce some truly funky metadata.
         * I saw these next two occur during unit testing when
         * decimal_widget.big_decimal_field reported a columnSize of 65535 and
         * a decimalDigits of 65534, instead of the correct answers of 18 and 2.
         *
         * A -1 for either of these will cause their validation to be bypassed,
         * which is a shame but it's all we can do.  No one should be surprised
         * if we end up needing more of these.
         */
        if (columnSize > PostgreSQLTypeInfo.MAX_PRECISION)
        {
            columnSize = -1;
        }

        if (decimalDigits > PostgreSQLTypeInfo.MAX_PRECISION)
        {
            decimalDigits = -1;
        }
    }
}
