/**********************************************************************
Copyright (c) 2002 TJDO and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
    Andy Jefferson - coding standards
    ...
**********************************************************************/
package org.jpox.store.rdbms.fieldmanager;

import java.sql.ResultSet;

import org.jpox.PersistenceManager;
import org.jpox.StateManager;
import org.jpox.store.StatementExpressionIndex;
import org.jpox.store.fieldmanager.AbstractFieldManager;

/**
 * ResultSet getter implementation of a field manager.
 *
 * @version $Revision: 1.1 $
 **/
public class ResultSetGetter extends AbstractFieldManager
{
    private final StateManager sm;
    private final PersistenceManager pm;
    private final ResultSet rs;
    private final StatementExpressionIndex[] statementExpressionIndex;

    public ResultSetGetter(StateManager sm, ResultSet rs, StatementExpressionIndex[] statementExpressionIndex)
    {
        this.sm = sm;
        this.pm = sm.getPersistenceManager();
        this.rs = rs;
        this.statementExpressionIndex = statementExpressionIndex;
    }

    public boolean fetchBooleanField(int fieldNumber)
    {
        return statementExpressionIndex[fieldNumber].getMapping().getBoolean(pm, rs, statementExpressionIndex[fieldNumber].getExpressionIndex());
    }

    public char fetchCharField(int fieldNumber)
    {
        return statementExpressionIndex[fieldNumber].getMapping().getChar(pm, rs, statementExpressionIndex[fieldNumber].getExpressionIndex());
    }

    public byte fetchByteField(int fieldNumber)
    {
        return statementExpressionIndex[fieldNumber].getMapping().getByte(pm, rs, statementExpressionIndex[fieldNumber].getExpressionIndex());
    }

    public short fetchShortField(int fieldNumber)
    {
        return statementExpressionIndex[fieldNumber].getMapping().getShort(pm, rs, statementExpressionIndex[fieldNumber].getExpressionIndex());
    }

    public int fetchIntField(int fieldNumber)
    {
        return statementExpressionIndex[fieldNumber].getMapping().getInt(pm, rs, statementExpressionIndex[fieldNumber].getExpressionIndex());
    }

    public long fetchLongField(int fieldNumber)
    {
        return statementExpressionIndex[fieldNumber].getMapping().getLong(pm, rs, statementExpressionIndex[fieldNumber].getExpressionIndex());
    }

    public float fetchFloatField(int fieldNumber)
    {
        return statementExpressionIndex[fieldNumber].getMapping().getFloat(pm, rs, statementExpressionIndex[fieldNumber].getExpressionIndex());
    }

    public double fetchDoubleField(int fieldNumber)
    {
        return statementExpressionIndex[fieldNumber].getMapping().getDouble(pm, rs, statementExpressionIndex[fieldNumber].getExpressionIndex());
    }

    public String fetchStringField(int fieldNumber)
    {
        return statementExpressionIndex[fieldNumber].getMapping().getString(pm, rs, statementExpressionIndex[fieldNumber].getExpressionIndex());
    }

    public Object fetchObjectField(int fieldNumber)
    {
        Object value = statementExpressionIndex[fieldNumber].getMapping().getObject(pm, rs, statementExpressionIndex[fieldNumber].getExpressionIndex());

        return sm.newSCOInstance(fieldNumber, value);
    }
}
