/**********************************************************************
Copyright (c) 2003 Erik Bengtson and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
    Andy Jefferson - updated link to StoreManagerHelper
    Erik Bengtson - removed unused import
    ...
**********************************************************************/
package org.jpox.store.rdbms.foreignkey;

import java.sql.ResultSet;

import javax.jdo.JDOFatalDataStoreException;

/**
 * Represents the metadata of a specific exported key column.  This class is
 * basically a data structure that makes accessing the JDBC exported key
 * metadata easier.  Each of the items returned by
 * {@link java.sql.DatabaseMetaData#getExportedKeys(String,String,String)}
 * is represented by a public field in this class.
 *
 * Subclasses of ExportedKeyInfo can be created on a per-DBMS basis to supply
 * missing metadata or correct faulty metadata obtained from that DBMS's JDBC
 * driver(s).
 *
 * @see org.jpox.store.rdbms.adapter.DatabaseAdapter#newExportedKeyInfo
 * @version $Revision: 1.2 $ 
 */
public class ExportedKeyInfo extends ForeignKeyInfo
{

    /**
     * Constructs a exported key information object from the current row of the
     * given result set.
     * The {@link ResultSet} object passed must have been obtained from a call
     * to java.sql.DatabaseMetaData.getExportedKeys().
     *
     * <p>This method only retrieves the values from the current row; the caller
     * is required to advance to the next row with {@link ResultSet#next}.
     *
     * @param rs    The result set returned from
     *              java.sql.DatabaseMetaData.getImportedKeys() or
     *              java.sql.DatabaseMetaData.getExportedKeys().
     *
     * @exception JDOFatalDataStoreException
     *      if a column of foreign key information could not be retrieved from
     *      the result set.
     */
    public ExportedKeyInfo(ResultSet rs) throws JDOFatalDataStoreException
    {
    	super(rs);
    }

    /**
     * Constructs a foreign key information object from its individual
     * attributes.
     *
     * <p>This can be useful to subclasses and/or custom DatabaseAdapters that
     * need to modify and/or correct the metadata returned by the JDBC driver.
     */
    public ExportedKeyInfo(String pkTableCat,
                          String pkTableSchem,
                          String pkTableName,
                          String pkColumnName,
                          String fkTableCat,
                          String fkTableSchem,
                          String fkTableName,
                          String fkColumnName,
                          short keySeq,
                          short updateRule,
                          short deleteRule,
                          String fkName,
                          String pkName,
                          short deferrability)
    {
			super(pkTableCat,
                   pkTableSchem,
                   pkTableName,
                   pkColumnName,
                   fkTableCat,
                   fkTableSchem,
                   fkTableName,
                   fkColumnName,
                   keySeq,
                   updateRule,
                   deleteRule,
                   fkName,
                   pkName,
                   deferrability);
    }
}