/**********************************************************************
Copyright (c) 2002 Mike Martin and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
    Andy Jefferson - coding standards
    ...
**********************************************************************/
package org.jpox.store.rdbms.foreignkey;

import java.sql.ResultSet;


/**
 * Represents the metadata of a table's foreign key column in PostgreSQL.
 *
 * @version $Revision: 1.1 $ 
 */
public class PostgreSQLForeignKeyInfo extends ForeignKeyInfo
{
    /**
     * Constructs a foreign key information object from the current row of the
     * given result set.
     * The {@link ResultSet} object passed must have been obtained from a call
     * to DatabaseMetaData.getImportedKeys()
     * or DatabaseMetaData.getExportedKeys().
     *
     * <p>This method only retrieves the values from the current row; the
     * caller is required to advance to the next row with
     * {@link ResultSet#next}.
     *
     * @param rs The result set returned from
     * DatabaseMetaData.getImportedKeys()/DatabaseMetaData.getExportedKeys()
     */
    public PostgreSQLForeignKeyInfo(ResultSet rs)
    {
        super(rs);

        /*
         * The PostgreSQL drivers sometimes produce some truly funky metadata.
         * In this case, the FK_NAME column has been known to have miscellaneous
         * other strings dangling off the end of it separated by "\000", as in
         *
         *    WIDGET_FK1\000WIDGET\000BLAH ...
         *
         * instead of:
         *
         *    WIDGET_FK1
         */
        int firstBackslashIdx = fkName.indexOf('\\');

        if (firstBackslashIdx > 0)
        {
            fkName = fkName.substring(0, firstBackslashIdx);
        }
    }
}
