/**********************************************************************
Copyright (c) 2003 Mike Martin (TJDO) and others. All rights reserved. 
This program and the accompanying materials are made available under 
the terms of the JPOX License v1.0 which accompanies this distribution. 

Contributors:
2004 Andy Jefferson - equality operator
2004 Erik Bengtson - addInheritedColumn() for app-id
    ...
**********************************************************************/
package org.jpox.store.rdbms.key;

import org.jpox.store.DatastoreField;
import org.jpox.store.rdbms.table.TableImpl;

/**
 * Representation of a Candidate key.
 * This represents a UNIQUE key on a column or columns.
 *
 * @version $Revision: 1.3 $
 **/
public class CandidateKey extends Key
{
    /**
     * Constructor.
     * @param table Table to apply this key to
     */
    public CandidateKey(TableImpl table)
    {
        super(table);
    }

    /**
     * Mutator for the column spec, to add/change a column.
     * @param seq Sequence number of the column
     * @param col The column
     */
    public void setDatastoreField(int seq, DatastoreField col)
    {
        assertSameDatastoreObject(col);

        setMinSize(columns, seq + 1);

        if (columns.get(seq) != null)
        {
            //throw new JDOFatalInternalException("Key part #" + seq + " for " + table + " already set");
        }

        columns.set(seq, col);
    }

    /**
     * Add column from super class. For classes with Application Identity
     * that inherits the primary key for the super class 
     * @param col The column
     */
    public void addInheritedDatastoreField(DatastoreField col)
    {
        columns.add(col);
    }

    /**
     * Accessor for the size.
     * @return The size.
     **/
    public int size()
    {
        return columns.size();
    }

    /**
     * Equality operator.
     * @param obj The object to compare against
     * @return Whether the objects are equal
     */
    public boolean equals(Object obj)
    {
        if (obj == this)
        {
            return true;
        }

        if (!(obj instanceof CandidateKey))
        {
            return false;
        }

        return super.equals(obj);
    }

    /**
     * Stringify method.
     * @return String version of this object.
     **/
    public String toString()
    {
        StringBuffer s = new StringBuffer("UNIQUE ").append(getColumnList(columns));

        return s.toString();
    }
}