/**********************************************************************
Copyright (c) 2003 Andy Jefferson and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
	TJDO - original version
	Andy Jefferson - equality operator
	Andy Jefferson - setName()
    ...
**********************************************************************/
package org.jpox.store.rdbms.key;

import javax.jdo.JDOFatalInternalException;

import org.jpox.store.rdbms.Column;
import org.jpox.store.rdbms.table.TableImpl;

/**
 * Representation of an index.
 *
 * @version $Revision: 1.4 $
 **/
public class Index extends Key
{
    private final boolean isUnique;

    /**
     * Constructor.
     * @param table The table
     * @param isUnique Whether the index is unique
     */
    public Index(TableImpl table, boolean isUnique)
    {
        super(table);

        this.isUnique = isUnique;
    }

    /**
     * Constructor.
     * @param ck Candidate key to use as a basis
     */
    public Index(CandidateKey ck)
    {
        super(ck.getDatastoreContainerObject());

        isUnique = true;
        columns.addAll(ck.getColumns());
    }

    /**
     * Constructor.
     * @param fk Foreign key to use as a basis
     */
    public Index(ForeignKey fk)
    {
        super(fk.getDatastoreContainerObject());

        isUnique = false;
        columns.addAll(fk.getColumns());
    }

    /**
     * Accessor for whether the index is unique
     * @return Whether it is unique.
     */
    public boolean getUnique()
    {
        return isUnique;
    }


    public void setColumn(int seq, Column col)
    {
        assertSameDatastoreObject(col);

        setMinSize(columns, seq + 1);

        if (columns.get(seq) != null)
        {
            throw new JDOFatalInternalException("Index part #" + seq + " for " + table + " already set");
        }

        columns.set(seq, col);
    }

    /**
     * Accessor for the size.
     * @return The size.
     **/
    public int size()
    {
        return columns.size();
    }

    /**
     * Hashcode operator.
     * @return The hashcode
     **/
    public int hashCode()
    {
        return (isUnique ? 0 : 1) ^ super.hashCode();
    }

    /**
     * Equality operator.
     * @param obj Object to compare against
     * @return Whether they are equal.
     **/
    public boolean equals(Object obj)
    {
        if (obj == this)
        {
            return true;
        }
        if (!(obj instanceof Index))
        {
            return false;
        }

        Index idx = (Index)obj;
        if (idx.isUnique != isUnique)
        {
            return false;
        }

        return super.equals(obj);
    }

    /**
     * Stringify method.
     * @return String version of this object.
     **/
    public String toString()
    {
        return getColumnList();
    }
}