/**********************************************************************
Copyright (c) 2004 Erik Bengtson and others. All rights reserved. 
This program and the accompanying materials are made available under 
the terms of the JPOX License v1.0 which accompanies this distribution. 

Contributors:
2004 Andy Jefferson - coding standards and localised messages
	...
**********************************************************************/
package org.jpox.store.rdbms.mapping;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.StreamCorruptedException;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;

import javax.jdo.JDODataStoreException;
import javax.jdo.JDOFatalUserException;

import org.jpox.store.DatastoreField;
import org.jpox.store.StoreManager;
import org.jpox.store.mapping.JavaTypeMapping;
import org.jpox.store.rdbms.Column;
import org.jpox.store.rdbms.typeinfo.TypeInfo;
import org.jpox.util.JPOXLogger;
import org.jpox.util.TypeConversionHelper;

/**
 * Maps a embbeded field  
 * @version $Revision: 1.4 $
 **/
public class Blob2RDBMSMapping extends ColumnMapping
{
    /**
     * @param storeMgr Store Manager
     * @param mapping The Java mapping
     */
    protected Blob2RDBMSMapping(StoreManager storeMgr, JavaTypeMapping mapping)
    {
        super(storeMgr, mapping);
    }

    public Blob2RDBMSMapping(JavaTypeMapping mapping, StoreManager storeMgr, DatastoreField field)
    {
		super(storeMgr, mapping);
		column = (Column) field;
		initialize();
	}

    private void initialize()
    {
		initTypeInfo();
    }
    
    public TypeInfo getTypeInfo()
    {
        return getDatabaseAdapter().getTypeInfo(Types.BLOB);
    }

	public void setObject(Object ps, int param, Object value)
	{
		if (value == null)
		{
			try
			{
				((PreparedStatement) ps).setNull(param,Types.BLOB);
			}
			catch (SQLException sqle)
			{
                String msg=LOCALISER.msg("RDBMSMapping.UnableToSetParam","Object", "" + param);
                JPOXLogger.RDBMS.error(msg);
				throw new JDODataStoreException(msg, sqle);
			}
		}
		else
		{
			try
			{
			    
                if( value instanceof boolean[] )
                {
	                byte[] data = TypeConversionHelper.getByteArrayFromBooleanArray(value);
	                ((PreparedStatement) ps).setBytes(param, data);
                }
                else if( value instanceof char[] )
                {
	                byte[] data = TypeConversionHelper.getByteArrayFromCharArray(value);
	                ((PreparedStatement) ps).setBytes(param, data);
                }                
                else if( value instanceof double[] )
                {
	                byte[] data = TypeConversionHelper.getByteArrayFromDoubleArray(value);
	                ((PreparedStatement) ps).setBytes(param, data);
                }                
                else if( value instanceof float[] )
                {
	                byte[] data = TypeConversionHelper.getByteArrayFromFloatArray(value);
	                ((PreparedStatement) ps).setBytes(param, data);
                }                
                else if( value instanceof int[] )
                {
	                byte[] data = TypeConversionHelper.getByteArrayFromIntArray(value);
	                ((PreparedStatement) ps).setBytes(param, data);
                }                
                else if( value instanceof long[] )
                {
	                byte[] data = TypeConversionHelper.getByteArrayFromLongArray(value);
	                ((PreparedStatement) ps).setBytes(param, data);
                }                
                else if( value instanceof short[] )
                {
	                byte[] data = TypeConversionHelper.getByteArrayFromShortArray(value);
	                ((PreparedStatement) ps).setBytes(param, data);
                }                
                else if( value instanceof byte[] )
                {
	                ((PreparedStatement) ps).setBytes(param, (byte[]) value);
                }
                else
                {			    
					ByteArrayOutputStream baos = new ByteArrayOutputStream();
					ObjectOutputStream oos = new ObjectOutputStream(baos);
					oos.writeObject(value);
					byte[] bytes = baos.toByteArray();
					((PreparedStatement) ps).setBinaryStream(param, new ByteArrayInputStream(bytes),bytes.length);
                }
			}
			catch (SQLException sqle)
			{
                String msg=LOCALISER.msg("RDBMSMapping.UnableToSetParam", "Object", "" + param);
                JPOXLogger.RDBMS.error(msg);
				throw new JDODataStoreException(msg, sqle);
			}
			catch (IOException ioe)
			{
                String msg=LOCALISER.msg("RDBMSMapping.UnableToSetParam", "Object", "" + param);
                JPOXLogger.RDBMS.error(msg);
				throw new JDODataStoreException(msg, ioe);
			}
		}
	}

	public Object getObject(Object rs, int param)
	{
		Object obj = null;
		try
		{
		    
            byte[] bytes = ((ResultSet) rs).getBytes(param);
			if (!((ResultSet) rs).wasNull())
			{
				try
				{
			        if( getJavaTypeMapping().getJavaType().getName().equals(boolean[].class.getName()))
			        {
			            return TypeConversionHelper.getBooleanArrayfromByteArray(bytes);
			        }
			        else if( getJavaTypeMapping().getJavaType().getName().equals(char[].class.getName()))
			        {
			            return TypeConversionHelper.getCharArrayFromByteArray(bytes);
			        }
			        else if( getJavaTypeMapping().getJavaType().getName().equals(double[].class.getName()))
			        {
			            return TypeConversionHelper.getDoubleArrayFromByteArray(bytes);
			        }
			        else if( getJavaTypeMapping().getJavaType().getName().equals(float[].class.getName()))
			        {
			            return TypeConversionHelper.getFloatArrayFromByteArray(bytes);
			        }
			        else if( getJavaTypeMapping().getJavaType().getName().equals(int[].class.getName()))
			        {
			            return TypeConversionHelper.getIntArrayFromByteArray(bytes);
			        }
			        else if( getJavaTypeMapping().getJavaType().getName().equals(long[].class.getName()))
			        {
			            return TypeConversionHelper.getLongArrayFromByteArray(bytes);
			        }
			        else if( getJavaTypeMapping().getJavaType().getName().equals(short[].class.getName()))
			        {
			            return TypeConversionHelper.getShortArrayFromByteArray(bytes);
			        }
			        else if( getJavaTypeMapping().getJavaType().getName().equals(byte[].class.getName()))
			        {
			            return bytes;
			        }        
			        else
			        {				    
			            obj = new ObjectInputStream(new ByteArrayInputStream(bytes)).readObject();
			        }
				}
				catch (StreamCorruptedException e)
				{
                    String msg="StreamCorruptedException: object is corrupted";
                    JPOXLogger.RDBMS.error(msg);
					throw new JDOFatalUserException(msg, e);
				}
				catch (IOException e)
				{
                    String msg="IOException: error when reading object";
					throw new JDOFatalUserException(msg, e);
				}
				catch (ClassNotFoundException e)
				{
                    String msg="ClassNotFoundException: error when creating object";
                    JPOXLogger.RDBMS.error(msg);
					throw new JDOFatalUserException(msg, e);
				}
			}
		}
		catch (SQLException sqle)
		{
			throw new JDODataStoreException(LOCALISER.msg("RDBMSMapping.UnableToGetParam", "Object", "" + param),sqle);
		}
		return obj;
	}
}