/**********************************************************************
Copyright (c) 2004 Erik Bengtson and others. All rights reserved. 
This program and the accompanying materials are made available under 
the terms of the JPOX License v1.0 which accompanies this distribution. 

Contributors:
2004 Andy Jefferson - localised messages
    ...
**********************************************************************/
package org.jpox.store.rdbms.mapping;

import java.io.IOException;
import java.io.Serializable;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;

import javax.jdo.JDODataStoreException;

import org.jpox.store.DatastoreField;
import org.jpox.store.StoreManager;
import org.jpox.store.mapping.JavaTypeMapping;
import org.jpox.store.rdbms.Column;
import org.jpox.store.rdbms.datatype.BlobImpl;
import org.jpox.store.rdbms.typeinfo.TypeInfo;

/**
 * Mapping of a character RDBMS type.
 * 
 * @version $Revision: 1.3 $
 */
public class BlobRDBMSMapping extends ColumnMapping
{
    /**
     * @param storeMgr Store Manager
     * @param mapping Java type mapping
     */
    protected BlobRDBMSMapping(StoreManager storeMgr, JavaTypeMapping mapping)
    {
        super(storeMgr, mapping);
    }
    
    /**
     * Constructor.
     * @param mapping Java type mapping
     * @param storeMgr Store Manager
     * @param field Field to be mapped
     */
    public BlobRDBMSMapping(JavaTypeMapping mapping, StoreManager storeMgr, DatastoreField field)
    {
		super(storeMgr, mapping);
		column = (Column) field;
		initialize();
	}

    private void initialize()
    {
		initTypeInfo();
    }
    
    public TypeInfo getTypeInfo()
    {
        return getDatabaseAdapter().getTypeInfo(Types.BLOB);
    }
    
    public void setString(Object ps, int param, String value)
    {
        try
        {
            if (value == null)
            {
                if (column.isDefaultable() && column.getDefaultValue() != null)
                {
                    ((PreparedStatement) ps).setString(param,column.getDefaultValue().toString().trim());
                }
                else
                {
                    ((PreparedStatement) ps).setNull(param,getTypeInfo().dataType);
                }
            }
            else
            {
                ((PreparedStatement) ps).setString(param, value);
            }
        }
        catch (SQLException e)
        {
            throw new JDODataStoreException(LOCALISER.msg("RDBMSMapping.UnableToSetParam", "String", "" + value), e);
        }
    }

    public String getString(Object rs, int param)
    {
        String value;

        try
        {
            value = ((ResultSet) rs).getString(param);
        }
        catch (SQLException e)
        {
            throw new JDODataStoreException(LOCALISER.msg("RDBMSMapping.UnableToGetParam", "String", "" + param), e);
        }

        return value;
    }

    public void setObject(Object ps, int param, Object value)
    {
        if (value == null)
        {
            try
            {
                ((PreparedStatement) ps).setBytes(param,null);
            }
            catch (SQLException sqle)
            {
				throw new JDODataStoreException(LOCALISER.msg("RDBMSMapping.UnableToSetParam","Object", "" + param),sqle);
            }
        }
        else
        {
            try
            {
                if (! (value instanceof Serializable))
                {
                    throw new JDODataStoreException(LOCALISER.msg("ObjectMapping.NotSerializable",value.getClass().getName()));
                }
                BlobImpl b = new BlobImpl(value);
                ((PreparedStatement) ps).setBytes(param,b.getBytes(0,(int)b.length()));
            }
            catch (SQLException sqle)
            {
				throw new JDODataStoreException(LOCALISER.msg("RDBMSMapping.UnableToSetParam","Object", "" + param),sqle);
            }
            catch (IOException ioe)
            {
				throw new JDODataStoreException(LOCALISER.msg("RDBMSMapping.UnableToSetParam","Object", "" + param),ioe);
            }
        }
    }

    public Object getObject(Object rs, int param)
    {
        byte[] bytes = null;
        try
        {
            bytes = ((ResultSet) rs).getBytes(param);
        }
        catch (SQLException sqle)
        {
			throw new JDODataStoreException(LOCALISER.msg("RDBMSMapping.UnableToGetParam","Object", "" + param),sqle);
        }
        if (bytes == null)
        {
            return null;
        }
 
        BlobImpl blob;
        try
        {
            blob = new BlobImpl(bytes);
            return blob.getObject();
        }
        catch (IOException ioe)
        {
			throw new JDODataStoreException(LOCALISER.msg("RDBMSMapping.UnableToGetParam", "Object", "" + param),ioe);
        }
    }   
}