/**********************************************************************
Copyright (c) 2004 Erik Bengtson and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
	Andy Jefferson - localised messages
    ...
**********************************************************************/
package org.jpox.store.rdbms.mapping;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;

import javax.jdo.JDODataStoreException;
import javax.jdo.JDOUserException;

import org.jpox.store.DatastoreField;
import org.jpox.store.StoreManager;
import org.jpox.store.mapping.JavaTypeMapping;
import org.jpox.store.rdbms.Column;
import org.jpox.store.rdbms.typeinfo.TypeInfo;

/**
 * Mapping of a BOOLEAN RDBMS type.
 * 
 * @version $Revision: 1.3 $
 */
public class BooleanRDBMSMapping extends ColumnMapping
{
    /**
     * @param storeMgr Store Manager
     * @param mapping The java type mapping
     */
    protected BooleanRDBMSMapping(StoreManager storeMgr, JavaTypeMapping mapping)
    {
        super(storeMgr, mapping);
    }
    
    public BooleanRDBMSMapping(JavaTypeMapping mapping, StoreManager storeMgr, DatastoreField field)
    {
		super(storeMgr, mapping);
		column = (Column) field;
		initialize();
	}
    
    private void initialize()
    {
        initTypeInfo();
    }
    
    public TypeInfo getTypeInfo()
    {
        return getDatabaseAdapter().getTypeInfo(Types.BOOLEAN);
    }
    
    public DatastoreField getDatastoreField()
    {
        return column;
    }
    
    public void setString(Object ps, int param, String value)
    {
        try
        {
            if (value == null)
            {
                ((PreparedStatement) ps).setNull(param, getTypeInfo().dataType);
            }
            else
            {
                ((PreparedStatement) ps).setBoolean(param, value.equals("Y") ? true : false );
            }
        }
        catch (SQLException e)
        {
            throw new JDODataStoreException(LOCALISER.msg("RDBMSMapping.UnableToSetParam", "String", "" + value), e);
        }
    }

    public void setBoolean(Object ps, int param, boolean value)
    {
        try
        {
            ((PreparedStatement) ps).setBoolean(param, value);
        }
        catch (SQLException e)
        {
            throw new JDODataStoreException(LOCALISER.msg("RDBMSMapping.UnableToSetParam", "boolean", "" + value), e);
        }
    }

    public boolean getBoolean(Object rs, int param)
    {
        boolean value;

        try
        {
            value = ((ResultSet) rs).getBoolean(param);
        }
        catch (SQLException e)
        {
            throw new JDODataStoreException(LOCALISER.msg("RDBMSMapping.UnableToGetParam","Boolean", "" + param), e);
        }

        return value;
    }
    
    public String getString(Object rs, int param)
    {
        String value;

        try
        {
            boolean b = ((ResultSet) rs).getBoolean(param);
            if( b )
            {
                value = "Y";
            }
            else
            {
                value = "N";
            }
        }
        catch (SQLException e)
        {
            throw new JDODataStoreException(LOCALISER.msg("RDBMSMapping.UnableToGetParam","String", "" + param), e);
        }

        return value;
    }

    public void setObject(Object ps, int param, Object value)
    {
        try
        {
            if (value == null)
            {
                ((PreparedStatement) ps).setNull(param, getTypeInfo().dataType);
            }
            else
            {
                if (value instanceof String )
                {
                    ((PreparedStatement) ps).setBoolean(param, value.equals("Y") ? true : false);
                }
                else if (value instanceof Boolean )
                {
                    ((PreparedStatement) ps).setBoolean(param, ((Boolean)value).booleanValue());
                }
                else
                {
                    throw new JDOUserException(LOCALISER.msg("RDBMSMapping.IllegalValueInColumn", value, column));
                }                
            }
        }
        catch (SQLException e)
        {
            throw new JDODataStoreException(LOCALISER.msg("RDBMSMapping.UnableToSetParam","Object", "" + value), e);
        }
    }

    public Object getObject(Object rs, int param)
    {
        Object value;

        try
        {
            boolean b = ((ResultSet) rs).getBoolean(param);

            if (((ResultSet) rs).wasNull())
            {
                value = null;
            }
            else
            {
                if (getJavaTypeMapping().getJavaType().getName().equals(String.class.getName()) )
                {
                    if (b)
                    {
                        value = "Y";
                    }
                    else
                    {
                        value = "N";
                    }
                }
                else //Boolean.class
                {
                    if (b)
                    {
                        value = Boolean.TRUE;
                    }
                    else
                    {
                        value = Boolean.FALSE;
                    }
                }   
            }
        }
        catch (SQLException e)
        {
            throw new JDODataStoreException(LOCALISER.msg("RDBMSMapping.UnableToGetParam", "Object", "" + param), e);
        }

        return value;
    }    
}