/**********************************************************************
Copyright (c) 2004 Erik Bengtson and others. All rights reserved. 
This program and the accompanying materials are made available under 
the terms of the JPOX License v1.0 which accompanies this distribution. 

Contributors:
2005 Andy Jefferson - updated to derive from default
2005 Erik Bengtson - added int alias
2005 Andy Jefferson - added BLOB support for primitive arrays
    ...
**********************************************************************/
package org.jpox.store.rdbms.mapping;

import java.io.Serializable;

/**
 * Mapping manager for Cloudscape. Overrides some of the defaults due to lack of support,
 * and adds on some of its own for more flexibility.
 * @version $Revision: 1.9 $
 */
public class CloudscapeRDBMSMappingManager extends RDBMSMappingManager
{
    public void initialise()
    {
        super.initialise();

        // Cloudscape doesn't support BIT
        deregisterDatastoreMappingsForJavaType(boolean.class.getName());
        registerDatastoreMapping(boolean.class.getName(),CharRDBMSMapping.class, "CHAR", "CHAR", true);
        registerDatastoreMapping(boolean.class.getName(),BooleanRDBMSMapping.class, "BOOLEAN", "BOOLEAN", false);

        deregisterDatastoreMappingsForJavaType(Boolean.class.getName());
        registerDatastoreMapping(Boolean.class.getName(),CharRDBMSMapping.class, "CHAR", "CHAR", true);
        registerDatastoreMapping(Boolean.class.getName(),BooleanRDBMSMapping.class, "BOOLEAN", "BOOLEAN", false);

        // SQL INT types in Cloudscape are alias to the SQL INTEGER type
        registerDatastoreMapping(char.class.getName(),IntegerRDBMSMapping.class, "INTEGER", "INT", false);
        registerDatastoreMapping(Character.class.getName(),IntegerRDBMSMapping.class, "INTEGER", "INT", false);
        registerDatastoreMapping(int.class.getName(),IntegerRDBMSMapping.class, "INTEGER", "INT", false);
        registerDatastoreMapping(Integer.class.getName(),IntegerRDBMSMapping.class, "INTEGER", "INT", false);
        registerDatastoreMapping(Short.class.getName(),IntegerRDBMSMapping.class, "INTEGER", "INT", false);

        // Alternatives for storing Object, and Serializable
        registerDatastoreMapping(Object.class.getName(), Blob2RDBMSMapping.class, "BLOB", "BLOB", false);
        registerDatastoreMapping(Serializable.class.getName(), Blob2RDBMSMapping.class, "BLOB", "BLOB", false);

        // Alternatives for storing longer primitive arrays
        registerDatastoreMapping(byte[].class.getName(), Blob2RDBMSMapping.class, "BLOB", "BLOB", false);
        registerDatastoreMapping(int[].class.getName(), Blob2RDBMSMapping.class, "BLOB", "BLOB", false);
        registerDatastoreMapping(long[].class.getName(), Blob2RDBMSMapping.class, "BLOB", "BLOB", false);
        registerDatastoreMapping(float[].class.getName(), Blob2RDBMSMapping.class, "BLOB", "BLOB", false);
        registerDatastoreMapping(double[].class.getName(), Blob2RDBMSMapping.class, "BLOB", "BLOB", false);
        registerDatastoreMapping(char[].class.getName(), Blob2RDBMSMapping.class, "BLOB", "BLOB", false);
        registerDatastoreMapping(short[].class.getName(), Blob2RDBMSMapping.class, "BLOB", "BLOB", false);
        registerDatastoreMapping(boolean[].class.getName(), Blob2RDBMSMapping.class, "BLOB", "BLOB", false);
    }
}