/**********************************************************************
Copyright (c) 2004 Erik Bengtson and others. All rights reserved. 
This program and the accompanying materials are made available under 
the terms of the JPOX License v1.0 which accompanies this distribution. 

Contributors:
2004 Andy Jefferson - localised messages
    ...
**********************************************************************/
package org.jpox.store.rdbms.mapping;

import java.io.IOException;
import java.io.InputStreamReader;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;

import javax.jdo.JDODataStoreException;

import oracle.sql.CLOB;

import org.jpox.metadata.ColumnDefinitionException;
import org.jpox.store.DatastoreField;
import org.jpox.store.StoreManager;
import org.jpox.store.mapping.JavaTypeMapping;
import org.jpox.store.rdbms.Column;
import org.jpox.store.rdbms.typeinfo.TypeInfo;

/**
 * Mapping for Oracle CLOB type.
 *
 * @version $Revision: 1.3 $
 **/
public class OracleClobRDBMSMapping extends ColumnMapping
{
    /**
     * Constructor
     * @param storeMgr Store Manager
     * @param mapping The Java type mapping
     */
    protected OracleClobRDBMSMapping(StoreManager storeMgr, JavaTypeMapping mapping)
    {
        super(storeMgr, mapping);
    }
    
    public OracleClobRDBMSMapping(JavaTypeMapping mapping, StoreManager storeMgr, DatastoreField field)
    {
		super(storeMgr, mapping);
		column = (Column) field;
		initialize();
	}
    
    private void initialize()
    {
        if (column != null)
        {
    		if (column.getLengthType() != Column.UNLIMITED_LENGTH)
    		{
    			throw new ColumnDefinitionException("Invalid length specified for CLOB column " + column + ", must be 'unlimited'");
    		}
        }
		initTypeInfo();
    }    

    public TypeInfo getTypeInfo()
    {
        return getDatabaseAdapter().getTypeInfo(Types.CLOB);
    }

    public String getInsertionInputParameter()
    {
        return "EMPTY_CLOB()";
    }
    
    public boolean includeInSQLFetchStatement()
    {
        return true;
    }

    public String getUpdateInputParameter()
    {
        return "EMPTY_CLOB()";
    }
      
    public String getString(Object rs, int param)
    {
        String value = null;

        try
        {
            char[] cbuf = null;
            CLOB clob = (CLOB)((ResultSet) rs).getClob(param);

            if (clob != null)
            {
                // Note: Using clob.stringValue() results in StoreManagerTest
                // exception: "java.sql.SQLException: Conversion to String failed"

                StringBuffer sbuf = new StringBuffer();
                InputStreamReader reader = new InputStreamReader(clob.asciiStreamValue());

                try
                {
                    final int BUFF_SIZE = 4096;
                    cbuf = new char[BUFF_SIZE];
                    int charsRead = reader.read(cbuf);

                    while (-1 != charsRead)
                    {
                        sbuf.append(cbuf, 0, charsRead);

                        java.util.Arrays.fill(cbuf, (char)0);
                        charsRead = reader.read(cbuf);
                    }

                }
                catch (IOException e)
                {
                    throw new JDODataStoreException("Error reading Oracle CLOB object: param = " + param, e);
                }

                value = sbuf.toString();

                if (value.length() == 0)
                {
                    value = null;
                }
                else if (value.equals(getDatabaseAdapter().getSurrogateForEmptyStrings()))
                {
                    value = "";
                }
            }
        }
        catch (SQLException e)
        {
            throw new JDODataStoreException(LOCALISER.msg("RDBMSMapping.UnableToGetParam","String", "" + param), e);
        }

        return value;
    }

    public Object getObject(Object rs, int param)
    {
        return getString(rs, param);
    }
}