/******************************************************************
Copyright (c) 2004 Erik Bengtson and others. All rights reserved. 
This program and the accompanying materials are made available under
the terms of the JPOX License v1.0 which accompanies this distribution.

Contributors:
2004 Andy Jefferson - moved getMapping() from OracleDatabaseAdapter
    ...
*****************************************************************/
package org.jpox.store.rdbms.mapping;

import java.io.Serializable;
import java.math.BigDecimal;
import java.math.BigInteger;

import org.jpox.metadata.FieldMetaData;
import org.jpox.store.DatastoreAdapter;
import org.jpox.store.DatastoreClass;
import org.jpox.store.OID;
import org.jpox.store.mapping.JavaTypeMapping;
import org.jpox.store.mapping.OracleStringMapping;
import org.jpox.util.JavaUtils;

/**
 * Mapping manager for Oracle RDBMS adapters.
 * 
 * @version $Revision: 1.5 $
 */
public class OracleRDBMSMappingManager extends AbstractRDBMSMappingManager
{
    public void initialise()
    {
        super.initialise();
        
        registerDatastoreMapping(boolean.class.getName(),NumericRDBMSMapping.class,"NUMERIC","NUMERIC");
        registerDatastoreMapping(Boolean.class.getName(),NumericRDBMSMapping.class,"NUMERIC","NUMERIC");

        registerDatastoreMapping(byte.class.getName(),NumericRDBMSMapping.class,"NUMERIC","NUMERIC");
        registerDatastoreMapping(Byte.class.getName(),NumericRDBMSMapping.class,"NUMERIC","NUMERIC");

        registerDatastoreMapping(char.class.getName(),NumericRDBMSMapping.class,"NUMERIC","NUMERIC");
        registerDatastoreMapping(Character.class.getName(),NumericRDBMSMapping.class,"NUMERIC","NUMERIC");

        registerDatastoreMapping(int.class.getName(),NumericRDBMSMapping.class,"NUMERIC","NUMERIC");
        registerDatastoreMapping(Integer.class.getName(),NumericRDBMSMapping.class,"NUMERIC","NUMERIC");

        registerDatastoreMapping(long.class.getName(),NumericRDBMSMapping.class,"NUMERIC","NUMERIC");
        registerDatastoreMapping(Long.class.getName(),NumericRDBMSMapping.class,"NUMERIC","NUMERIC");

        registerDatastoreMapping(float.class.getName(),FloatRDBMSMapping.class,"FLOAT","FLOAT",true);
        registerDatastoreMapping(float.class.getName(),DecimalRDBMSMapping.class,"DECIMAL","DECIMAL",false);

        registerDatastoreMapping(Float.class.getName(),FloatRDBMSMapping.class,"FLOAT","FLOAT",true);
        registerDatastoreMapping(Float.class.getName(),DecimalRDBMSMapping.class,"DECIMAL","DECIMAL",false);

        registerDatastoreMapping(double.class.getName(),FloatRDBMSMapping.class,"FLOAT","FLOAT",true);
        registerDatastoreMapping(double.class.getName(),DecimalRDBMSMapping.class,"DECIMAL","DECIMAL",false);

        registerDatastoreMapping(Double.class.getName(),FloatRDBMSMapping.class,"FLOAT","FLOAT",true);
        registerDatastoreMapping(Double.class.getName(),DecimalRDBMSMapping.class,"DECIMAL","DECIMAL",false);

        registerDatastoreMapping(short.class.getName(),NumericRDBMSMapping.class,"NUMERIC","NUMERIC");
        registerDatastoreMapping(Short.class.getName(),NumericRDBMSMapping.class,"NUMERIC","NUMERIC");

        registerDatastoreMapping(String.class.getName(),VarCharRDBMSMapping.class,"VARCHAR","VARCHAR", true);
        registerDatastoreMapping(String.class.getName(),CharRDBMSMapping.class,"CHAR","CHAR", false);
        registerDatastoreMapping(String.class.getName(),LongVarcharRDBMSMapping.class,"BLOB","BLOB", false);
        registerDatastoreMapping(String.class.getName(),OracleClobRDBMSMapping.class,"CLOB","CLOB", false);

        registerDatastoreMapping(java.util.Locale.class.getName(),CharRDBMSMapping.class,"CHAR","CHAR", true);
        registerDatastoreMapping(java.util.Locale.class.getName(),VarCharRDBMSMapping.class,"VARCHAR","VARCHAR", false);
        registerDatastoreMapping(java.util.TimeZone.class.getName(),CharRDBMSMapping.class,"CHAR","CHAR", true);
        registerDatastoreMapping(java.util.TimeZone.class.getName(),VarCharRDBMSMapping.class,"VARCHAR","VARCHAR", false);

        if (JavaUtils.isJRE1_4OrAbove())
        {
            registerDatastoreMapping(java.util.Currency.class.getName(),CharRDBMSMapping.class,"CHAR","CHAR", true);
            registerDatastoreMapping(java.util.Currency.class.getName(),VarCharRDBMSMapping.class,"VARCHAR","VARCHAR", false);
        }

        registerDatastoreMapping(java.util.Date.class.getName(),TimestampRDBMSMapping.class,"TIMESTAMP","TIMESTAMP");
        registerDatastoreMapping(java.util.Date.class.getName(),DateRDBMSMapping.class,"DATE","DATE");
        registerDatastoreMapping(java.sql.Date.class.getName(),TimestampRDBMSMapping.class,"TIMESTAMP","TIMESTAMP");
        registerDatastoreMapping(java.sql.Date.class.getName(),DateRDBMSMapping.class,"DATE","DATE");
        registerDatastoreMapping(java.sql.Time.class.getName(),TimestampRDBMSMapping.class,"TIMESTAMP","TIMESTAMP");
        registerDatastoreMapping(java.sql.Time.class.getName(),TimeRDBMSMapping.class,"TIME","TIME");
        registerDatastoreMapping(java.sql.Timestamp.class.getName(),TimestampRDBMSMapping.class,"TIMESTAMP","TIMESTAMP");

        registerDatastoreMapping(Object.class.getName(),Blob2RDBMSMapping.class,"BLOB","BLOB");
        registerDatastoreMapping(Serializable.class.getName(),Blob2RDBMSMapping.class,"BLOB","BLOB");
        registerDatastoreMapping(OID.class.getName(),NumericRDBMSMapping.class,"NUMERIC","NUMERIC");
        registerDatastoreMapping(BigDecimal.class.getName(),NumericRDBMSMapping.class,"DECIMAL","DECIMAL");
        registerDatastoreMapping(BigInteger.class.getName(),NumericRDBMSMapping.class,"NUMERIC","NUMERIC");

        registerDatastoreMapping(byte[].class.getName(),Blob2RDBMSMapping.class,"BLOB","BLOB");
        registerDatastoreMapping(int[].class.getName(),Blob2RDBMSMapping.class,"BLOB","BLOB");
        registerDatastoreMapping(long[].class.getName(),Blob2RDBMSMapping.class,"BLOB","BLOB");
        registerDatastoreMapping(float[].class.getName(),Blob2RDBMSMapping.class,"BLOB","BLOB");
        registerDatastoreMapping(double[].class.getName(),Blob2RDBMSMapping.class,"BLOB","BLOB");
        registerDatastoreMapping(char[].class.getName(),Blob2RDBMSMapping.class,"BLOB","BLOB");
        registerDatastoreMapping(short[].class.getName(),Blob2RDBMSMapping.class,"BLOB","BLOB");
        registerDatastoreMapping(boolean[].class.getName(),Blob2RDBMSMapping.class,"BLOB","BLOB");
    }
    
    /**
     * Accessor for the mapping for the field of the specified table.
     * @param table Table to query
     * @param fmd FieldMetaData for the field to map
     * @param dba Datastore Adapter
     * @return The mapping for the field.
     */
    public JavaTypeMapping getMapping(DatastoreClass table, FieldMetaData fmd, DatastoreAdapter dba)
    {
        // Override CLOB Java type to use OracleString.
        if (fmd.getType().equals(String.class))
        {
            if (fmd.getColumnMetaData() != null && fmd.getColumnMetaData().length > 0)
            {
                String jdbcType = fmd.getColumnMetaData()[0].getJdbcType();

                if (jdbcType != null && jdbcType.toLowerCase().indexOf("clob") > 0)
                {
                    return new OracleStringMapping(dba, fmd, table);
                }
            }
        }
		return super.getMapping(table, fmd, dba);
    }
}