/**********************************************************************
Copyright (c) 2003 Erik Bengtson and others. All rights reserved. 
This program and the accompanying materials are made available under 
the terms of the JPOX License v1.0 which accompanies this distribution. 

Contributors:
2003 Andy Jefferson - converted to use Reflection
2004 Andy Jefferson - changed to give targetException on Invocation error
    ...
**********************************************************************/
package org.jpox.store.rdbms.mapping;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

import javax.jdo.JDOFatalInternalException;

import org.jpox.store.DatastoreField;
import org.jpox.store.StoreManager;
import org.jpox.store.mapping.DatastoreMapping;
import org.jpox.store.mapping.JavaTypeMapping;
import org.jpox.store.rdbms.adapter.DatabaseAdapter;
import org.jpox.util.Localiser;

/**
 * Factory class for creating Mapping instances
 *
 * @version $Revision: 1.3 $ 
 */
public final class RDBMSMappingFactory
{
    private static final Localiser LOCALISER=Localiser.getInstance("org.jpox.store.mapping.Localisation");

    /** Private constructor to prevent instantiation. */
    private RDBMSMappingFactory()
    {
    }

    /**
     * Get a new instance of the Mapping using the Store Manager, type and field.
     * @param mappingClass the Mapping class to be created
     * @param mapping The java mapping type
     * @param storeMgr The Store Manager
     * @param column The column to map
     * @return The Mapping
     */
    public static DatastoreMapping createMapping(Class mappingClass, JavaTypeMapping mapping, StoreManager storeMgr, DatastoreField column)
    {
        Object obj = null;
        Object[] args = new Object[]{mapping, storeMgr, column};
        Class[] ctr_args_classes = new Class[]{JavaTypeMapping.class, StoreManager.class, DatastoreField.class};
        try
        {
            Constructor ctr = mappingClass.getConstructor(ctr_args_classes);
            try
            {
                obj = ctr.newInstance(args);
            }
            catch (InvocationTargetException e)
            {
                throw new JDOFatalInternalException(LOCALISER.msg("Mapping.ConstructionFailure", mappingClass.getName()), e.getTargetException());
            }
            catch (Exception e)
            {
                throw new JDOFatalInternalException(LOCALISER.msg("Mapping.ConstructionFailure", mappingClass.getName()), e);
            }
        }
        catch (NoSuchMethodException nsme)
        {
            throw new JDOFatalInternalException(LOCALISER.msg("Mapping.NoSuchConstructor", DatabaseAdapter.class, Class.class, mappingClass.getName()));
        }
        return (DatastoreMapping) obj;
    }
}