/******************************************************************
Copyright (c) 2004 Erik Bengtson and others. All rights reserved. 
This program and the accompanying materials are made available under 
the terms of the JPOX License v1.0 which accompanies this distribution.

Contributors:
    ...
*****************************************************************/
package org.jpox.store.rdbms.mapping;

import java.io.Serializable;
import java.math.BigDecimal;
import java.math.BigInteger;

import org.jpox.store.OID;
import org.jpox.util.JavaUtils;

/**
 * Mapping manager for RDBMS datastores.
 * Provides mappings from standard Java types defined in org.jpox.store.mapping to
 * datastore mappings for JDBC types.
 * 
 * @version $Revision: 1.7 $
 */
public class RDBMSMappingManager extends AbstractRDBMSMappingManager
{
    public void initialise()
    {
        super.initialise();

        registerDatastoreMapping(boolean.class.getName(),BitRDBMSMapping.class,"BIT","BIT", true);
        registerDatastoreMapping(boolean.class.getName(),CharRDBMSMapping.class,"CHAR","CHAR", false);
        registerDatastoreMapping(boolean.class.getName(),BooleanRDBMSMapping.class,"BOOLEAN","BOOLEAN", false);

        registerDatastoreMapping(Boolean.class.getName(),BitRDBMSMapping.class,"BIT","BIT", true);
        registerDatastoreMapping(Boolean.class.getName(),CharRDBMSMapping.class,"CHAR","CHAR", false);
        registerDatastoreMapping(Boolean.class.getName(),BooleanRDBMSMapping.class,"BOOLEAN","BOOLEAN", false);

        registerDatastoreMapping(byte.class.getName(),TinyIntRDBMSMapping.class,"TINYINT","TINYINT", true);

        registerDatastoreMapping(Byte.class.getName(),TinyIntRDBMSMapping.class,"TINYINT","TINYINT", true);

        registerDatastoreMapping(char.class.getName(),IntegerRDBMSMapping.class,"INTEGER","INTEGER", true);
        registerDatastoreMapping(char.class.getName(),CharRDBMSMapping.class,"CHAR","CHAR", false);

        registerDatastoreMapping(Character.class.getName(),IntegerRDBMSMapping.class,"INTEGER","INTEGER", true);
        registerDatastoreMapping(Character.class.getName(),CharRDBMSMapping.class,"CHAR","CHAR", false);

        registerDatastoreMapping(int.class.getName(),IntegerRDBMSMapping.class,"INTEGER","INTEGER", true);
        registerDatastoreMapping(int.class.getName(),BigIntRDBMSMapping.class,"BIGINT","BIGINT", false);
        registerDatastoreMapping(int.class.getName(),NumericRDBMSMapping.class,"NUMERIC","NUMERIC", false);

        registerDatastoreMapping(Integer.class.getName(),IntegerRDBMSMapping.class,"INTEGER","INTEGER", true);
        registerDatastoreMapping(Integer.class.getName(),BigIntRDBMSMapping.class,"BIGINT","BIGINT", false);
        registerDatastoreMapping(Integer.class.getName(),NumericRDBMSMapping.class,"NUMERIC","NUMERIC", false);

        registerDatastoreMapping(long.class.getName(),BigIntRDBMSMapping.class,"BIGINT","BIGINT", true);
        registerDatastoreMapping(long.class.getName(),NumericRDBMSMapping.class,"NUMERIC","NUMERIC", false);

        registerDatastoreMapping(Long.class.getName(),BigIntRDBMSMapping.class,"BIGINT","BIGINT", true);
        registerDatastoreMapping(Long.class.getName(),NumericRDBMSMapping.class,"NUMERIC","NUMERIC", false);

        registerDatastoreMapping(float.class.getName(),FloatRDBMSMapping.class,"FLOAT","FLOAT", true);
        registerDatastoreMapping(float.class.getName(),DoubleRDBMSMapping.class,"DOUBLE","DOUBLE", false);
        registerDatastoreMapping(float.class.getName(),RealRDBMSMapping.class,"REAL","REAL", false);

        registerDatastoreMapping(Float.class.getName(),FloatRDBMSMapping.class,"FLOAT","FLOAT", true);
        registerDatastoreMapping(Float.class.getName(),DoubleRDBMSMapping.class,"DOUBLE","DOUBLE", false);
        registerDatastoreMapping(Float.class.getName(),RealRDBMSMapping.class,"REAL","REAL", false);

        registerDatastoreMapping(double.class.getName(),DoubleRDBMSMapping.class,"DOUBLE","DOUBLE", true);
        registerDatastoreMapping(double.class.getName(),DecimalRDBMSMapping.class,"DECIMAL","DECIMAL", false);

        registerDatastoreMapping(Double.class.getName(),DoubleRDBMSMapping.class,"DOUBLE","DOUBLE", true);
        registerDatastoreMapping(Double.class.getName(),DecimalRDBMSMapping.class,"DECIMAL","DECIMAL", false);

        registerDatastoreMapping(short.class.getName(),SmallIntRDBMSMapping.class,"SMALLINT","SMALLINT", true);
        registerDatastoreMapping(short.class.getName(),IntegerRDBMSMapping.class,"INTEGER","INTEGER", false);

        registerDatastoreMapping(Short.class.getName(),SmallIntRDBMSMapping.class,"SMALLINT","SMALLINT", true);
        registerDatastoreMapping(Short.class.getName(),IntegerRDBMSMapping.class,"INTEGER","INTEGER", false);

        registerDatastoreMapping(String.class.getName(),VarCharRDBMSMapping.class,"VARCHAR","VARCHAR", true);
        registerDatastoreMapping(String.class.getName(),CharRDBMSMapping.class,"CHAR","CHAR", false);
        registerDatastoreMapping(String.class.getName(),LongVarcharRDBMSMapping.class,"BLOB","BLOB", false);
        registerDatastoreMapping(String.class.getName(),LongVarcharRDBMSMapping.class,"LONGVARCHAR","LONGVARCHAR", false);

        registerDatastoreMapping(java.util.Date.class.getName(),TimestampRDBMSMapping.class,"TIMESTAMP","TIMESTAMP", true);
        registerDatastoreMapping(java.util.Date.class.getName(),DateRDBMSMapping.class,"DATE","DATE", false);

        registerDatastoreMapping(java.util.Locale.class.getName(),CharRDBMSMapping.class,"CHAR","CHAR", true);
        registerDatastoreMapping(java.util.Locale.class.getName(),VarCharRDBMSMapping.class,"VARCHAR","VARCHAR", false);

        if (JavaUtils.isJRE1_4OrAbove())
        {
            registerDatastoreMapping(java.util.Currency.class.getName(),CharRDBMSMapping.class,"CHAR","CHAR", true);
            registerDatastoreMapping(java.util.Currency.class.getName(),VarCharRDBMSMapping.class,"VARCHAR","VARCHAR", false);
        }

        registerDatastoreMapping(java.util.TimeZone.class.getName(),CharRDBMSMapping.class,"CHAR","CHAR", true);
        registerDatastoreMapping(java.util.TimeZone.class.getName(),VarCharRDBMSMapping.class,"VARCHAR","VARCHAR", false);

        registerDatastoreMapping(java.sql.Date.class.getName(),TimestampRDBMSMapping.class,"TIMESTAMP","TIMESTAMP", true);
        registerDatastoreMapping(java.sql.Date.class.getName(),DateRDBMSMapping.class,"DATE","DATE", false);

        registerDatastoreMapping(java.sql.Time.class.getName(),TimestampRDBMSMapping.class,"TIMESTAMP","TIMESTAMP", true);
        registerDatastoreMapping(java.sql.Time.class.getName(),TimeRDBMSMapping.class,"TIME","TIME", false);

        registerDatastoreMapping(java.sql.Timestamp.class.getName(),TimestampRDBMSMapping.class,"TIMESTAMP","TIMESTAMP", true);

        registerDatastoreMapping(Object.class.getName(),LongVarBinaryRDBMSMapping.class,"LONGVARBINARY","LONGVARBINARY", true);
        registerDatastoreMapping(Serializable.class.getName(),LongVarBinaryRDBMSMapping.class,"LONGVARBINARY","LONGVARBINARY", true);
        
        registerDatastoreMapping(OID.class.getName(),BigIntRDBMSMapping.class,"BIGINT","BIGINT", true);
        registerDatastoreMapping(OID.class.getName(),CharRDBMSMapping.class,"CHAR","CHAR", false);
        
        registerDatastoreMapping(BigDecimal.class.getName(),DecimalRDBMSMapping.class,"DECIMAL","DECIMAL", true);
        registerDatastoreMapping(BigDecimal.class.getName(),NumericRDBMSMapping.class,"NUMERIC","NUMERIC", false);
        
        registerDatastoreMapping(BigInteger.class.getName(),NumericRDBMSMapping.class,"NUMERIC","NUMERIC", true);

        registerDatastoreMapping(byte[].class.getName(),LongVarBinaryRDBMSMapping.class,"LONGVARBINARY","LONGVARBINARY", true);
        registerDatastoreMapping(int[].class.getName(),LongVarBinaryRDBMSMapping.class,"LONGVARBINARY","LONGVARBINARY", true);
        registerDatastoreMapping(long[].class.getName(),LongVarBinaryRDBMSMapping.class,"LONGVARBINARY","LONGVARBINARY", true);
        registerDatastoreMapping(float[].class.getName(),LongVarBinaryRDBMSMapping.class,"LONGVARBINARY","LONGVARBINARY", true);
        registerDatastoreMapping(double[].class.getName(),LongVarBinaryRDBMSMapping.class,"LONGVARBINARY","LONGVARBINARY", true);
        registerDatastoreMapping(char[].class.getName(),LongVarBinaryRDBMSMapping.class,"LONGVARBINARY","LONGVARBINARY", true);
        registerDatastoreMapping(short[].class.getName(),LongVarBinaryRDBMSMapping.class,"LONGVARBINARY","LONGVARBINARY", true);
        registerDatastoreMapping(boolean[].class.getName(),LongVarBinaryRDBMSMapping.class,"LONGVARBINARY","LONGVARBINARY", true);
    }
}