/**********************************************************************
Copyright (c) 2004 Erik Bengtson and others. All rights reserved. 
This program and the accompanying materials are made available under 
the terms of the JPOX License v1.0 which accompanies this distribution. 

Contributors:
2004 Andy Jefferson - localised messages
    ...
**********************************************************************/
package org.jpox.store.rdbms.mapping;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;

import javax.jdo.JDODataStoreException;

import org.jpox.store.DatastoreField;
import org.jpox.store.StoreManager;
import org.jpox.store.exceptions.NullValueException;
import org.jpox.store.mapping.JavaTypeMapping;
import org.jpox.store.rdbms.Column;
import org.jpox.store.rdbms.typeinfo.TypeInfo;

/**
 * Mapping of a REAL RDBMS type.
 * 
 * @version $Revision: 1.3 $
 */
public class RealRDBMSMapping extends ColumnMapping
{
    /**
     * @param storeMgr Store Manager
     * @param mapping java type mapping
     */
    protected RealRDBMSMapping(StoreManager storeMgr, JavaTypeMapping mapping)
    {
        super(storeMgr, mapping);
    }

    /**
     * Constructor.
     * @param mapping Java type mapping
     * @param storeMgr Store Manager
     * @param field Field to be mapped
     */
    public RealRDBMSMapping(JavaTypeMapping mapping, StoreManager storeMgr, DatastoreField field)
    {
		super(storeMgr, mapping);
		column = (Column) field;
		initialize();
	}
    
    private void initialize()
    {
        if (column != null)
        {
            column.checkPrimitive();
        }
		initTypeInfo();
    }
    
    public TypeInfo getTypeInfo()
    {
        return getDatabaseAdapter().getTypeInfo(Types.REAL);
    }
    
    public DatastoreField getDatastoreField()
    {
        return column;
    }
    
    public void setFloat(Object ps, int param, float value)
    {
        try
        {
            ((PreparedStatement) ps).setFloat(param, value);
        }
        catch (SQLException e)
        {
            throw new JDODataStoreException(LOCALISER.msg("RDBMSMapping.UnableToSetParam","float","" + value), e);
        }
    }

    public float getFloat(Object rs, int param)
    {
        float value;

        try
        {
            value = ((ResultSet) rs).getFloat(param);
            if (((ResultSet) rs).wasNull())
            {
                throw new NullValueException(LOCALISER.msg("RDBMSMapping.IllegalNullValueInColumn",column));
            }
        }
        catch (SQLException e)
        {           
            try
            {
                //when value is real in database, cause cause a parse error when calling getFloat
                //JDBC error:Value can not be converted to requested type.

                value = Float.parseFloat(((ResultSet) rs).getString(param));
                if (((ResultSet) rs).wasNull())
                {
                    throw new NullValueException(LOCALISER.msg("RDBMSMapping.IllegalNullValueInColumn",column));
                }
            }
            catch (SQLException e1)
            {
                try
                {
                    throw new JDODataStoreException("Can't get float result: param = " + param + " - " +((ResultSet) rs).getString(param), e);
                }
                catch (SQLException e2)
                {
                    throw new JDODataStoreException(LOCALISER.msg("RDBMSMapping.UnableToGetParam","float","" + param), e);
                }
            }
        }

        return value;
    }

    public void setObject(Object ps, int param, Object value)
    {
        try
        {
            if (value == null)
            {
                ((PreparedStatement) ps).setNull(param, getTypeInfo().dataType);
            }
            else
            {
                ((PreparedStatement) ps).setFloat(param, ((Float)value).floatValue());
            }
        }
        catch (SQLException e)
        {
            throw new JDODataStoreException(LOCALISER.msg("RDBMSMapping.UnableToSetParam","Object","" + value), e);
        }
    }

    public Object getObject(Object rs, int param)
    {
        Object value;

        try
        {
            float f = ((ResultSet) rs).getFloat(param);
            value = ((ResultSet) rs).wasNull() ? null : new Float(f);
        }
		catch (SQLException e)
		{           
			try
			{
				//when value is real in database, cause cause a parse error when calling getFloat
				//JDBC error:Value can not be converted to requested type.

				value = new Float(Float.parseFloat(((ResultSet) rs).getString(param)));
				if (((ResultSet) rs).wasNull())
				{
					throw new NullValueException(LOCALISER.msg("RDBMSMapping.IllegalNullValueInColumn",column));
				}
			}
			catch (SQLException e1)
			{
				try
				{
					throw new JDODataStoreException("Can't get float result: param = " + param + " - " +((ResultSet) rs).getString(param), e);
				}
				catch (SQLException e2)
				{
					throw new JDODataStoreException(LOCALISER.msg("RDBMSMapping.UnableToGetParam","Object","" + param), e);
				}
			}
		}        

        return value;
    }
}