/**********************************************************************
Copyright (c) 2004 Erik Bengtson and others. All rights reserved. 
This program and the accompanying materials are made available under 
the terms of the JPOX License v1.0 which accompanies this distribution. 

Contributors:
2004 Andy Jefferson - localised messages
    ...
**********************************************************************/
package org.jpox.store.rdbms.mapping;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;

import javax.jdo.JDODataStoreException;

import org.jpox.store.DatastoreField;
import org.jpox.store.StoreManager;
import org.jpox.store.exceptions.NullValueException;
import org.jpox.store.mapping.JavaTypeMapping;
import org.jpox.store.rdbms.Column;
import org.jpox.store.rdbms.typeinfo.TypeInfo;

/**
 * Mapping of a SMALLINT RDBMS type.
 * 
 * @version $Revision: 1.3 $
 */
public class SmallIntRDBMSMapping extends ColumnMapping
{
    /**
     * @param storeMgr Store Manager
     * @param mapping Java type mapping
     */
    protected SmallIntRDBMSMapping(StoreManager storeMgr, JavaTypeMapping mapping)
    {
        super(storeMgr, mapping);
    }
    
    /**
     * Constructor.
     * @param mapping Java type mapping
     * @param storeMgr Store Manager
     * @param field Field to be mapped
     */
    public SmallIntRDBMSMapping(JavaTypeMapping mapping, StoreManager storeMgr, DatastoreField field)
    {
		super(storeMgr, mapping);
		column = (Column) field;
		initialize();
	}
    
    private void initialize()
    {
        if (column != null)
        {
            column.checkPrimitive();
        }
		initTypeInfo();
    }
    
    public TypeInfo getTypeInfo()
    {
        return getDatabaseAdapter().getTypeInfo(Types.SMALLINT);
    }
    
    public DatastoreField getDatastoreField()
    {
        return column;
    }
    
    public void setShort(Object ps, int param, short value)
    {
        try
        {
            ((PreparedStatement) ps).setShort(param, value);
        }
        catch (SQLException e)
        {
            throw new JDODataStoreException(LOCALISER.msg("RDBMSMapping.UnableToSetParam","short","" + value), e);
        }
    }

    public short getShort(Object rs, int param)
    {
        short value;

        try
        {
            value = ((ResultSet) rs).getShort(param);

            if (((ResultSet) rs).wasNull())
            {
                throw new NullValueException(LOCALISER.msg("RDBMSMapping.IllegalNullValueInColumn",column));
            }
        }
        catch (SQLException e)
        {
            throw new JDODataStoreException(LOCALISER.msg("RDBMSMapping.UnableToGetParam","short","" + param), e);
        }

        return value;
    }
    
    public void setInt(Object ps, int param, int value)
    {
        try
        {
            ((PreparedStatement) ps).setInt(param, value);
        }
        catch (SQLException e)
        {
            throw new JDODataStoreException(LOCALISER.msg("RDBMSMapping.UnableToSetParam","int","" + value), e);
        }
    }

    public int getInt(Object rs, int param)
    {
        int value;

        try
        {
            value = ((ResultSet) rs).getInt(param);

            if (((ResultSet) rs).wasNull())
            {
                throw new NullValueException(LOCALISER.msg("RDBMSMapping.IllegalNullValueInColumn",column));
            }
        }
        catch (SQLException e)
        {
            throw new JDODataStoreException(LOCALISER.msg("RDBMSMapping.UnableToGetParam","int","" + param), e);
        }

        return value;
    }
    
    public void setObject(Object ps, int param, Object value)
    {
        try
        {
            if (value == null)
            {
                ((PreparedStatement) ps).setNull(param, getTypeInfo().dataType);
            }
            else
            {
                if( value instanceof Integer )
                {
                    ((PreparedStatement) ps).setShort(param, ((Integer)value).shortValue());
                }
                else if( value instanceof Short )
                {
                    ((PreparedStatement) ps).setShort(param, ((Short)value).shortValue());
                }
            }
        }
        catch (SQLException e)
        {
            throw new JDODataStoreException(LOCALISER.msg("RDBMSMapping.UnableToSetParam","Object","" + value), e);
        }
    }

    public Object getObject(Object rs, int param)
    {
        Object value;

        try
        {
            short d = ((ResultSet) rs).getShort(param);
            if( getJavaTypeMapping().getJavaType().getName().equals(Integer.class.getName()) )
            {
                value = ((ResultSet) rs).wasNull() ? null : new Integer(d);
            }
            else
            {
                value = ((ResultSet) rs).wasNull() ? null : new Short(d);
            }
        }
        catch (SQLException e)
        {
            throw new JDODataStoreException(LOCALISER.msg("RDBMSMapping.UnableToGetParam","Object","" + param), e);
        }

        return value;
    }    
}