/**********************************************************************
Copyright (c) Oct 2, 2004 Erik Bengtson and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
    ...
**********************************************************************/
package org.jpox.store.rdbms.mapping;

import java.io.Serializable;
import java.math.BigDecimal;
import java.math.BigInteger;

import org.jpox.store.OID;
import org.jpox.util.JavaUtils;

/**
 * @author erik
 * @version $Revision: 1.3 $
 */
public class SybaseRDBMSMappingManager extends AbstractRDBMSMappingManager
{
    public void initialise()
    {
        super.initialise();

        registerDatastoreMapping(int.class.getName(),IntegerRDBMSMapping.class,"INTEGER","INT");
        registerDatastoreMapping(Integer.class.getName(),IntegerRDBMSMapping.class,"INTEGER","INT");

        registerDatastoreMapping(long.class.getName(),IntegerRDBMSMapping.class,"INTEGER","INT");
        registerDatastoreMapping(Long.class.getName(),IntegerRDBMSMapping.class,"INTEGER","INT");

        registerDatastoreMapping(float.class.getName(),DoubleRDBMSMapping.class,"DOUBLE","DOUBLE");
        registerDatastoreMapping(Float.class.getName(),DoubleRDBMSMapping.class,"DOUBLE","DOUBLE");

        registerDatastoreMapping(double.class.getName(),DoubleRDBMSMapping.class,"DOUBLE","DOUBLE");
        registerDatastoreMapping(Double.class.getName(),DoubleRDBMSMapping.class,"DOUBLE","DOUBLE");

        registerDatastoreMapping(String.class.getName(),VarCharRDBMSMapping.class,"VARCHAR","VARCHAR", true);
        registerDatastoreMapping(String.class.getName(),CharRDBMSMapping.class,"CHAR","CHAR", false);
        registerDatastoreMapping(String.class.getName(),LongVarcharRDBMSMapping.class,"BLOB","BLOB", false);

        registerDatastoreMapping(char.class.getName(),IntegerRDBMSMapping.class,"INTEGER","INTEGER");
        registerDatastoreMapping(Character.class.getName(),IntegerRDBMSMapping.class,"INTEGER","INTEGER");

        registerDatastoreMapping(short.class.getName(),SmallIntRDBMSMapping.class,"SMALLINT","SMALLINT");
        registerDatastoreMapping(Short.class.getName(),SmallIntRDBMSMapping.class,"SMALLINT","SMALLINT");

        registerDatastoreMapping(boolean.class.getName(),BitRDBMSMapping.class,"BIT","BIT");
        registerDatastoreMapping(Boolean.class.getName(),BitRDBMSMapping.class,"BIT","BIT");

        registerDatastoreMapping(byte.class.getName(),TinyIntRDBMSMapping.class,"TINYINT","TINYINT");
        registerDatastoreMapping(Byte.class.getName(),TinyIntRDBMSMapping.class,"TINYINT","TINYINT");

        registerDatastoreMapping(Object.class.getName(),LongVarBinaryRDBMSMapping.class,"LONGVARBINARY","LONGVARBINARY");
        registerDatastoreMapping(Serializable.class.getName(),LongVarBinaryRDBMSMapping.class,"LONGVARBINARY","LONGVARBINARY");

        registerDatastoreMapping(java.util.Locale.class.getName(),CharRDBMSMapping.class,"CHAR","CHAR", true);
        registerDatastoreMapping(java.util.Locale.class.getName(),VarCharRDBMSMapping.class,"VARCHAR","VARCHAR", false);

        registerDatastoreMapping(java.util.TimeZone.class.getName(),CharRDBMSMapping.class,"CHAR","CHAR", true);
        registerDatastoreMapping(java.util.TimeZone.class.getName(),VarCharRDBMSMapping.class,"VARCHAR","VARCHAR", false);

        if (JavaUtils.isJRE1_4OrAbove())
        {
            registerDatastoreMapping(java.util.Currency.class.getName(),CharRDBMSMapping.class,"CHAR","CHAR", true);
            registerDatastoreMapping(java.util.Currency.class.getName(),VarCharRDBMSMapping.class,"VARCHAR","VARCHAR", false);
        }

        registerDatastoreMapping(java.util.Date.class.getName(),TimestampRDBMSMapping.class,"TIMESTAMP","TIMESTAMP");
        registerDatastoreMapping(java.sql.Date.class.getName(),TimestampRDBMSMapping.class,"TIMESTAMP","TIMESTAMP");
        registerDatastoreMapping(java.sql.Time.class.getName(),TimestampRDBMSMapping.class,"TIMESTAMP","TIMESTAMP");
        registerDatastoreMapping(java.sql.Timestamp.class.getName(),TimestampRDBMSMapping.class,"TIMESTAMP","TIMESTAMP");
        registerDatastoreMapping(java.lang.Object.class.getName(),LongVarBinaryRDBMSMapping.class,"LONGVARBINARY","LONGVARBINARY");
        registerDatastoreMapping(OID.class.getName(),IntegerRDBMSMapping.class,"INTEGER","INT");
        registerDatastoreMapping(BigDecimal.class.getName(),DecimalRDBMSMapping.class,"DECIMAL","DECIMAL");
        registerDatastoreMapping(BigInteger.class.getName(),NumericRDBMSMapping.class,"NUMERIC","NUMERIC");


        registerDatastoreMapping(byte[].class.getName(),LongVarBinaryRDBMSMapping.class,"LONGVARBINARY","LONGVARBINARY");
        registerDatastoreMapping(int[].class.getName(),LongVarBinaryRDBMSMapping.class,"LONGVARBINARY","LONGVARBINARY");
        registerDatastoreMapping(long[].class.getName(),LongVarBinaryRDBMSMapping.class,"LONGVARBINARY","LONGVARBINARY");
        registerDatastoreMapping(float[].class.getName(),LongVarBinaryRDBMSMapping.class,"LONGVARBINARY","LONGVARBINARY");
        registerDatastoreMapping(double[].class.getName(),LongVarBinaryRDBMSMapping.class,"LONGVARBINARY","LONGVARBINARY");
        registerDatastoreMapping(char[].class.getName(),LongVarBinaryRDBMSMapping.class,"LONGVARBINARY","LONGVARBINARY");
        registerDatastoreMapping(short[].class.getName(),LongVarBinaryRDBMSMapping.class,"LONGVARBINARY","LONGVARBINARY");
        registerDatastoreMapping(boolean[].class.getName(),LongVarBinaryRDBMSMapping.class,"LONGVARBINARY","LONGVARBINARY");
    }
}