/**********************************************************************
Copyright (c) 2004 Erik Bengtson and others. All rights reserved. 
This program and the accompanying materials are made available under 
the terms of the JPOX License v1.0 which accompanies this distribution. 

Contributors:
2004 Andy Jefferson - localised messages
    ...
**********************************************************************/
package org.jpox.store.rdbms.mapping;

import java.math.BigInteger;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;

import javax.jdo.JDODataStoreException;

import org.jpox.store.DatastoreField;
import org.jpox.store.StoreManager;
import org.jpox.store.exceptions.NullValueException;
import org.jpox.store.mapping.JavaTypeMapping;
import org.jpox.store.rdbms.Column;
import org.jpox.store.rdbms.typeinfo.TypeInfo;

/**
 * Mapping of a TINYINT RDBMS type.
 * 
 * @version $Revision: 1.3 $
 */
public class TinyIntRDBMSMapping extends ColumnMapping
{
    /**
     * @param storeMgr Store Manager
     * @param mapping Java type mapping
     */
    protected TinyIntRDBMSMapping(StoreManager storeMgr, JavaTypeMapping mapping)
    {
        super(storeMgr, mapping);
    }
    
    /**
     * Constructor.
     * @param mapping Java type mapping
     * @param storeMgr Store Manager
     * @param field Field to be mapped
     */
    public TinyIntRDBMSMapping(JavaTypeMapping mapping, StoreManager storeMgr, DatastoreField field)
    {
		super(storeMgr, mapping);
		column = (Column) field;
		initialize();
	}

    private void initialize()
    {
        if( column != null )
        {
            column.checkPrimitive();
        }
		initTypeInfo();
    }
    
    public TypeInfo getTypeInfo()
    {
        return getDatabaseAdapter().getTypeInfo(Types.SMALLINT);
    }
    
    public DatastoreField getDatastoreField()
    {
        return column;
    }
    
    public void setInt(Object ps, int param, int value)
    {
        try
        {
            ((PreparedStatement) ps).setInt(param, value);
        }
        catch (SQLException e)
        {
            throw new JDODataStoreException(LOCALISER.msg("RDBMSMapping.UnableToSetParam","int", "" + value), e);
        }
    }

    public int getInt(Object rs, int param)
    {
        int value;

        try
        {
            value = ((ResultSet) rs).getInt(param);

            if (((ResultSet) rs).wasNull())
            {
                throw new NullValueException(LOCALISER.msg("RDBMSMapping.IllegalNullValueInColumn",column));
            }
        }
        catch (SQLException e)
        {
            throw new JDODataStoreException(LOCALISER.msg("RDBMSMapping.UnableToGetParam","int", "" + param), e);
        }

        return value;
    }

    public void setByte(Object ps, int param, byte value)
    {
        try
        {
            //TODO bug with SQL SERVER DRIVER. It doesnt accept Byte -128
//          ps.setByte(param, value);
            ((PreparedStatement) ps).setInt(param, value);
            
        }
        catch (SQLException e)
        {
            throw new JDODataStoreException(LOCALISER.msg("RDBMSMapping.UnableToSetParam","byte", "" + value), e);
        }
    }

    public byte getByte(Object rs, int param)
    {
        byte value;

        try
        {
            value = ((ResultSet) rs).getByte(param);

            if (((ResultSet) rs).wasNull())
            {
                throw new NullValueException(LOCALISER.msg("RDBMSMapping.IllegalNullValueInColumn",column));
            }
        }
        catch (SQLException e)
        {
            throw new JDODataStoreException(LOCALISER.msg("RDBMSMapping.UnableToGetParam","byte", "" + param), e);
        }

        return value;
    }

    public void setObject(Object ps, int param, Object value)
    {
        try
        {
            if (value == null)
            {
                ((PreparedStatement) ps).setNull(param, getTypeInfo().dataType);
            }
            else
            {
                if (value instanceof Byte)
                {
                    ((PreparedStatement) ps).setInt(param, ((Byte)value).byteValue());
                }
                else if (value instanceof BigInteger)
                {
                    ((PreparedStatement) ps).setInt(param, ((BigInteger)value).byteValue());
                }
                //TODO bug with SQL SERVER DRIVER. It doesnt accept Byte -128
                //ps.setByte(param, ((Byte)value).byteValue());
            }
        }
        catch (SQLException e)
        {
            throw new JDODataStoreException(LOCALISER.msg("RDBMSMapping.UnableToSetParam","Byte", "" + value), e);
        }
    }

    public Object getObject(Object rs, int param)
    {
        Object value;

        try
        {
            byte b = ((ResultSet) rs).getByte(param);
            value = ((ResultSet) rs).wasNull() ? null : new Byte(b);
        }
        catch (SQLException e)
        {
            throw new JDODataStoreException(LOCALISER.msg("RDBMSMapping.UnableToGetParam","Byte", "" + param), e);
        }

        return value;
    }    
}