/**********************************************************************
Copyright (c) 2002 TJDO and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
    Erik Bengtson - refactored
    ...
**********************************************************************/
package org.jpox.store.rdbms.sqlidentifier;

import org.jpox.store.exceptions.TooManyForeignKeysException;
import org.jpox.store.rdbms.adapter.DatabaseAdapter;
import org.jpox.store.rdbms.table.Table;

/**
 * Identifier for a FK.
 * 
 * @version $Revision: 1.2 $
 */
public class ForeignKeyIdentifier extends SQLIdentifier
{
    public static final int MAX_FOREIGN_KEYS = 36;

    public ForeignKeyIdentifier(Table table, int seq)
    {
        super(table.getStoreManager().getDatastoreAdapter());

        this.javaName = null;

        char suffix;

        if (seq < 10)
        {
            suffix = (char)('0' + seq);
        }
        else if (seq < MAX_FOREIGN_KEYS)
        {
            suffix = (char)('A' + seq);
        }
        else
        {
            throw new TooManyForeignKeysException(table.toString());
        }

        String baseID = truncate(((SQLIdentifier) table.getName()).getSQLIdentifier(), getMaxLength() - 4);

        setSQLIdentifier(baseID + "_FK" + suffix);
    }

    protected int getMaxLength()
    {
        return ((DatabaseAdapter) dba).getMaxConstraintNameLength();
    }
}