/**********************************************************************
Copyright (c) 2002 Kelly Grizzle and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
2002 Mike Martin - unknown changes
2003 Andy Jefferson - added localiser
2003 Andy Jefferson - replaced TableMetadata with SQLIdentifier and java name
2004 Andy Jefferson - changed to extend TableImpl
    ...
**********************************************************************/
package org.jpox.store.rdbms.table;

import java.lang.reflect.Field;

import javax.jdo.JDOFatalInternalException;

import org.jpox.metadata.FieldMetaData;
import org.jpox.metadata.FieldPersistenceModifier;
import org.jpox.store.rdbms.RDBMSManager;
import org.jpox.store.rdbms.sqlidentifier.SQLIdentifier;

/**
 * Abstract class representing a field that maps to a table in the datastore.
 * This will be something like an SCO, such as Set, Map, List where a link table is used.
 * It could potentially be used where the user wants to map some field into its own SCO table.
 *
 * @version $Revision: 1.1 $
 **/
abstract class FieldTable extends TableImpl
{
    protected final FieldMetaData fmd;
    protected final Field field;
    protected final String ownerType;

    /**
     * Constructor.
     *
     * @param tableName The Table SQL identifier
     * @param fmd Field meta data for the owner field
     * @param storeMgr The Store Manager for the datastore.
     **/
    protected FieldTable(SQLIdentifier tableName,FieldMetaData fmd,RDBMSManager storeMgr)
    {
        super(tableName, storeMgr);

        this.fmd = fmd;
        this.field = fmd.getField();
        this.ownerType = fmd.getClassName(true);

        if (fmd.getPersistenceModifier() == FieldPersistenceModifier.NONE)
        {
            throw new JDOFatalInternalException(LOCALISER.msg("Table.InvalidPersistenceModifierForField",fmd.getName()));
        }
    }

    /**
     * Accessor for the owner field.
     * @return The field
     **/
    public Field getField()
    {
        return field;
    }
}