/**********************************************************************
Copyright (c) 2003 Erik Bengtson and others. All rights reserved. 
This program and the accompanying materials are made available under 
the terms of the JPOX License v1.0 which accompanies this distribution. 

Contributors:
2003 Andy Jefferson - coding standards
    ...
**********************************************************************/
package org.jpox.store.rdbms.typeinfo;

import java.sql.ResultSet;
import java.sql.Types;


/**
 * Represents the metadata of a HSQL data type.
 * 
 * @version $Revision: 1.1 $ 
 */
public class HSQLTypeInfo extends TypeInfo
{
    /** The maximum precision we allow to be reported. */
    public static final int MAX_PRECISION = Integer.MAX_VALUE;

    /**
     * Constructs a type information object from the current row of the given
     * result set.  The {@link ResultSet} object passed must have been obtained
     * from a call to DatabaseMetaData.getTypeInfo().
     *
     * <p>This method only retrieves the values from the current row; the caller
     * is required to advance to the next row with {@link ResultSet#next}.
     *
     * @param rs    The result set returned from DatabaseMetaData.getTypeInfo().
     */
    public HSQLTypeInfo(ResultSet rs)
    {
        super(rs);

        /*
         * Not really sure what we should do about the precision field.  
         * 
         */
/*
 * HSQL 1.7.0 reference
        NAME    RANGE   Java type
        INTEGER | INT   as Java type    "int" | "java.lang.Integer"
        DOUBLE [PRECISION] | FLOAT  as Java type    "double" | "java.lang.Double"
        VARCHAR     Integer.MAXVALUE    "java.lang.String"
        VARCHAR_IGNORECASE  Integer.MAXVALUE    "java.lang.String"
        CHAR | CHARACTER    Integer.MAXVALUE    "java.lang.String"
        LONGVARCHAR     Integer.MAXVALUE    "java.lang.String"
        DATE    as Java type    "java.sql.Date"
        TIME    as Java type    "java.sql.Time"
        TIMESTAMP | DATETIME    as Java type    "java.sql.Timestamp"
        DECIMAL     No limit    "java.math.BigDecimal"
        NUMERIC     No limit    "java.math.BigDecimal"
        BIT     as Java type    "boolean" | "java.lang.Boolean"
        TINYINT     as Java type    "byte" | "java.lang.Byte"
        SMALLINT    as Java type    "short" | "java.lang.Short"
        BIGINT  as Java type    "long" | "java.lang.Long"
        REAL    as Java type    "double" | "java.lang.Double"
        BINARY  Integer.MAXVALUE    "byte[]"
        VARBINARY   Integer.MAXVALUE    "byte[]"
        LONGVARBINARY   Integer.MAXVALUE    "byte[]"
        OTHER | OBJECT  Integer.MAXVALUE    "java.lang.Object"
*/      
        if (typeName.equalsIgnoreCase("varchar") ||
            typeName.equalsIgnoreCase("char"))
        {
            /*
             * VARCHAR gets an arbitrary maximum precision assigned to it.
             * Requests fo anything larger than this will be converted to
             * LONGVARCHAR (i.e. TEXT) in StringMapping.
             */
            precision = MAX_PRECISION;
        }
        else if (typeName.equalsIgnoreCase("numeric"))
        {
            precision = MAX_PRECISION;
        }
        else if (typeName.equalsIgnoreCase("text"))
        {
            /* TEXT is equated to Types.LONGVARCHAR (as it should be). */
            dataType = Types.LONGVARCHAR;
        }

        if (precision > MAX_PRECISION)
        {
            precision = MAX_PRECISION;
        }
    }
}
