/**********************************************************************
Copyright (c) 2002 Mike Martin (TJDO) and others. All rights reserved.
This program and the accompanying materials are made available under
the terms of the JPOX License v1.0 which accompanies this distribution. 

Contributors:
2003 Erik Bengtson - type handling
    ...
**********************************************************************/
package org.jpox.store.rdbms.typeinfo;

import java.sql.ResultSet;
import java.sql.Types;

import org.jpox.store.rdbms.columninfo.ColumnInfo;

/**
 * Represents the metadata of a MSSQL data type.
 * 
 * @version $Revision: 1.1 $
 */
public class MSSQLTypeInfo extends TypeInfo
{
    public static final int NVARCHAR = -9;
    public static final int NTEXT = -10;
    public static final int UNIQUEIDENTIFIER = -11;

    /**
     * Constructs a type information object from the current row of the given
     * result set.  The {@link ResultSet} object passed must have been obtained
     * from a call to DatabaseMetaData.getTypeInfo().
     *
     * <p>This method only retrieves the values from the current row; the caller
     * is required to advance to the next row with {@link ResultSet#next}.
     *
     * @param rs    The result set returned from DatabaseMetaData.getTypeInfo().
     */
    public MSSQLTypeInfo(ResultSet rs)
    {
        super(rs);
    }

    public boolean isCompatibleWith(ColumnInfo colInfo)
    {
        if (super.isCompatibleWith(colInfo))
            return true;

        switch (dataType)
        {
            case Types.VARCHAR:
                return colInfo.getDataType() == NVARCHAR;
            case Types.LONGVARCHAR:
                return colInfo.getDataType() == NTEXT;
            case Types.VARBINARY:
            case Types.LONGVARBINARY:
            case UNIQUEIDENTIFIER:
                return (colInfo.getDataType() == Types.VARBINARY) || (colInfo.getDataType() == Types.LONGVARBINARY) || (colInfo.getDataType() == UNIQUEIDENTIFIER);
            default:
                return false;
        }
    }
}