/**********************************************************************
Copyright (c) 2004 Erik Bengtson and others. All rights reserved. 
This program and the accompanying materials are made available under 
the terms of the JPOX License v1.0 which accompanies this distribution. 

Contributors:
    ...
**********************************************************************/
package org.jpox.store.rdbms.typeinfo;

import java.sql.ResultSet;

/**
 * Represents the metadata of a McKoi data type.
 * 
 * @version $Revision: 1.1 $ 
 */
public class McKoiTypeInfo extends TypeInfo
{
    /** The maximum precision we allow to be reported. McKoit supports up to 1 billion characters */
    public static final int MAX_PRECISION = Integer.MAX_VALUE;

    /**
     * Constructs a type information object from the current row of the given
     * result set.  The {@link ResultSet} object passed must have been obtained
     * from a call to DatabaseMetaData.getTypeInfo().
     *
     * <p>This method only retrieves the values from the current row; the caller
     * is required to advance to the next row with {@link ResultSet#next}.
     *
     * @param rs    The result set returned from DatabaseMetaData.getTypeInfo().
     */
    public McKoiTypeInfo(ResultSet rs)
    {
        super(rs);
        if (typeName.equalsIgnoreCase("varchar") ||
            typeName.equalsIgnoreCase("char"))
        {
            /*
             * VARCHAR gets an arbitrary maximum precision assigned to it.
             * Requests fo anything larger than this will be converted to
             * LONGVARCHAR (i.e. TEXT) in StringMapping.
             */
            precision = MAX_PRECISION;
        }
        if (precision > MAX_PRECISION)
        {
            precision = MAX_PRECISION;
        }
    }
}
