/**********************************************************************
Copyright (c) 2004 Erik Bengtson and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
2004 Andy Jefferson - added getNewSingleFieldIdentity reflection method
    ...
**********************************************************************/
package org.jpox.util;

import java.lang.reflect.Constructor;

import javax.jdo.ByteIdentity;
import javax.jdo.CharIdentity;
import javax.jdo.IntIdentity;
import javax.jdo.LongIdentity;
import javax.jdo.ShortIdentity;
import javax.jdo.SingleFieldIdentity;
import javax.jdo.StringIdentity;
import javax.jdo.spi.PersistenceCapable;

import org.jpox.metadata.ClassMetaData;

/**
 * Application Identity 'id' utils.
 *
 * @version $Revision: 1.2 $ 
 */
public class AIDUtils
{
    /**
     * Constructor
     */
    private AIDUtils()
    {
    }
 
    /**
     * Create a new ObjectId instance and copy the key fields from the
     * persistence capable instance to the objectId instance
     * @param pc The persistence capable object
     * @return The object id for the persistence capable object
     */
    public static Object getNewObjectId(PersistenceCapable pc, ClassMetaData cmd)
    {
        Object userOid = pc.jdoNewObjectIdInstance();

        if (!isSingleFieldIdentityClass(cmd.getObjectidClass()))
        {
            pc.jdoCopyKeyFieldsToObjectId(userOid);
        }
        return userOid;
    }

    /**
     * Utility to create a new SingleFieldIdentity using reflection when you know the
     * type of the PersistenceCapable, and also which SingleFieldIdentity, and the value
     * of the key.
     * @param idType Type of SingleFieldIdentity
     * @param pcType Type of the PersistenceCapable
     * @param value The value for the identity (the Long, or Int, or ... etc).
     * @return The SingleFieldIdentity key
     */
    public static SingleFieldIdentity getNewSingleFieldIdentity(Class idType, Class pcType, Object value)
    {
        if (!SingleFieldIdentity.class.isAssignableFrom(idType))
        {
            // TODO Log this illegal access
            return null;
        }

        SingleFieldIdentity id = null;
        Class keyType = null;
        if (idType == LongIdentity.class)
        {
            keyType = Long.class;
        }
        else if (idType == IntIdentity.class)
        {
            keyType = Integer.class;
        }
        else if (idType == StringIdentity.class)
        {
            keyType = String.class;
        }
        else if (idType == ByteIdentity.class)
        {
            keyType = Byte.class;
        }
        else if (idType == ShortIdentity.class)
        {
            keyType = Short.class;
        }
        else if (idType == CharIdentity.class)
        {
            keyType = Character.class;
        }

        try
        {
            Class[] ctrArgs = new Class[] {Class.class, keyType};
            Constructor ctr = idType.getConstructor(ctrArgs);

            Object[] args = new Object[] {pcType, value};
            id = (SingleFieldIdentity)ctr.newInstance(args);
        }
        catch (Exception e)
        {
            JPOXLogger.JDO.error("Error encountered while creating SingleFieldIdentity instance of type \"" + idType.getName() + "\"");
            JPOXLogger.JDO.error(e);

            return null;
        }

        return id;
    }

    /**
     * Checks whether the argument is equals to one the single field identity class names
     * @param className the full class name
     * @return true if the argument is equals to one the single field identity class names 
     */
    public static boolean isSingleFieldIdentityClass(String className)
    {
        if (className == null)
        {
            return false;
        }
        if (className.length() < 1)
        {
            return false;
        }

        return (className.equals(StringIdentity.class.getName()) || 
                className.equals(ByteIdentity.class.getName()) || 
                className.equals(CharIdentity.class.getName()) ||
                className.equals(IntIdentity.class.getName()) || 
                className.equals(ShortIdentity.class.getName()) || 
                className.equals(LongIdentity.class.getName()));
    }
}