/**********************************************************************
Copyright (c) 2003 Erik Bengtson and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
    ...
**********************************************************************/

package org.jpox.util;

import java.util.Locale;

/**
 * Utility class for internationalization. This class provides a 
 * central location to do specialized formatting in both 
 * a default and a locale specfic manner.
 *
 * @version $Revision: 1.1 $ 
 */
public final class I18nUtils
{
    private I18nUtils()
    {
        // protects from instantiation 
    }
    
    /**
     * Convert a string based locale into a Locale Object
     * 
     * language_country_variant
     * 
     * Examples: "en", "de_DE", "_GB", "en_US_WIN", "de__POSIX", "fr_MAC"
     *  
     * @param localeString
     * @return the Locale
     */
    public static Locale getLocaleFromString(String localeString)
    {
        if (localeString == null)
        {
            return null;
        }
        localeString = localeString.trim();
        if (localeString.toLowerCase().equals("default"))
        {
            return Locale.getDefault();
        }
        int languageIndex = localeString.indexOf('_');
        if (languageIndex == -1)
        {
            return new Locale(localeString, "");
        }
        int countryIndex = localeString.indexOf('_', languageIndex + 1);
        String country = null;
        if (countryIndex == -1)
        {
            if (localeString.length() > languageIndex)
            {
                country = localeString.substring(languageIndex + 1, localeString.length());
            }
            else
            {
                return null;
            }
        }
        int variantIndex = -1;
        if (countryIndex != -1)
        {
            countryIndex = localeString.indexOf('_', countryIndex + 1);
        }
        String language = localeString.substring(0, languageIndex);
        String variant = null;
        if (variantIndex != -1)
        {
            variant = localeString.substring(variantIndex + 1, localeString.length());
        }
        if (variant != null)
        {
            return new Locale(language, country, variant);
        }
        else
        {
            return new Locale(language, country);
        }

    }
}
