/**********************************************************************
Copyright (c) 2005 Andy Jefferson and others. All rights reserved.
This program and the accompanying materials are made available under 
the terms of the JPOX License v1.0 which accompanies this distribution.

Contributors:
    ...
**********************************************************************/
package org.jpox.util;

import java.util.StringTokenizer;

/**
 * Utilities relating to the version of Java in use at runtime.
 * @version $Revision: 1.2 $
 */
public class JavaUtils
{
    private static boolean versionInitialised=false;
    private static int majorVersion=1;
    private static int minorVersion=0;

    /**
     * Accessor for whether the JRE is 1.4 (or above).
     * Checks for the presence of a known 1.4 class.
     * @return Whether the JRE is 1.4 or above
     */
    public static boolean isJRE1_4OrAbove()
    {
        try
        {
            Class.forName("java.util.Currency");
            return true;
        }
        catch (Exception e)
        {
            return false;
        }
    }

    /**
     * Accessor for the major version number of the JRE.
     * @return The major version number of the JRE
     */
    public static int getJREMajorVersion()
    {
        if (!versionInitialised)
        {
            initialiseJREVersion();
        }
        return majorVersion;
    }

    /**
     * Accessor for the minor version number of the JRE.
     * @return The minor version number of the JRE
     */
    public static int getJREMinorVersion()
    {
        if (!versionInitialised)
        {
            initialiseJREVersion();
        }
        return minorVersion;
    }

    /**
     * Utility to initialise the values of the JRE major/minor version.
     * Assumes the "java.version" string is in the form "XX.YY.ZZ".
     * Works for SUN JRE's.
     */
    private static void initialiseJREVersion()
    {
        String version = System.getProperty("java.version");
        
        // Assume that the version string is of the form XX.YY.ZZ (works for SUN JREs)
        StringTokenizer tokeniser = new StringTokenizer(version, ".");
        String token = tokeniser.nextToken();
        try
        {
            Integer ver = new Integer(token);
            majorVersion = ver.intValue();
            
            token = tokeniser.nextToken();
            ver = new Integer(token);
            minorVersion = ver.intValue();
        }
        catch (Exception e)
        {
            // Do nothing
        }
        versionInitialised = true;
    }
}