/**********************************************************************
Copyright (c) 2004 Andy Jefferson and others.
All rights reserved. This program and the accompanying materials
are made available under the terms of the JPOX License v1.0
which accompanies this distribution. 

Contributors:
    ...
**********************************************************************/
package org.jpox.util;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.jdo.JDOFatalUserException;

/**
 * Utilities for handling Views.
 *
 * @version $Revision: 1.1 $   
 **/
public class ViewUtils
{
    protected static final Localiser LOCALISER=Localiser.getInstance("org.jpox.util.Localisation");

    /**
     * Check for any circular view references between referencer and referencee.
     * If one is found, throw a JDOFatalUserException with the chain of
     * references.
     * @param viewReferences The Map of view references to check.
     * @param referencer_name Name of the class that has the reference.
     * @param referencee_name Name of the class that is being referenced.
     * @param referenceChain The List of class names that have been referenced
     * @throws JDOFatalUserException If a circular reference is found in
     *                               the view definitions.
     */
    public static void checkForCircularViewReferences(Map viewReferences,
                                                      String referencer_name,
                                                      String referencee_name,
                                                      List referenceChain)
    throws JDOFatalUserException
    {
        Set class_names = (Set)viewReferences.get(referencee_name);
        if (class_names != null)
        {
            // Initialize the chain of references if needed.  Add the referencee
            // to the chain.
            if (referenceChain == null)
            {
                referenceChain = new ArrayList();
                referenceChain.add(referencer_name);
            }
            referenceChain.add(referencee_name);

            // Iterate through all referenced classes from the referencee.  If
            // any reference the referencer, throw an exception.
            for (Iterator it=class_names.iterator(); it.hasNext(); )
            {
                String current_name=(String)it.next();
                if (current_name.equals(referencer_name))
                {
                    StringBuffer error=new StringBuffer(LOCALISER.msg("View.CircularDependencyError"));

                    for (Iterator chainIter=referenceChain.iterator(); chainIter.hasNext(); )
                    {
                        error.append(chainIter.next());
                        if (chainIter.hasNext())
                        {
                            error.append(" -> ");
                        }
                    }

                    throw new JDOFatalUserException(error.toString());
                }
                else
                {
                    // Make recursive call to check for any nested dependencies.
                    // e.g A references B, B references C, C references A.
                    checkForCircularViewReferences(viewReferences,
                                                   referencer_name,
                                                   current_name,
                                                   referenceChain);
                }
            }
        }
    }
}
