/**********************************************************************
Copyright (c) 2002 Mike Martin (TJDO) and others. All rights reserved.
This program and the accompanying materials are made available under
the terms of the JPOX License v1.0 which accompanies this distribution. 

Contributors:
2002 Kelly Grizzle (TJDO)
2003 Andy Jefferson - coding standards
    ...
**********************************************************************/
package org.jpox.util;

import java.lang.ref.ReferenceQueue;
import java.lang.ref.WeakReference;
import java.util.Map;

/**
 * A <code>java.util.Map</code> implementation using weak reference values.
 *
 * <p>The values are stored in the map as weak references.  If the garbage
 * collector clears the reference, the corresponding key is automatically
 * removed from the map.
 *
 * @see WeakReference
 * @version $Revision: 1.1 $
 */
public class WeakValueMap extends ReferenceValueMap
{
    /**
     * Default Constructor
     **/
    public WeakValueMap()
    {
        super();
    }

    /**
     * Constructor taking the initial capacity.
     * @param initialCapacity The Initial capacity of the collection
     **/
    public WeakValueMap(int initialCapacity)
    {
        super(initialCapacity);
    }

    /**
     * Constructor taking the initial capacity and load factor.
     * @param initialCapacity The Initial capacity of the collection
     * @param loadFactor The Load Factor of the collection
     **/
    public WeakValueMap(int initialCapacity, float loadFactor)
    {
        super(initialCapacity, loadFactor);
    }

    /**
     * Constructor taking a Map for definition.
     * @param m The Map 
     **/
    public WeakValueMap(Map m)
    {
        super(m);
    }

    /**
     * Inner class to represent a weak reference - one that can be garbage
     * collected.
     **/
    private static class WeakValueReference extends WeakReference implements ReferenceValueMap.ValueReference
    {
        private final Object key;

        WeakValueReference(Object key, Object value, ReferenceQueue q)
        {
            super(value, q);
            this.key = key;
        }

        public Object getKey()
        {
            return key;
        }
    }

    protected ReferenceValueMap.ValueReference newValueReference(Object key, Object value, ReferenceQueue queue)
    {
        return new WeakValueReference(key, value, queue);
    }
}
